/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include <stdio.h>
#include <sys/stat.h>
#include <string.h>

#include "applic.h"
#define MAX_WORKBOOK 20

static Wkb *wkb[MAX_WORKBOOK];
int nbdoc = 0;
int totnbdoc = 0;

Wkb *
applic_gwkb (int i)
{
  if (i < 0 || i >= MAX_WORKBOOK)
    {
      fprintf (stderr, "app_gwork: Maximum number of wkb is %d\n", MAX_WORKBOOK);
      i = 0;
    }

  if (i < nbdoc)
    return wkb[i];
  else
    fprintf (stderr, "app_gwork: Asking for non existing wkb %d\n", i);
  return wkb[0];
}

Wkb *
applic_addwkb (filename)
     char *filename;
{
  newdoc (filename);
  return wkb[nbdoc - 1];

}

int
newdoc (filename)
     char *filename;
{
  char name[16];

  if (filename == NULL)
    {
      sprintf (name, "Document%d", totnbdoc + 1);
      filename = name;
    }

  if (nbdoc < MAX_WORKBOOK)
    {
      wkb[nbdoc] = newwkb (filename);
      activate_doc (nbdoc);
      nbdoc++;
      totnbdoc++;
      add_window_name (filename);
    }
  else
    fprintf (stderr, "newdoc: Maximum number of wkb is %d\n", MAX_WORKBOOK);
}

int
close_doc ()
{
  int i = 0;
  int j;
  Wkb *tofree;

  if (nbdoc == 1)
    return;

  while (i < nbdoc && ActivWkb != wkb[i])
    i++;
  if (i == nbdoc)
    return;

  remove_window_name (ActivWkb->name);
  tofree = ActivWkb;

  for (j = i; j < nbdoc; j++)
    wkb[j] = wkb[j + 1];
  wkb[nbdoc] = NULL;
  nbdoc--;

  activate_doc (nbdoc - 1);

  freewkb (tofree);

}

int
activate_doc (int n)
{

  ActivateWkb (wkb[n]);
  rstsel (0, 0, 0);
  xrepnt ();

}

int
applic_changesht (sht)
     int sht;
{
  if (ActivWkb->activwks == ActivWkb->Wkss[sht])
    return -1;

  ActivWkb->activwks = ActivWkb->Wkss[sht];
  ActivateWks (ActivWkb->Wkss[sht]);
  return 0;
}

int
Actwkb2 (filename)
     char *filename;
{
  int i = 0;
  while (i < nbdoc && strcmp (filename, wkb[i]->name))
    i++;
  if (i < nbdoc)
    ActivateWkb (wkb[i]);
  xrepnt ();

}

void
openfl (filename)
     char *filename;
{
  FILE *fp = fopen (filename, "r");

  char line[258];
  char *end;
  Wkb *wb = applic_addwkb (filename);
  ActivateWkb (wb);

  while (fgets (line, 256, fp))
    {
      addlinecode (wb->macrofile, line);

    }

  runcode (wb->macrofile);
  ActivateWkb (wb);

}

void
savefl ()
{
  wkb_s (ActivWkb, ActivWkb->name);
}

void
saveasfile (filename)
     char *filename;
{
  remove_window_name (ActivWkb->name);
  add_window_name (filename);

  wkb_s (ActivWkb, filename);
}

init_applic ()
{
  newdoc ("Document1");
}

Cl *
appliccl (i, j, new)
     int i, j, new;
{

  if (i < 0 || i > NBLIN)
    i = 1;
  if (j < 0 || j > NBCOL)
    j = 1;
  return wks_gcl (ActivWks, i, j, new);
}

char *
get_text (i, j)
     int i, j;
{
  return cl_gtext (appliccl (i, j, -1));
  return NULL;
}

double
get_value (i, j)
     int i, j;
{
  Cl *cl = appliccl (i, j, -1);
  return ((cl != NULL) ? cl_gvalue (cl) : 0.0);
}

set_value (i, j, x)
     int i, j;
     double x;
{
  Cl *c;

  c = appliccl (i, j, 1);
  cl_setvalue (c, x);

  return 1;
}

set_form (i, j, buf)
     int i, j;
     char *buf;
{
  Cl *c;
  c = (appliccl (i, j, 1));

  cl_setform (c, buf);

};

set_text (i, j, buf)
     int i, j;
     char *buf;
{
  Cl *c;
  c = (appliccl (i, j, 1));

  cl_settext (c, buf);

};

char *
get_form (i, j)
     int i, j;
{
  return cl_gform (appliccl (i, j, 0));
};

int
set_fg (i, j, f)
     int i, j, f;
{
  return cl_setfg (appliccl (i, j, 1), f);
}

int
get_fg (i, j)
     int i, j;
{
  return cl_gfg (appliccl (i, j, 0));
}

int
set_bg (i, j, f)
     int i, j, f;
{
  return cl_setbg (appliccl (i, j, 1), f);
}

int
get_bg (i, j)
     int i, j;
{
  return cl_gbg (appliccl (i, j, 0));
}

int
set_format (i, j, f, l)
     int i, j, f, l;
{
  return cl_setformat (appliccl (i, j, 1), f, l);
}

int
get_format (i, j, f, l)
     int i, j;
     int *f, *l;
{
  return cl_gformat (appliccl (i, j, 0), f, l);
}

int
set_border (i, j, f)
     int i, j, f;
{
  return cl_setborder (appliccl (i, j, 1), f);
}

int
get_border (i, j)
     int i, j;
{
  return cl_gborder (appliccl (i, j, 0));
}

int
set_just (i, j, f)
     int i, j, f;
{
  return cl_setjust (appliccl (i, j, 1), f);
}

int
get_just (i, j)
     int i, j;
{
  return cl_gjust (appliccl (i, j, 0));
}

int
set_font (i, j, f, g, h)
     int i, j, f, g, h;
{
  return cl_setfont (appliccl (i, j, 1), f, g, h);
}

int
get_font (i, j)
     int i, j;
{
  return cl_gfont (appliccl (i, j, 0));
}

int
get_fonts (i, j)
     int i, j;
{
  return cl_gfonts (appliccl (i, j, 0));
}

int
get_fontw (i, j)
     int i, j;
{
  return cl_gfontw (appliccl (i, j, 0));
}

static int copyi = 0;
static int copyj = 0;
static int fromi = 1;
static int fromj = 1;
static Cl ***copyapplic;

copy_applic (i1, j1, i2, j2)
     int i1, j1, i2, j2;
{
  int i, j;
  char *buf;
  char *buf1;
  Cl *cl1;
  Cl *cl2;

  fromi = i1;
  fromj = j1;

  for (i = 0; i < copyi; i++)
    for (j = 0; j < copyj; j++)
      {
	freecl (copyapplic[i][j]);
      }
  for (i = 0; i < copyi; i++)
    absfree (copyapplic[i], "copy_applic:copyapplic[i] ");
  if (copyapplic != NULL)
    absfree (copyapplic, "copy_applic:copyapplic ");

  copyi = i2 - i1 + 1;
  copyj = j2 - j1 + 1;

  copyapplic = (Cl ***) absmalloc (sizeof (Cl **) * copyi, "copy_applic:copyapplic ");
  for (i = 0; i < copyi; i++)
    copyapplic[i] = (Cl **) absmalloc (sizeof (Cl *) * copyj, "copy_applic:copyapplic[i] ");

  for (i = 0; i < copyi; i++)
    for (j = 0; j < copyj; j++)
      {
	cl2 = appliccl (i1 + i, j1 + j, -1);
	if (cl2 != NULL)
	  {
	    copyapplic[i][j] = newcl (i, j);
	    copyapplic[i][j]->wks = NULL;
	    cl_cpy (copyapplic[i][j], cl2);
	  }
	else
	  copyapplic[i][j] = NULL;

      }
}

paste_applic (i1, j1, i2, j2)
     int i1, j1, i2, j2;
{
  int i, j;
  int _curi, _curj, _di, _dj;
  char *buf;
  Cl *cl1;
  Cl *cl2;

  for (i = i1; i <= i2; i++)
    for (j = j1; j <= j2; j++)
      {
	set_form (i, j, "");
      }

  for (i = 0; i < copyi && i <= i2 - i1; i++)
    for (j = 0; j < copyj && j <= j2 - j1; j++)
      {
	cl2 = copyapplic[i][j];
	if (cl2 != NULL)
	  {
	    cl1 = appliccl (i1 + i, j1 + j, 1);
	    _curi = i1 + i;
	    _curj = j1 + j;
	    _di = (i1 - fromi);
	    _dj = (j1 - fromj);
	    copymod2 (cl1, cl2, _curi, _curj, _di, _dj);

	    if (cl2->style != NULL)
	      {
		if (cl1->style == NULL)
		  cl1->style = newstyle (STY_CELL);
		style_cpy (cl1->style, cl2->style);
	      }
	    else
	      {
		if (cl1->style != NULL)
		  {
		    freestyle (cl1->style);
		    cl1->style = NULL;
		  }
	      }
	  }

      }

}

cut_applic (i1, j1, i2, j2)
     int i1, j1, i2, j2;
{
  Cl *cl;
  int i, j;
  copy_applic (i1, j1, i2, j2);

  for (i = i1; i <= i2; i++)
    for (j = j1; j <= j2; j++)
      {
	cl = appliccl (i, j, -1);
	if (cl != NULL)
	  {
	    cl_setform (cl, "");
	    cl_setvalue (cl, 0);
	  }
      }

}

Chrt *
newgra ()
{
  Chrt *activgra = wks_nchrt (ActivWks);
  ActivateChrt (activgra);
  return activgra;
}

Chrt *
getlastgra ()
{
  return ActivChrt;
}

int
getngr ()
{
  return ActivWks->nbchrt;
}

setactivgra (n)
     int n;
{
  ActivateChrt (ActivWks->Chrts[n]);
}

Chrt *
getgr (n)
{
  return ActivWks->Chrts[n];
}

int
getactivgra ()
{
  int i;
  for (i = 0; i < getngr (); i++)
    if (ActivChrt == getgr (i))
      return i;
  return 0;
}

void
getrange (serie, buf, name)
     int serie;
     char *buf;
     char *name;
{
  Chrt *gr;
  char i1j1[20];
  char i2j2[20];
  int j;
  gr = ActivChrt;
  j = serie - 1;
  if (serie > gr->numrange)
    {
      sprintf (buf, "");
      sprintf (name, "");
    }
  else
    {
      n2a (i1j1, (gr->range[j]).i1, (gr->range[j]).j1);
      n2a (i2j2, (gr->range[j]).i2, (gr->range[j]).j2);
      sprintf (buf, "%s:%s", i1j1, i2j2);
      strcpy (name, gr->range[j].name);
    }
  return;
}

void
setrange (serie, buf, name)
     int serie;
     char *buf;
     char *name;
{
  Chrt *gr;
  char *i1j1;
  char *i2j2;
  char *separ;

  int j;

  separ = strchr (buf, ':');
  if (separ != NULL)
    {
      i1j1 = buf;
      i2j2 = separ + 1;
      separ[0] = '\0';
    }

  gr = ActivChrt;
  j = serie - 1;
  if (serie > gr->numrange)
    gr->numrange = serie;

  if (separ != NULL)
    alphatonum (i1j1, &(gr->range[j]).i1, &(gr->range[j]).j1);
  if (separ != NULL)
    alphatonum (i2j2, &(gr->range[j]).i2, &(gr->range[j]).j2);
  if (name != NULL)
    if (strlen (name) > 0)
      strcpy (gr->range[j].name, name);

  return;
}

void
settype (type)
     int type;
{
  ActivChrt->type = type;
}

int
gettype ()
{
  return ActivChrt->type;
}

int
setgridh (on)
     Boolean on;
{
  chrt_setgrid (ActivChrt, 2, on);
}

int
setgridv (on)
     Boolean on;
{
  chrt_setgrid (ActivChrt, 1, on);
}

void
settitre1 (titre)
     char *titre;
{
  if (strlen (titre) < 1)
    {
      ActivChrt->HasTitle = False;
      title_free (&ActivChrt->ChtTitle);
    }
  else
    {
      ActivChrt->HasTitle = True;
      title_settext (&ActivChrt->ChtTitle, titre);
    }
}

char *
gettitre1 ()
{
  return ActivChrt->ChtTitle.Text;
}

void
setXaxis (titre)
     char *titre;
{
  if (strlen (titre) < 1)
    {
      ActivChrt->Axes[xlCategory].HasTitle = False;
      title_free (&ActivChrt->Axes[xlCategory].AxisTitle);
    }
  else
    {
      ActivChrt->Axes[xlCategory].HasTitle = True;
      title_settext (&ActivChrt->Axes[xlCategory].AxisTitle, titre);
    }
}

char *
getXaxis ()
{
  return ActivChrt->Axes[xlCategory].AxisTitle.Text;
}

void
setYaxis (titre)
     char *titre;
{
  if (strlen (titre) < 1)
    {
      ActivChrt->Axes[xlValue].HasTitle = False;
      title_free (&ActivChrt->Axes[xlValue].AxisTitle);
    }
  else
    {
      ActivChrt->Axes[xlValue].HasTitle = True;
      title_settext (&ActivChrt->Axes[xlValue].AxisTitle, titre);
    }
}

char *
getYaxis ()
{
  return ActivChrt->Axes[xlValue].AxisTitle.Text;
}

getndr ()
{
  return ActivWks->nbdrwing;
}

Drwing *
getdr (n)
{
  return ActivWks->Drwings[n];
}

Drwing *
newdra ()
{
  Drwing *activdra = wks_ndrwing (ActivWks);
  ActivateDrwing (activdra);
  return activdra;
}

Drwing *
gldra ()
{
  return ActivDrwing;
}

setactivdra (n)
     int n;
{
  ActivateDrwing (ActivWks->Drwings[n]);
}

int
getactivdra ()
{
  int i;
  for (i = 0; i < getndr (); i++)
    if (ActivDrwing == getdr (i))
      return i;
  return 0;
}

cpygr ()
{
  if (ActivChrt == NULL)
    return;

};

cutgra ()
{
  int i, pos;

  if (ActivChrt == NULL)
    return;

  if (cpbdchrt != NULL)
    freechrt (cpbdchrt);
  cpbdchrt = ActivChrt;

  pos = getactivgra ();
  for (i = pos; i < getngr () - 1; i++)
    ActivWks->Chrts[i] = ActivWks->Chrts[i + 1];
  ActivWks->nbchrt--;

  ActivChrt = NULL;

};

pastegra ()
{
  if (cpbdchrt == NULL)
    return;

};

cpydr ()
{
  if (ActivDrwing == NULL)
    return;

  if (cpbddrwing != NULL)
    freedrwing (cpbddrwing);
  cpbddrwing = newdrwing1 (ActivDrwing);

};

cutdra ()
{
  int i, pos;

  if (ActivDrwing == NULL)
    return;

  if (cpbddrwing != NULL)
    freedrwing (cpbddrwing);
  cpbddrwing = ActivDrwing;

  pos = getactivdra ();
  for (i = pos; i < getndr () - 1; i++)
    ActivWks->Drwings[i] = ActivWks->Drwings[i + 1];
  ActivWks->nbdrwing--;

  ActivDrwing = NULL;

};

pastedra ()
{

  if (cpbddrwing == NULL)
    return;
  drwing_copy (wks_ndrwing (ActivWks), cpbddrwing);

};

update ()
{
};
