/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include "param.h"
#include <stdio.h>
#include "cblist.h"
#include "pixmap.h"
#include "myscroll.h"
#include "mainwin.h"
#include "wks.h"

static String fallback_resources[] =
{
  "*window.width:         800",
  "*window.height:        600",
  "abs*background:         #C000C000C000",
  "abs*foreground:         black",
  "*keyboardFocusPolicy:           explicit",
  NULL
};

typedef struct
  {
    Pixel foreground;
    Pixel background;
    XFontStruct *font;
    int ncol;
    Boolean showsize;
  }
res_data, *res_data_ptr;
static res_data res;

static XtResource applic_resources[] =
{
  {XtNforeground, XtCForeground, XtRPixel, sizeof (Pixel),
   XtOffset (res_data_ptr, foreground), XtRString, "Black"},
  {XtNbackground, XtCBackground, XtRPixel, sizeof (Pixel),
   XtOffset (res_data_ptr, background), XtRString, "White"},
};

static XrmOptionDescRec options[] =
{
  {"-ncol", "*ncol", XrmoptionSepArg, NULL},
  {"-fg", "*foreground", XrmoptionSepArg, (XtPointer) NULL},
  {"-bg", "*background", XrmoptionSepArg, (XtPointer) NULL},

};

static XtActionsRec actionsTable[] =
{
  {"DocBtn1Up", cbdb1up},
  {"PopDown", cb_popdown},
  {"DocMotion", cbdmv},
  {"DocBtn1Motion", cb_DocBtn1Motion},
  {"DocBtn1Down", cbdb1down},
  {"DocBtn2Up", cbdb2up},
  {"DocBtn2Motion", cb_DocBtn2Motion},
  {"DocBtn2Down", cbdn2down},
  {"TextReturn", cb_TextReturn},
  {"TextUp", cbtup},
  {"TextDown", cbtdown},
  {"TextLeft", cbtleft},
  {"TextRight", cbtright},
  {"DocBtn3Up", cb_DocBtn3Up},
  {"Delete", cbdel},
  {"DocDelete", cb_DocDelete},
  {"BackSpace", cbbak},
  {"PageUp", cbpgup},
  {"PageDown", cbpgdwn},
  {"PageLeft", cbpgleft},
  {"PageRight", cbpgrig},
  {"PageHome", cb_PageHome},
};

static char defaultTranslations[] = "\
<Btn1Up>:     DocBtn1Up()\n\
<Btn1Motion>: DocBtn1Motion()\n\
<Btn2Up>:     DocBtn2Up()\n\
<Btn2Motion>: DocBtn2Motion()\n\
<Motion>:    DocMotion()\n\
<Btn1Down>:  PopDown() DocBtn1Down()\n\
<Btn2Down>:  PopDown() DocBtn2Down()\n\
<Btn3Down>:  PopDown() \n\
<Key>Delete: DocDelete() \n\
<Key>Page_Up: PageUp() \n\
<Key>Page_Down: PageDown() \n\
<Key>Tab: PageRight() \n\
Ctrl<Key>Tab: PageRight() \n\
<Key>Home: PageHome() \n\
";

static char textTranslations[] = "\
<Key>Return:   TextReturn(%d)\n\
<Key>KP_Enter:   TextReturn(%d)\n\
<Key>osfUp:    TextUp() \n\
<Key>osfDown:   TextDown() \n\
<Key>osfLeft:   TextLeft() \n\
<Key>osfRight:   TextRight() \n\
<Key>Up:    TextUp() \n\
<Key>Down:  TextDown() \n\
<Key>Left:  TextLeft() \n\
<Key>Right:  TextRight() \n\
<Key>BackSpace: BackSpace() \n\
<Key>Delete: Delete() \n\
";

static char alltextTranslations[] = "\
<Key>BackSpace: BackSpace() \n\
<Key>Delete: Delete() \n\
";

static char popupTranslations[] = "\
<Motion>:	unhighlight() highlight()\n\
<Btn1Up>:           PopDown() notify() unhighlight()\n\
";

Mainwin mainwin;

Widget text;
Myscroll *scroll1, *scroll2;

Widget
gettop ()
{
  return mainwin.toplevel;
};

static Display *dpy;
static Display *display;
static int screen;
static Visual *vis;
static Window win;
XtAppContext app_con;

void
resize (widget, pointer, event, ctd)
     Widget widget;
     XtPointer pointer;
     XEvent *event;
     Boolean *ctd;
{
  int n;
  Dimension h, w;
  Widget wi;
  Arg args[2];
  wi = (Widget) pointer;

  n = 0;
  XtSetArg (args[n], XtNheight, &h);
  n++;
  XtSetArg (args[n], XtNwidth, &w);
  n++;
  XtGetValues (wi, args, n);
  cb_resize (w, h);
}

void
scrollv (v)
     int v;
{
  double V = v;
  lindrag (V);
  return;

  if (ActivWks->mrow > 400)
    lindrag (V / ActivWks->mrow);
  else
    lindrag (V / 400);

}

void
scrollh (v)
     int v;
{
  double V = v;
  coldrag (V);
  return;
  coldrag (V / NBCOL);
}

void
setscrollv (v)
     double v;
{

  setscrollval (scroll1, v);
}

void
setscrollh (v)
     double v;
{

  setscrollval (scroll2, v);
}

setscrollvlimit (vmin, vmax)
     int vmin, vmax;
{
  setscrolllimit (scroll1, vmin, vmax);
}

setscrollhlimit (vmin, vmax)
     int vmin, vmax;
{
  setscrolllimit (scroll2, vmin, vmax);
}

main (argc, argv)
     int argc;
     char **argv;
{

  xsetup (argc, argv);

  xmainloop ();

}

int
xsetup (argc, argv)
     int argc;
     char **argv;
{
  XtTranslations trans_table;
  XtTranslations text_trans_table;
  XtTranslations popup_trans_table;
  Widget info, cancel, ok;
  Widget dial, up, down, left, right;
  Widget colind, linind, corner;
  Widget w3, w4, w5, w6, root;
  Widget startbase, start;
  Pixmap startpixmap;
  int w, h;
  Arg args[20];
  register int n;
  char buf[256];
  Pixmap bm;

  Arg valueArgs[2];
  Arg centerArgs[2];
  Position source_x, source_y;
  Position dest_x, dest_y;
  Dimension center_width, center_height;
  Dimension prompt_width, prompt_height;

  XtSetLanguageProc ((XtAppContext) NULL,
		     (XtLanguageProc) NULL, (XtPointer) NULL);

  mainwin.toplevel = XtAppInitialize (&app_con, "abs",
				      options, XtNumber (options),
				  &argc, argv, fallback_resources, NULL, 0);

  if (argc != 1)
    {
      (void) fprintf (stderr, "Usage: %s [Xt options]\n", argv[0]);
      exit (1);
    }

  if (mainwin.toplevel != NULL)
    {
      display = XtDisplayOfObject (mainwin.toplevel);
    }
  if (display == NULL)
    {
      fprintf (stderr, "Could not open display.");
      exit (1);
    }
  XtSetArg (args[0], XtNinput, True);
  XtSetValues (mainwin.toplevel, args, 1);

  XtGetApplicationResources (mainwin.toplevel, &res, applic_resources,
			     XtNumber (applic_resources), NULL, 0);

  w_n = 0;
  w_dim (800, 600);
  w_inbord (0);
  XtSetArg (w_args[w_n], XtNdefaultDistance, 1);
  w_n++;
  mainwin.baseform = XtCreateManagedWidget
    ("main_window", formWidgetClass, mainwin.toplevel, w_args, w_n);

  w_dim (800, 20);
  w_rel (NULL, 0, 0);
  w_bord (0);
  w_noresize ();
  mainwin.menubar = XtCreateManagedWidget
    ("menubar", boxWidgetClass, mainwin.baseform, w_args, w_n);
  mainwin.menubar_obj = (Menubar *) newmenubar (mainwin.menubar);
  XtAddEventHandler (mainwin.menubar, ExposureMask, FALSE, shadow, (caddr_t) mainwin.menubar);

  w_n = 0;
  w_dim (800, 20);
  w_relh (NULL, 0);
  w_relv (mainwin.menubar, 0);
  w_bord (0);
  w_noresize ();
  mainwin.buttonbar = XtCreateManagedWidget
    ("buttonbar", boxWidgetClass, mainwin.baseform, w_args, w_n);
  XtAddEventHandler (mainwin.buttonbar, ExposureMask, FALSE, shadow, (caddr_t) mainwin.buttonbar);

  w_n = 0;
  w_dim (800, 25);
  w_relh (NULL, 0);
  w_relv (mainwin.buttonbar, 0);
  w_bord (0);
  w_noresize ();
  mainwin.commandline = XtCreateManagedWidget
    ("commandline", boxWidgetClass, mainwin.baseform, w_args, w_n);
  XtAddEventHandler (mainwin.commandline, ExposureMask, FALSE, shadow, (caddr_t) mainwin.commandline);

  mainwin.commandline_obj = newcommandline (mainwin.commandline);

  text = mainwin.commandline_obj->text;
  info = mainwin.commandline_obj->info;

  w_n = 0;
  w_relh (NULL, 0);
  w_relv (mainwin.commandline, 0);
  w_bord (0);
  XtSetArg (w_args[w_n], XtNleft, XtChainLeft);
  w_n++;
  XtSetArg (w_args[w_n], XtNtop, XtChainTop);
  w_n++;
  XtSetArg (w_args[w_n], XtNbottom, XtChainBottom);
  w_n++;
  XtSetArg (w_args[w_n], XtNright, XtChainRight);
  w_n++;
  XtSetArg (w_args[w_n], XtNdefaultDistance, 0);
  w_n++;

  mainwin.wbform = (Widget) XtCreateManagedWidget ("window", formWidgetClass, mainwin.baseform,
						   w_args, w_n);
  XtAddEventHandler (mainwin.wbform, ExposureMask, FALSE, shadow, (caddr_t) mainwin.wbform);
  XtAddEventHandler (mainwin.wbform, StructureNotifyMask, False, resize, (caddr_t) mainwin.wbform);
  w_n = 0;
  w_dim (30, 20);
  w_rel (NULL, 0, 0);
  w_bord (0);
  w_noresize ();
  corner = (Widget) XtCreateManagedWidget ("window", simpleWidgetClass, mainwin.wbform,
					   w_args, w_n);

  w_n = 0;
  w_dim (30, 510);
  w_relh (NULL, 0);
  w_relv (corner, 0);
  w_bord (0);
  w_nohresize ();
  w_vresize ();
  XtSetArg (w_args[w_n], XtNtop, XtChainTop);
  w_n++;
  linind = (Widget) XtCreateManagedWidget ("window", panedWidgetClass, mainwin.wbform,
					   w_args, w_n);

  w_n = 0;
  w_dim (784, 20);
  w_relv (NULL, 0);
  w_relh (corner, 0);
  w_bord (0);
  w_novresize ();
  w_hresize ();
  XtSetArg (w_args[w_n], XtNleft, XtChainLeft);
  w_n++;
  colind = (Widget) XtCreateManagedWidget ("window", panedWidgetClass, mainwin.wbform,
					   w_args, w_n);

  w_n = 0;
  w_dim (784, 510);
  w_relv (colind, 0);
  w_relh (linind, 0);
  w_bord (0);
  w_hresize ();
  w_vresize ();

  mainwin.drw = (Widget) XtCreateManagedWidget ("window", panedWidgetClass, mainwin.wbform,
						w_args, w_n);

  make_pixmap_from_data (mainwin.baseform, scrollback_xpm, &bm);

  w_n = 0;
  w_relv (mainwin.commandline, 0);
  w_relh (mainwin.wbform, 1);
  w_bord (0);
  XtSetArg (w_args[w_n], XtNtop, XtChainTop);
  w_n++;
  XtSetArg (w_args[w_n], XtNleft, XtChainRight);
  w_n++;
  XtSetArg (w_args[w_n], XtNright, XtChainRight);
  w_n++;
  XtSetArg (w_args[w_n], XtNbottom, XtChainBottom);
  w_n++;
  XtSetArg (w_args[w_n], XtNdefaultDistance, 1);
  w_n++;

  scroll1 = CreateMyscroll (mainwin.baseform, 500, 14, 0, 400, scrollv);

  w3 = scroll1->back;

  w_n = 0;
  w_dim (360, 20);
  w_relh (NULL, 35);
  w_relv (mainwin.wbform, 0);
  w_bord (0);
  XtSetArg (w_args[w_n], XtNtop, XtChainBottom);
  w_n++;
  XtSetArg (w_args[w_n], XtNbottom, XtChainBottom);
  w_n++;
  XtSetArg (w_args[w_n], XtNleft, XtChainLeft);
  w_n++;
  XtSetArg (w_args[w_n], XtNright, XtChainLeft);
  w_n++;
  w5 = XtCreateManagedWidget ("window", formWidgetClass, mainwin.baseform,
			      w_args, w_n);
  build_thumb (w5);

  w_n = 0;
  w_relv (mainwin.wbform, 1);
  w_relh (NULL, 453);
  w_bord (0);
  XtSetArg (w_args[w_n], XtNtop, XtChainBottom);
  w_n++;
  XtSetArg (w_args[w_n], XtNbottom, XtChainBottom);
  w_n++;
  XtSetArg (w_args[w_n], XtNleft, XtChainLeft);
  w_n++;
  XtSetArg (w_args[w_n], XtNright, XtChainRight);
  w_n++;
  XtSetArg (w_args[w_n], XtNdefaultDistance, 1);
  w_n++;

  scroll2 = CreateMyscroll (mainwin.baseform, 14, 360, 0, NBCOL, scrollh);
  w4 = scroll2->back;

  mainwin.buttonbar_obj = newbuttonbar (mainwin.buttonbar);

  n = 0;
  XtSetArg (args[n], XtNheight, 60);
  n++;
  XtSetArg (args[n], XtNwidth, 50);
  n++;
  mainwin.popup = (Widget) make_alone_popup (mainwin.wbform, "pop1");

  XtAddEventHandler (mainwin.drw, ExposureMask, FALSE, cb_Repaint, NULL);
  XtAddEventHandler (mainwin.drw, StructureNotifyMask, FALSE, cb_Repaint, NULL);

  XtAppAddActions (app_con, actionsTable, XtNumber (actionsTable));

  trans_table = XtParseTranslationTable (defaultTranslations);
  text_trans_table = XtParseTranslationTable (textTranslations);
  popup_trans_table = XtParseTranslationTable (popupTranslations);

  XtOverrideTranslations (text, text_trans_table);

  XtOverrideTranslations (mainwin.popup, popup_trans_table);

  n = 0;

  XtOverrideTranslations (mainwin.drw, trans_table);
  XtOverrideTranslations (linind, trans_table);
  XtOverrideTranslations (colind, trans_table);
  XtSetValues (mainwin.drw, args, n);
  XtSetValues (linind, args, n);
  XtSetValues (colind, args, n);

  XtAddEventHandler (mainwin.drw, ButtonReleaseMask, True, cb_DocBtn3Up,
		     (XtPointer) mainwin.popup);

  XtAddEventHandler (text, KeyReleaseMask, True, cb_TextChange, text);

  make_format_dial ();
  make_plot_dial ();
  make_print_dial ();
  XtRealizeWidget (mainwin.toplevel);

  w_n = 0;
  w_dim (462, 319);
  startbase = XtCreatePopupShell
    ("abs Starting ...", transientShellWidgetClass, mainwin.toplevel, w_args, w_n);

  XtSetArg (centerArgs[0], XtNwidth, &center_width);
  XtSetArg (centerArgs[1], XtNheight, &center_height);
  XtGetValues (mainwin.baseform, centerArgs, 2);
  XtSetArg (centerArgs[0], XtNwidth, &prompt_width);
  XtSetArg (centerArgs[1], XtNheight, &prompt_height);
  XtGetValues (startbase, centerArgs, 2);
  source_x = (int) (center_width - prompt_width) / 2;
  source_y = (int) (center_height - prompt_height) / 2;
  XtTranslateCoords (mainwin.baseform, source_x, source_y, &dest_x, &dest_y);
  XtSetArg (centerArgs[0], XtNx, dest_x);
  XtSetArg (centerArgs[1], XtNy, dest_y);
  XtSetValues (startbase, centerArgs, 2);

  XtManageChild (startbase);

  make_pixmap_from_data (startbase, abs_xpm, &startpixmap);
  XSetWindowBackgroundPixmap (XtDisplay (startbase), XtWindow (startbase), startpixmap);
  XFreePixmap (XtDisplay (startbase), startpixmap);
  XClearWindow (XtDisplay (startbase), XtWindow (startbase));

  cbsetup (mainwin.toplevel, mainwin.drw, 760, 500, colind, linind, text, info);

  XtUnmanageChild (startbase);

  XtSetKeyboardFocus (mainwin.toplevel, mainwin.drw);

}

xmainloop ()
{
  XtAppMainLoop (app_con);
  return (0);
}

void
cb_DocBtn3Up (Widget w, XtPointer cd, XEvent * event, Boolean * ctd)
{

  Arg args[20];
  register int n;
  int x, y;

  if (((XButtonEvent *) event)->button != Button3)
    return;

  n = 0;
  XtSetArg (args[n], XtNx, &x);
  n++;
  XtSetArg (args[n], XtNy, &y);
  n++;
  XtGetValues (mainwin.drw, args, n);

  x = event->xbutton.x_root - 30;
  y = event->xbutton.y_root - 10;

  XtSetArg (args[n], XtNx, x);
  n++;
  XtSetArg (args[n], XtNy, y);
  n++;
  XtSetValues (mainwin.popup, args, n);
  XtPopup (mainwin.popup, XtGrabNone);

}

void
cb_popdown (w, event, params, num_params)
     Widget w;
     XEvent *event;
     String *params;
     int *num_params;
{

  XtPopdown (mainwin.popup);

}
