/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include "chrt.h"

Chrt *ActivChrt;

static int nchrt = 1;

Chrt *
newchrt ()
{
  int i;
  Chrt *chrt;
  chrt = (Chrt *) absmalloc (sizeof (Chrt), "newchrt:chrt ");

  chrt->zm = 100;
  chrt->numrange = 0;
  chrt->type = 0;
  chrt->graphic_grid = grboth;
  chrt->axe_cat_grid = 0;
  chrt->axe_val_grid = 0;

  chrt->Axes[xlCategory].HasMajorGridLines = 0;
  chrt->Axes[xlCategory].MinimumScaleIsAuto = True;
  chrt->Axes[xlCategory].MaximumScaleIsAuto = True;
  chrt->Axes[xlCategory].MajorUnitIsAuto = True;
  chrt->Axes[xlCategory].HasTitle = True;
  title_settext (&chrt->Axes[xlCategory].AxisTitle, "'X axis");

  chrt->Axes[xlValue].HasMajorGridLines = 0;
  chrt->Axes[xlValue].MinimumScaleIsAuto = True;
  chrt->Axes[xlValue].MaximumScaleIsAuto = True;
  chrt->Axes[xlValue].MajorUnitIsAuto = True;
  chrt->Axes[xlValue].HasTitle = True;
  title_settext (&chrt->Axes[xlValue].AxisTitle, "'Y axis");

  for (i = 0; i < 10; i++)
    {
      sprintf ((chrt->range[i]).name, "'Data%d", i);
      chrt->graphic_format[i] = grboth;
    }

  chrt->HasTitle = True;
  title_settext (&chrt->ChtTitle, "'title");

  sprintf ((chrt->range[0]).name, "'X axis");

  chrt->objn = nchrt++;

  return chrt;
};

Chrt *
newchrt1 (gr)
     Chrt *gr;
{
  Chrt *chrt = newchrt ();
  chrt_copy (chrt, gr);
  chrt->objn = nchrt++;
  return chrt;
};

int
chrt_copy (chrt, gr)
     Chrt *chrt;
     Chrt *gr;
{
  int i;
  if (chrt == NULL || gr == NULL || (gr == chrt))
    return;

  chrt->zm = gr->zm;
  chrt->numrange = gr->numrange;
  chrt->type = gr->type;
  chrt->graphic_grid = gr->graphic_grid;
  chrt->axe_cat_grid = gr->axe_cat_grid;
  chrt->Axes[xlCategory].HasMajorGridLines = gr->Axes[xlCategory].HasMajorGridLines;
  chrt->Axes[xlCategory].MinimumScaleIsAuto = gr->Axes[xlCategory].MinimumScaleIsAuto;
  chrt->Axes[xlCategory].MajorUnitIsAuto = gr->Axes[xlCategory].MajorUnitIsAuto;
  chrt->Axes[xlCategory].HasTitle = gr->Axes[xlCategory].HasTitle;

  chrt->Axes[xlValue].HasMajorGridLines = gr->Axes[xlValue].HasMajorGridLines;
  chrt->Axes[xlValue].MinimumScaleIsAuto = gr->Axes[xlValue].MinimumScaleIsAuto;
  chrt->Axes[xlValue].MajorUnitIsAuto = gr->Axes[xlValue].MajorUnitIsAuto;
  chrt->Axes[xlValue].HasTitle = gr->Axes[xlValue].HasTitle;

  for (i = 0; i < 10; i++)
    {
      strcpy ((chrt->range[i]).name, (gr->range[i]).name);
      chrt->graphic_format[i] = gr->graphic_format[i];
    }

  chrt->HasTitle = gr->HasTitle;

}

int
freechrt (gr)
     Chrt *gr;
{
  if (gr != NULL);
  absfree (gr, "freechrt:gr ");
};

int
chrt_w (gr, fp)
     Chrt *gr;
     FILE *fp;
{
  char i1j1[16];
  char i2j2[16];
  int j;
  char *type;
  switch (gr->type)
    {
    case 0:
      type = "xlLine";
      break;
    case 1:
      type = "xlPie";
      break;
    case 2:
      type = "xlbarClustered";
      break;
    }

  fprintf (fp, "ChartObjects.Add (%d,%d,%d,%d)\n", gr->x, gr->y, gr->w, gr->h);
  fprintf (fp, "ActiveChart.ChartType = %s\n", type);

  for (j = 0; j < gr->numrange; j++)
    {
      n2a (i1j1, (gr->range[j]).i1, (gr->range[j]).j1);
      n2a (i2j2, (gr->range[j]).i2, (gr->range[j]).j2);
      fprintf (fp, "ActiveChart.SeriesCollection(%d).Name = \"%s\"\n", j, (gr->range[j]).name);
      fprintf (fp, "ActiveChart.SeriesCollection(%d).Values = \"%s:%s\"\n", j, i1j1, i2j2);
    }

  if (gr->HasTitle)
    {
      fprintf (fp, "ActiveChart.HasTitle = True\n");
      fprintf (fp, "ActiveChart.ChartTitle.Text = \"%s\"\n", gr->ChtTitle.Text);
    }

  if (gr->Axes[xlCategory].HasMajorGridLines)
    fprintf (fp, "ActiveChart.Axes(xlCategory).HasMajorGridlines = True\n");

  if (gr->Axes[xlCategory].HasTitle)
    {
      fprintf (fp, "ActiveChart.Axes(xlCategory).HasTitle = True\n");
      fprintf (fp, "ActiveChart.Axes(xlCategory).AxisTitle.Text = \"%s\"\n",
	       gr->Axes[xlCategory].AxisTitle.Text);
    }

  if (gr->Axes[xlValue].HasMajorGridLines)
    fprintf (fp, "ActiveChart.Axes(xlValue).HasMajorGridlines = True\n");

  if (gr->Axes[xlValue].HasTitle)
    {
      fprintf (fp, "ActiveChart.Axes(xlValue   ).HasTitle = True\n");
      fprintf (fp, "ActiveChart.Axes(xlValue   ).AxisTitle.Text = \"%s\"\n",
	       gr->Axes[xlValue].AxisTitle.Text);
    }

}

int
ActivateChrt (chrt)
     Chrt *chrt;
{
  ActivChrt = chrt;

};

int
chrt_settype (chrt, type)
     Chrt *chrt;
     int type;
{

  chrt->type = type;
}

int
chrt_setseriename (chrt, serie, name)
     Chrt *chrt;
     int serie;
     char *name;
{

  sprintf ((chrt->range[serie]).name, "%s", name);
}

int
chrt_setserievalues (gr, serie, valuerange)
     Chrt *gr;
     int serie;
     char *valuerange;
{

  char *i1j1;
  char *i2j2;
  char *separ;
  if (serie >= 10)
    return;

  separ = strchr (valuerange, ':');
  i1j1 = valuerange;
  i2j2 = separ + 1;
  separ[0] = '\0';

  alphatonum (i1j1, &(gr->range[serie]).i1, &(gr->range[serie]).j1);
  alphatonum (i2j2, &(gr->range[serie]).i2, &(gr->range[serie]).j2);

  if (serie + 1 > gr->numrange)
    gr->numrange = serie + 1;

}

int
chrt_setgrid (gr, axe, on)
     Chrt *gr;
     int axe;
     int on;
{
  if (axe == 1)
    gr->Axes[xlCategory].HasMajorGridLines = on;
  if (axe == 2)
    gr->Axes[xlValue].HasMajorGridLines = on;
}

int
chrt_setaxistitle (gr, axe, buf)
     Chrt *gr;
     int axe;
     char *buf;
{
  if (axe == 1)
    title_settext (&gr->Axes[xlCategory].AxisTitle, buf);
  if (axe == 2)
    title_settext (&gr->Axes[xlValue].AxisTitle, buf);
}
