/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "cl.h"
#include "wks.h"
#include "wkb.h"
#include "style.h"
#include "y.tab.h"

static char currentString[1024];
static int currentpos = 0;
static int updating = 0;
static int parsing = 0;
static int setting_form = 0;
static int upi = 0;
static int upj = 0;
static int uph = 0;
static int ncl = 0;
static Cl **tosetform = NULL;
static char **formtoset = NULL;
static int ntosetform = 0;
static int parsingmacro;

static Cl **toredrw = NULL;
static int ntoredrw = 0;

static char tmpbuf[256];

Cl *ActivCl;

Cl *
newcl (r, c)
     int r, c;
{
  Cl *nc;
  nc = (Cl *) absmalloc (sizeof (Cl), "newcl:Cl ");
  if (nc == NULL)
    fprintf (stderr, "Cl: not allocated!");

  nc->val.rec.d = 0.0;
  nc->val.type = FLOATING_CONSTANT;
  nc->form = NULL;

  nc->style = NULL;
  nc->r = r;
  nc->c = c;

  nc->familly = NULL;
  nc->wks = ActivWks;

  nc->flag1 = 0;

  nc->objn = ncl++;

  return nc;
}

freecl (cl)
     Cl *cl;
{
  if (cl == NULL)
    return;

  if (cl->val.type == STRING_CONSTANT && cl->val.rec.s != cl->form + 1)
    if (cl->val.rec.s != NULL)
      absfree (cl->val.rec.s, "freecl:cl->val.rec.s ");

  if (cl->form != NULL)
    absfree (cl->form, "freecl:cl->form ");
  if (cl->style != NULL)
    freestyle (cl->style);
  absfree (cl, "freecl:cl ");
}

int
cl_w (cl, fp)
     Cl *cl;
     FILE *fp;
{
  int nextc = 0;
  int nextr = 0;
  int deci, format;
  char *end;

  if (cl->form != NULL)
    {
      end = strchr (cl->form, '\n');
      if (end != NULL)
	end[0] = '\0';
      if (cl->form[0] == '\'' || cl->form[0] == '-' || cl->form[0] == '=')
	fprintf (fp, "Cells(%d,%d).Formula=\"%s\"\n",
		 cl->r, cl->c, cl->form);
      else
	fprintf (fp, "Cells(%d,%d).Formula=\"=%s\"\n",
		 cl->r, cl->c, cl->form);
    }

  if (cl->style != NULL)
    {
      if (getstyle (cl->style, STY_BORDER_TOP) &&
	  getstyle (cl->style, STY_BORDER_BOT) &&
	  getstyle (cl->style, STY_BORDER_RIG) &&
	  getstyle (cl->style, STY_BORDER_LEF)
	)
	{
	  fprintf (fp, "Cells(%d,%d).Borders.LineStyle = xlLineStyleContinuous\n", cl->r, cl->c);
	}
      else
	{
	  if (getstyle (cl->style, STY_BORDER_TOP))
	    fprintf (fp, "Cells(%d,%d).Borders(xlEdgeTop).LineStyle = xlLineStyleContinuous\n", cl->r, cl->c);
	  if (getstyle (cl->style, STY_BORDER_BOT))
	    fprintf (fp, "Cells(%d,%d).Borders(xlEdgeBottom).LineStyle = xlLineStyleContinuous\n", cl->r, cl->c);
	  if (getstyle (cl->style, STY_BORDER_RIG))
	    fprintf (fp, "Cells(%d,%d).Borders(xlEdgeRight).LineStyle = xlLineStyleContinuous\n", cl->r, cl->c);
	  if (getstyle (cl->style, STY_BORDER_LEF))
	    fprintf (fp, "Cells(%d,%d).Borders(xlEdgeLeft).LineStyle = xlLineStyleContinuous\n", cl->r, cl->c);
	}

      if (getstyle (cl->style, STY_FONT) != getstyle (ActivWkb->style, STY_FONT))
	switch (getstyle (cl->style, STY_FONT))
	  {
	  case 0:
	    fprintf (fp, "Cells(%d,%d).Font.Name = \"Fixed\"\n", cl->r, cl->c);
	    break;
	  case 1:
	    fprintf (fp, "Cells(%d,%d).Font.Name = \"Courier\"\n", cl->r, cl->c);
	    break;
	  case 2:
	    fprintf (fp, "Cells(%d,%d).Font.Name = \"Helvetica\"\n", cl->r, cl->c);
	    break;
	  case 3:
	    fprintf (fp, "Cells(%d,%d).Font.Name = \"Times\"\n", cl->r, cl->c);
	    break;
	  case 4:
	    fprintf (fp, "Cells(%d,%d).Font.Name = \"Symbol\"\n", cl->r, cl->c);
	    break;
	  }

      if (getstyle (cl->style, STY_FONTS) != getstyle (ActivWkb->style, STY_FONTS))
	switch (getstyle (cl->style, STY_FONTS))
	  {
	  case 0:
	    fprintf (fp, "Cells(%d,%d).Font.Size = 6\n", cl->r, cl->c);
	    break;
	  case 1:
	    fprintf (fp, "Cells(%d,%d).Font.Size = 8\n", cl->r, cl->c);
	    break;
	  case 2:
	    fprintf (fp, "Cells(%d,%d).Font.Size = 10\n", cl->r, cl->c);
	    break;
	  case 3:
	    fprintf (fp, "Cells(%d,%d).Font.Size = 12\n", cl->r, cl->c);
	    break;
	  case 4:
	    fprintf (fp, "Cells(%d,%d).Font.Size = 14\n", cl->r, cl->c);
	    break;
	  case 5:
	    fprintf (fp, "Cells(%d,%d).Font.Size = 18\n", cl->r, cl->c);
	    break;
	  case 6:
	    fprintf (fp, "Cells(%d,%d).Font.Size = 24\n", cl->r, cl->c);
	    break;
	  }
      if (getstyle (cl->style, STY_FONTW) == 1 || getstyle (cl->style, STY_FONTW) == 3)
	fprintf (fp, "Cells(%d,%d).Font.Bold = True \n", cl->r, cl->c);

      if (getstyle (cl->style, STY_FONTW) == 2 || getstyle (cl->style, STY_FONTW) == 3)
	fprintf (fp, "Cells(%d,%d).Font.Italic = True \n", cl->r, cl->c);

      if (getstyle (cl->style, STY_JUSTIF) != getstyle (ActivWkb->style, STY_JUSTIF))
	switch (getstyle (cl->style, STY_JUSTIF))
	  {
	  case 0:
	    fprintf (fp, "Cells(%d,%d).HorizontalAlignment = xlLeft\n",
		     cl->r, cl->c);
	    break;
	  case 1:
	    fprintf (fp, "Cells(%d,%d).HorizontalAlignment = xlCenter\n",
		     cl->r, cl->c);
	    break;
	  case 2:
	    fprintf (fp, "Cells(%d,%d).HorizontalAlignment = xlRight\n",
		     cl->r, cl->c);
	    break;
	  }

      if (getstyle (cl->style, STY_BG) != getstyle (ActivWkb->style, STY_BG))
	fprintf (fp, "Cells(%d,%d).Interior.ColorIndex = %d\n",
		 cl->r, cl->c, getstyle (cl->style, STY_BG) - 5);
      if (getstyle (cl->style, STY_FG) != getstyle (ActivWkb->style, STY_FG))
	fprintf (fp, "Cells(%d,%d).Font.ColorIndex = %d\n",
		 cl->r, cl->c, getstyle (cl->style, STY_FG) - 5);

      format = getstyle (cl->style, STY_FORMAT);
      deci = getstyle (cl->style, STY_DECIMAL);
      if ((format != getstyle (ActivWkb->style, STY_FORMAT) ||
	   deci != getstyle (ActivWkb->style, STY_DECIMAL)) &&
	  format != 0)
	{
	  if (format >= 2 && format <= 4)
	    {
	      fprintf (fp, "Cells(%d,%d).NumberFormat = \"0", cl->r, cl->c);
	      if (deci)
		fprintf (fp, ".");
	      while (deci > 0)
		{
		  fprintf (fp, "0");
		  deci--;
		};
	    }
	  switch (format)
	    {
	    case 2:
	      fprintf (fp, "\"\n");
	      break;
	    case 3:
	      fprintf (fp, "E+00\"\n");
	      break;
	    case 4:
	      fprintf (fp, "%%\"\n");
	      break;
	    case 5:
	      {
		fprintf (fp, "Cells(%d,%d).NumberFormat = \"", cl->r, cl->c);
		fprintf (fp, "dd/mm/yy\"\n");
		break;
	      }
	    default:
	      fprintf (fp, "\"\n");
	    }
	}

    }

}

int
cl_setnumberformat (cl, buf)
     Cl *cl;
     char *buf;
{

  int deci = 0;
  char *dot;
  char *token;

  if (strstr (buf, "General"))
    return;

  token = strchr (buf, ';');
  if (token != NULL)
    {
      token[0] = '\0';
      token++;
    }

  if (buf == NULL)
    return;

  if (strchr (buf, '/'))
    {
      cl_setformat (cl, 5, 0);
      return;
    }

  dot = strchr (buf, '.');

  while (dot[deci + 1] == '0' && deci < strlen (dot))
    {
      deci++;
    }

  if (strchr (buf, 'E') || strchr (buf, 'e'))
    {
      cl_setformat (cl, 3, deci);
    }
  else if (strchr (buf, '%'))
    {
      cl_setformat (cl, 4, deci);
    }
  else
    {
      cl_setformat (cl, 2, deci);
    }

};

int
cl_setform (cl, buf)
     Cl *cl;
     char *buf;
{
  char *end;
  char backup[256];
  int format, deci, date;
  int i;
  int ret;

  if (cl == NULL)
    {
      return;
    }

  if (parsingmacro)
    {
      tosetform = (Cl **) absrealloc (tosetform, sizeof (Cl *) * (ntosetform + 1), "cl_setform:tosetform ");
      if (tosetform == NULL)
	fprintf (stderr, "tosetform not allocated!\n");
      formtoset = (char **) absrealloc (formtoset, sizeof (char *) * (ntosetform + 1), "cl_setform:formtoset ");
      if (formtoset == NULL)
	fprintf (stderr, "formtoset not allocated!\n");

      formtoset[ntosetform] = (char *) absmalloc (strlen (buf) + 2, "cl_setform:formtoset[ntosetform] ");
      if (formtoset[ntosetform] == NULL)
	fprintf (stderr, "formtoset[]  not allocated!\n");

      strcpy (formtoset[ntosetform], buf);

      tosetform[ntosetform] = cl;
      ntosetform++;
      return;
    }

  if (cl->form != NULL)
    absfree (cl->form, "cl_setform:cl->form ");

  cl_isnew (cl);

  if (buf == NULL)
    buf = "\n";

  if (strlen (buf) > 0)
    {
      while (buf[0] == ' ' && buf[0] != '\0' && buf[0] != '\n')
	buf++;

      if (buf[0] == '=')
	buf++;

      cl->form = (char *) absmalloc (sizeof (char) * (strlen (buf) + 5), "cl_setform:cl->form ");
      if (cl->form == NULL)
	fprintf (stderr, "form not allocated\n");

      cl_gformat (cl, &format, &deci);
      if (format == 5)
	{
	  date = scandate (buf);
	  if (date)
	    sprintf (buf, "%d\n", date);
	};

      strcpy (cl->form, buf);

      if (strncmp (buf, "'", 1) == 0)
	{
	  end = NULL;
	  end = strchr (buf, '\n');
	  if (end != NULL)
	    end = '\0';
	  cl->val.type = STRING_CONSTANT;
	  cl->val.rec.s = cl->form + 1;
	  cl->flag1 = 1;
	  setting_form = 1;
	  nupdate2 (cl);
	  setting_form = 0;
	  return 1;
	}

      end = NULL;
      end = strchr (buf, '\n');
      if (end == NULL)
	{
	  strcat (cl->form, "\n\0");
	}

      cl->flag1 = 0;

      ActivCl = cl;

      setting_form = 1;

      freenewstring ();

      setcl2parse (cl);
      ret = yyparse ();
      setcl2parse (NULL);

      if (ret)
	{
	  freenewstring ();
	  cl->flag1 = 1;
	  for (i = strlen (cl->form); i > 0; i--)
	    cl->form[i] = cl->form[i - 1];
	  cl->form[0] = '\'';

	  end = NULL;
	  end = strchr (cl->form, '\n');
	  if (end != NULL)
	    end = '\0';

	  cl->val.type = STRING_CONSTANT;
	  cl->val.rec.s = cl->form + 1;
	  nupdate2 (cl);
	  setting_form = 0;
	  return 1;
	};

      end = NULL;
      end = strchr (cl->form, '\n');
      if (end != NULL)
	end = '\0';

      setting_form = 0;
      nupdate2 (cl);
      return 0;
    }
  cl->form = NULL;
  cl->val.type = STRING_CONSTANT;
  cl->val.rec.s = NULL;
  cl->flag1 = 1;
}

int
cl_settext (cl, buf)
     Cl *cl;
     char *buf;
{
  char *end;
  char *first;

  if (cl == NULL)
    {
      return;
    }
  if (cl->val.type == STRING_CONSTANT)
    if (cl->val.rec.s != NULL)
      absfree (cl->val.rec.s, "cl_settext:cl->val.rec.s ");

  if (strlen (buf) > 0)
    {

      end = strchr (buf, '\n');
      if (end != NULL)
	end = '\0';

      cl->val.rec.s = (char *) absmalloc (sizeof (char) * (strlen (buf) + 5), "cl_settext:cl->form ");
      if (cl->val.rec.s == NULL)
	fprintf (stderr, "val.rec.s not allocated\n");

      first = buf;
      while (*first == ' ' && first < buf + strlen (buf))
	first++;
      if (*first == '\'')
	{
	  strcpy (cl->val.rec.s, "'");
	  strcat (cl->val.rec.s, first + 1);
	  cl->form = cl->val.rec.s;
	  cl->val.rec.s = cl->form + 1;
	}
      else
	strcpy (cl->val.rec.s, buf);

    }
  else
    cl->val.rec.s = NULL;

  cl->val.type = STRING_CONSTANT;
  cl->flag1 = 1;
}

char *
cl_gform (cl)
     Cl *cl;
{
  if (cl != NULL)
    return cl->form;
  return NULL;
}

char *
cl_gnumberformat (cl)
     Cl *cl;
{

  return NULL;
}

double
cl_setvalue (cl, val)
     Cl *cl;
     double val;
{
  if (cl != NULL)
    {

      cl->val.rec.d = val;
      cl->val.type = FLOATING_CONSTANT;
      return cl->val.rec.d;
    }
  return 0.0;
}

double
cl_setvalue2 (val)
     double val;
{

  if (ActivCl != NULL)
    {

      ActivCl->val.rec.d = val;
      ActivCl->val.type = FLOATING_CONSTANT;
      return ActivCl->val.rec.d;
    }
  return 0.0;
}

double
cl_gvalue (cl)
     Cl *cl;
{
  if (cl == NULL)
    {
      return 0.0;
    }

  if (cl->val.type == FLOATING_CONSTANT)
    return cl->val.rec.d;
  else
    return 0.0;
}

int
cl_istext (cl)
     Cl *cl;
{
  return cl->flag1;
}

char *
cl_gtext (cl)
     Cl *cl;
{
  char form[8];
  int format, deci;
  int dd, mm, yy;
  double value;
  if (cl == NULL)
    {
      return NULL;
    }
  if (cl->form == NULL)
    {
      return NULL;
    }
  if (cl->flag1)
    return cl->val.rec.s;

  cl_gformat (cl, &format, &deci);

  value = cl_gvalue (cl);

  switch (format)
    {
    case 1:
      {
	sprintf (form, "%%g");
	sprintf (tmpbuf, form, value);
	break;
      }
    case 2:
      {
	sprintf (form, "%%.%dlf", deci);
	sprintf (tmpbuf, form, value);
	break;
      }
    case 3:
      {
	sprintf (form, "%%.%de", deci);
	sprintf (tmpbuf, form, value);
	break;
      }
    case 4:
      {
	sprintf (form, "%%.%dlf%%%%", deci);
	sprintf (tmpbuf, form, value * 100.);
	break;
      }
    case 5:
      {
	int date = value;
	num2date (date, &dd, &mm, &yy);
	sprintf (tmpbuf, "%2d/%2d/%4d", dd, mm, yy);
	break;
      }

    }

  return tmpbuf;
}

int
cl_setfg (cl, f)
     Cl *cl;
     int f;
{
  if (cl == NULL)
    return -1;
  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);
  setstyle (cl->style, STY_FG, f + 5);
}
int
cl_gfg (cl)
     Cl *cl;
{
  if (cl == NULL || cl->style == NULL)
    return getstyle (ActivWkb->style, STY_FG);

  return getstyle (cl->style, STY_FG);
}

int
cl_setbg (cl, f)
     Cl *cl;
     int f;
{
  if (cl == NULL)
    return -1;
  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);

  setstyle (cl->style, STY_BG, f + 5);
}

int
cl_gbg (cl)
     Cl *cl;
{
  if (cl == NULL || cl->style == NULL)
    return getstyle (ActivWkb->style, STY_BG);
  return getstyle (cl->style, STY_BG);
}

int
cl_gtop (cl)
     Cl *cl;
{
  if (cl == NULL || cl->style == NULL)
    return getstyle (ActivWkb->style, STY_BORDER_TOP);
  return getstyle (cl->style, STY_BORDER_TOP);
}
int
cl_gbot (cl)
     Cl *cl;
{
  if (cl == NULL || cl->style == NULL)
    return getstyle (ActivWkb->style, STY_BORDER_BOT);
  return getstyle (cl->style, STY_BORDER_BOT);
}
int
cl_grig (cl)
     Cl *cl;
{
  if (cl == NULL || cl->style == NULL)
    return getstyle (ActivWkb->style, STY_BORDER_RIG);
  return getstyle (cl->style, STY_BORDER_RIG);
}
int
cl_glef (cl)
     Cl *cl;
{
  if (cl == NULL || cl->style == NULL)
    return getstyle (ActivWkb->style, STY_BORDER_LEF);
  return getstyle (cl->style, STY_BORDER_LEF);
}

int
cl_settop (cl, v)
     Cl *cl;
     int v;
{
  if (cl == NULL)
    return -1;
  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);
  setstyle (cl->style, STY_BORDER_TOP, v);
}
int
cl_setbot (cl, v)
     Cl *cl;
     int v;
{
  if (cl == NULL)
    return -1;
  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);
  setstyle (cl->style, STY_BORDER_BOT, v);
}
int
cl_setrig (cl, v)
     Cl *cl;
     int v;
{
  if (cl == NULL)
    return -1;
  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);
  setstyle (cl->style, STY_BORDER_RIG, v);
}
int
cl_setlef (cl, v)
     Cl *cl;
     int v;
{
  if (cl == NULL)
    return -1;
  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);
  setstyle (cl->style, STY_BORDER_LEF, v);
}

int
cl_setformat (cl, format, deci)
     Cl *cl;
     int format, deci;
{
  double date;

  if (cl == NULL)
    return -1;
  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);

  if (format == 5 && getstyle (cl->style, STY_FORMAT) != 5)
    {
      date = scandate (cl->form);
      if (date)
	cl_setvalue (cl, date);
    }

  setstyle (cl->style, STY_FORMAT, format);
  setstyle (cl->style, STY_DECIMAL, deci);

}

int
cl_gformat (cl, format, decimal)
     Cl *cl;
     int *format, *decimal;
{
  if (cl == NULL)
    return -1;
  if (cl->style == NULL)
    {
      *format = getstyle (ActivWkb->style, STY_FORMAT);
      *decimal = getstyle (ActivWkb->style, STY_DECIMAL);
      return 0;
    }
  *format = getstyle (cl->style, STY_FORMAT);
  *decimal = getstyle (cl->style, STY_DECIMAL);
  return 0;
}

int
cl_setjust (cl, just)
     Cl *cl;
     int just;
{
  if (cl == NULL)
    return -1;
  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);
  setstyle (cl->style, STY_JUSTIF, just);
}

int
cl_gjust (cl)
     Cl *cl;
{
  if (cl == NULL || cl->style == NULL)
    return getstyle (ActivWkb->style, STY_JUSTIF);
  return getstyle (cl->style, STY_JUSTIF);
}

int
cl_setfont (cl, font, weight, size)
     Cl *cl;
     int font;
{
  if (cl == NULL)
    return -1;
  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);
  if (font >= 0)
    setstyle (cl->style, STY_FONT, font);
  if (weight >= 0)
    setstyle (cl->style, STY_FONTW, weight);
  if (size >= 0)
    setstyle (cl->style, STY_FONTS, size);
}

int
cl_fname (cl, name)
     Cl *cl;
     char *name;
{

  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);
  if (!strcasecmp (name, "fixed"))
    setstyle (cl->style, STY_FONT, 0);
  if (!strcasecmp (name, "courier"))
    setstyle (cl->style, STY_FONT, 1);
  if (!strcasecmp (name, "helvetica"))
    setstyle (cl->style, STY_FONT, 2);
  if (!strcasecmp (name, "times"))
    setstyle (cl->style, STY_FONT, 3);
  if (!strcasecmp (name, "symbol"))
    setstyle (cl->style, STY_FONT, 4);
};

int
cl_fontsize (cl, v)
     Cl *cl;
     int v;
{

  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);
  switch (v)
    {
    case 6:
      setstyle (cl->style, STY_FONTS, 0);
      break;
    case 8:
      setstyle (cl->style, STY_FONTS, 1);
      break;
    case 10:
      setstyle (cl->style, STY_FONTS, 2);
      break;
    case 12:
      setstyle (cl->style, STY_FONTS, 3);
      break;
    case 14:
      setstyle (cl->style, STY_FONTS, 4);
      break;
    case 18:
      setstyle (cl->style, STY_FONTS, 5);
      break;
    case 24:
      setstyle (cl->style, STY_FONTS, 6);
      break;
    }
}

int
cl_fblt (cl, v)
     Cl *cl;
     int v;
{

  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);
  if (v == 0)
    {
      if (getstyle (cl->style, STY_FONTW) > 1)
	setstyle (cl->style, STY_FONTW, 2);
      else
	setstyle (cl->style, STY_FONTW, 0);
    }
  else
    {
      if (getstyle (cl->style, STY_FONTW) > 1)
	setstyle (cl->style, STY_FONTW, 3);
      else
	setstyle (cl->style, STY_FONTW, 1);
    }
}

int
cl_fital (cl, v)
     Cl *cl;
     int v;
{
  int w;

  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);
  w = getstyle (cl->style, STY_FONTW);
  if (v == 0)
    {
      if (w == 1 || w == 3)
	setstyle (cl->style, STY_FONTW, 1);
      else
	setstyle (cl->style, STY_FONTW, 0);
    }
  else
    {
      if (w == 1 || w == 3)
	setstyle (cl->style, STY_FONTW, 3);
      else
	setstyle (cl->style, STY_FONTW, 2);
    }
}

int
cl_gfont (cl)
     Cl *cl;
{
  if (cl == NULL || cl->style == NULL)
    return getstyle (ActivWkb->style, STY_FONT);
  return getstyle (cl->style, STY_FONT);
}

int
cl_gfontw (cl)
     Cl *cl;
{
  if (cl == NULL || cl->style == NULL)
    return getstyle (ActivWkb->style, STY_FONTW);
  return getstyle (cl->style, STY_FONTW);
}
int
cl_gfonts (cl)
     Cl *cl;
{
  if (cl == NULL || cl->style == NULL)
    return getstyle (ActivWkb->style, STY_FONTS);
  return getstyle (cl->style, STY_FONTS);
}

int
cl_setbord (cl, border, type)
     Cl *cl;
     int border;
     int type;
{

  if (cl == NULL)
    return -1;
  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);

  if (border == 8 || border == 0)
    setstyle (cl->style, STY_BORDER_TOP, type);
  if (border == 7 || border == 0)
    setstyle (cl->style, STY_BORDER_LEF, type);
  if (border == 9 || border == 0)
    setstyle (cl->style, STY_BORDER_BOT, type);
  if (border == 10 || border == 0)
    setstyle (cl->style, STY_BORDER_RIG, type);
}

int
cl_setborder (cl, border)
     Cl *cl;
     int border;
{
  if (cl == NULL)
    return -1;
  if (cl->style == NULL)
    cl->style = newstyle (STY_CELL);

  setstyle (cl->style, STY_BORDER, border);
}

int
cl_gborder (cl)
     Cl *cl;
{
  if (cl == NULL || cl->style == NULL)
    return getstyle (ActivWkb->style, STY_BORDER);
  return getstyle (cl->style, STY_BORDER);
}

static int flag = 0;
static int parsepos = 0;
static Cl *cl2parse = NULL;

setcl2parse (cl)
     Cl *cl;
{
  cl2parse = cl;
  parsepos = 0;
}

int
StringInput ()
{
  int n;
  int c;

  if (cl2parse != NULL)
    {
      if (cl2parse->form == NULL)
	return '\n';
      if (parsepos >= strlen (cl2parse->form))
	return '\n';
      parsepos++;
      return cl2parse->form[parsepos - 1];
    }

  if (currentpos < 1024)
    {
      c = (int) currentString[currentpos];
      currentpos++;
      return c;
    }
  return '\n';
}

int
copymod (ii, jj, i, j, incr)
     int i, j, ii, jj, incr;
{
  char *buf;
  Cl *cl1;
  Cl *cl2;
  cl1 = (Cl *) appliccl (i, j, -1);
  cl2 = (Cl *) appliccl (ii, jj, -1);

  if (cl2 == NULL)
    {
      if (cl1 != NULL)
	cl_setform (cl1, "");
      return;
    }

  cl1 = (Cl *) appliccl (i, j, 1);

  if (incr < 0)
    {
      setnodollars (1);
      incr = 1;
    }
  if (cl2->flag1 == 1)
    {
      buf = cl_gform (cl2);
      if (buf != NULL)
	cl_setform (cl1, buf);
      else
	cl_setform (cl1, "");
      return 0;
    }

  if (cl_gform (cl2) != NULL)
    {
      ActivCl = cl1;

      setdi ((i - ii) * incr);
      setdj ((j - jj) * incr);
      freenewstring ();

      setcl2parse (cl2);
      yyparse ();
      setcl2parse (NULL);

      if (cl1->form != NULL)
	absfree (cl1->form, "copymod:cl1->form ");

      cl1->form = (char *) absmalloc (sizeof (char) * (lennewstring () + 1), "copymod:cl1->form ");
      if (cl1->form == NULL)
	fprintf (stderr, "form cl1 not allocated\n");
      sprintf (cl1->form, "%s", (char *) getnewstring ());

      freenewstring ();
      setdi (0);
      setdj (0);
    }
  cl1->flag1 = 0;

  setnodollars (0);

  nupdate2 (cl1);
  return 0;
}

int
copymod2 (cl1, cl2, _curi, _curj, _di, _dj)
     Cl *cl1;
     Cl *cl2;
     int _curi, _curj, _di, _dj;
{
  if (cl2->flag1 == 1)
    {
      cl_setform (cl1, cl_gform (cl2));
      return 0;
    }

  if (cl_gform (cl2) != NULL)
    {
      ActivCl = cl1;

      setdi (_di);
      setdj (_dj);
      freenewstring ();

      setcl2parse (cl2);
      yyparse ();
      setcl2parse (NULL);

      if (cl1->form != NULL)
	absfree (cl1->form, "copymod2:cl1->form ");

      cl1->form = (char *) absmalloc (sizeof (char) * (lennewstring () + 1), "copymod2:cl1->form ");
      if (cl1->form == NULL)
	fprintf (stderr, "form cl1 not allocated\n");
      sprintf (cl1->form, "%s", (char *) getnewstring ());

      freenewstring ();
      setdi (0);
      setdj (0);
    }
  cl1->flag1 = 0;

  setnodollars (0);

  nupdate2 (cl1);
  return 0;
}

int
copymod2bad (cl1, cl2, _curi, _curj, _di, _dj)
     Cl *cl1;
     Cl *cl2;
     int _curi, _curj, _di, _dj;
{
  setdi (_di);
  setdj (_dj);

  cl_setform (cl1, cl_gform (cl2));

  cl1->form = (char *) absmalloc (sizeof (char) * (lennewstring () + 1), "copymod2bad:cl1->form ");
  if (cl1->form == NULL)
    {
      fprintf (stderr, "form cl1 not allocated\n");
      return;
    }

  strcpy (cl1->form, (char *) getnewstring ());

  freenewstring ();
  setdi (0);
  setdj (0);

  return 0;
}

cl1isfcl2 (cl1, cl2)
     Cl *cl1;
     Cl *cl2;
{
  int npar1;
  int nchi2;
  int k;
  Cl **tmp;

  if (cl1 == cl2)
    return;

  if (cl1->familly == NULL)
    {
      cl1->familly = (Fam *) absmalloc (sizeof (Fam), "cl1isfcl2:cl1->familly ");
      if (cl1->familly == NULL)
	fprintf (stderr, "cl1->familly not allocated!\n");
      cl1->familly->npar = 0;
      cl1->familly->dparlst = 0;
      cl1->familly->parlst = NULL;
      cl1->familly->nchi = 0;
      cl1->familly->dchilst = 0;
      cl1->familly->chilst = NULL;
    }
  if (cl2->familly == NULL)
    {
      cl2->familly = (Fam *) absmalloc (sizeof (Fam), "cl1isfcl2:cl2->familly ");
      if (cl2->familly == NULL)
	fprintf (stderr, "cl2->familly not allocated!\n");
      cl2->familly->npar = 0;
      cl2->familly->dparlst = 0;
      cl2->familly->parlst = NULL;
      cl2->familly->nchi = 0;
      cl2->familly->dchilst = 0;
      cl2->familly->chilst = NULL;
    }

  npar1 = cl1->familly->npar;
  nchi2 = cl2->familly->nchi;

  if (cl1->familly->dparlst < npar1 + 1)
    {
      tmp = (Cl **) absmalloc (sizeof (Cl *) * (2 * npar1 + 2), "cl1isfcl2:tmp ");
      if (tmp == NULL)
	fprintf (stderr, "tmp  not allocated!\n");
      for (k = 0; k < npar1; k++)
	tmp[k] = cl1->familly->parlst[k];
      if (cl1->familly->parlst != NULL)
	absfree (cl1->familly->parlst, "cl1isfcl2:cl1->familly->parlst ");
      cl1->familly->parlst = tmp;
      cl1->familly->dparlst = 2 * npar1 + 2;
    }

  if (cl2->familly->dchilst < nchi2 + 1)
    {
      tmp = (Cl **) absmalloc (sizeof (Cl *) * (2 * nchi2 + 2), "cl1isfcl2:tmp ");
      if (tmp == NULL)
	fprintf (stderr, "tmp  not allocated!\n");
      for (k = 0; k < nchi2; k++)
	{
	  tmp[k] = cl2->familly->chilst[k];

	}
      if (cl2->familly->dchilst > 0 && cl2->familly->chilst != NULL)
	absfree (cl2->familly->chilst, "cl1isfcl2:cl2->familly->chilst ");
      cl2->familly->chilst = tmp;
      cl2->familly->dchilst = 2 * nchi2 + 2;
    }

  for (k = 0; k < npar1; k++)
    {

      if (cl2 == cl1->familly->parlst[k])
	return;
    }
  cl1->familly->parlst[npar1] = cl2;
  cl1->familly->npar++;
  cl2->familly->chilst[nchi2] = cl1;
  cl2->familly->nchi++;

  return;
}

pfamilly2 (cl)
     Cl *cl;
{
  int k;
  Cl *cl2;
  if (cl == NULL)
    fprintf (stderr, "pfamilly of nullcl\n");

  if (cl->familly == NULL)
    return;

  printf ("cl %d (r=%d c=%d)\n", cl->objn, cl->r, cl->c);

  for (k = 0; k < cl->familly->nchi; k++)
    {
      cl2 = cl->familly->chilst[k];
      if (cl2)
	printf ("   - child %d: %d (r=%d c=%d)\n", k, cl2->objn, cl2->r, cl2->c);
      else
	printf ("   - child %d: NULL\n", k);
    }

  for (k = 0; k < cl->familly->npar; k++)
    {
      cl2 = cl->familly->parlst[k];
      if (cl2)
	printf ("   - parent %d: %d (r=%d c=%d)\n", k, cl2->objn, cl2->r, cl2->c);
      else
	printf ("   - parent %d: NULL\n", k);

    }

}

cl_isnew (cl)
     Cl *cl;
{
  int k;
  Cl *i2j2;
  int l, found;

  if (updating || parsing)
    return;

  if (cl->familly == NULL)
    return;

  for (k = 0; k < cl->familly->npar; k++)
    {
      i2j2 = cl->familly->parlst[k];

      found = 0;
      for (l = 0; l < i2j2->familly->nchi; l++)
	{
	  if (i2j2->familly->chilst[l] == cl)
	    found = 1;
	  if (found)
	    {
	      i2j2->familly->chilst[l] = i2j2->familly->chilst[l + 1];
	    }
	}
      i2j2->familly->nchi--;
    }

  if (cl->familly->npar > 0)
    {
      cl->familly->npar = 0;
      absfree (cl->familly->parlst, "cl_isnew:cl->familly->parlst ");
      cl->familly->parlst = NULL;
      cl->familly->dparlst = 0;
    }
  cl->familly->npar = 0;

  if (cl->familly->nchi == 0)
    {
      if (cl->familly->dchilst > 0)
	absfree (cl->familly->chilst, "cl_isnew:cl->familly->chilst ");
      absfree (cl->familly, "cl_isnew:cl->familly ");
      cl->familly = NULL;
    }

}

int
nupdate2 (cl)
     Cl *cl;
{
  int k;
  Cl *cl2;
  char buf[1024];
  char *t;
  if (parsing)
    return;

  if (cl == NULL)
    return;

  if (cl->familly == NULL)
    return;

  updating++;

  for (k = 0; k < cl->familly->nchi; k++)
    {
      cl2 = cl->familly->chilst[k];
      if (cl2->form != NULL)
	{
	  sprintf (buf, "%s", cl2->form);
	  cl_setform (cl2, buf);

	  toredrw = (Cl **) absrealloc (toredrw, sizeof (Cl *) * (ntoredrw + 1), "nupdate2:toredrw ");
	  toredrw[ntoredrw] = cl2;
	  ntoredrw++;
	}
    }
  updating--;

  if (updating == 0 && ntoredrw > 0)
    {
      for (k = 0; k < ntoredrw; k++)
	xdrwcl (toredrw[k]->r, toredrw[k]->c, 0);
      absfree (toredrw, "nupdate2:toredrw ");
      toredrw = NULL;
      ntoredrw = 0;
    }
}

macroparsing (buf)
     char *buf;
{
  setcl2parse (NULL);
  strcpy (currentString, buf);
  strcat (currentString, "\n");
  currentpos = 0;
  setmacro ();
  parsingmacro = 1;
  freenewstring ();
  yyparse ();
  freenewstring ();
  unsetmacro ();
  parsingmacro = 0;
  if (ntosetform > 0)
    {
      while (ntosetform > 0)
	{
	  ntosetform--;
	  cl_setform (tosetform[ntosetform], formtoset[ntosetform]);
	  absfree (formtoset[ntosetform], "macroparsing:formtoset[ntosetform] ");
	}
      absfree (formtoset, "macroparsing:formtoset ");
      absfree (tosetform, "macroparsing:tosetform ");
      formtoset = NULL;
      tosetform = NULL;
    }

}

extern char *obj2string (obj o);
extern obj getidval3 (int i);
extern int findid (char *s);

char *
textparsing (buf)
     char *buf;
{
  Cl *cl;
  Cl *activcl = ActivCl;
  if (parsingmacro)
    {
      sprintf (tmpbuf, "to update");
      return tmpbuf;
    }
  cl = newcl (0, 1);

  cl_setform (cl, buf);
  strcpy (tmpbuf, cl_gtext (cl));

  cl_isnew (cl);
  freecl (cl);
  ActivCl = activcl;
  return tmpbuf;
}

Cl *
cl_cpy (cl1, cl2)
     Cl *cl1;
     Cl *cl2;
{
  int len;

  if (cl1 == NULL || cl2 == NULL)
    return NULL;

  if (cl2->val.type == FLOATING_CONSTANT)
    {
      cl1->val.rec.d = cl2->val.rec.d;
      cl1->val.type = FLOATING_CONSTANT;
    }

  if (cl2->form != NULL)
    {
      len = strlen (cl2->form);
      if (len > 0)
	{
	  cl1->form = (char *) absmalloc (sizeof (char) * len, "cl_cpy:cl1->form ");
	  if (cl1->form == NULL)
	    fprintf (stderr, "cl1->form  not allocated!\n");
	  strcpy (cl1->form, cl2->form);
	}
    }
  cl1->flag1 = cl2->flag1;

  if (cl2->style != NULL)
    {
      if (cl1->style == NULL)
	cl1->style = newstyle (STY_CELL);
      style_cpy (cl1->style, cl2->style);
    }
  else
    {
      if (cl1->style != NULL)
	{
	  freestyle (cl1->style);
	  cl1->style = NULL;
	}
    }
  return cl1;
}

Cl *
cl_stycpy (cl1, cl2)
     Cl *cl1;
     Cl *cl2;
{
  int len;

  if (cl1 == NULL || cl2 == NULL)
    return NULL;

  if (cl2->style != NULL)
    {
      if (cl1->style == NULL)
	cl1->style = newstyle (STY_CELL);
      style_cpy (cl1->style, cl2->style);
    }
  else
    {
      if (cl1->style != NULL)
	{
	  freestyle (cl1->style);
	  cl1->style = NULL;
	}
    }
  return cl1;
}
