/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "drw.h"
#include "math.h"

Drwing *ActivDrwing = NULL;

Drwing *
newdrwing ()
{

  Drwing *drwing;
  drwing = (Drwing *) absmalloc (sizeof (Drwing), "newdrwing:drwing ");
  return drwing;

}

Drwing *
newdrwing1 (dr)
     Drwing *dr;
{

  Drwing *drwing;
  drwing = (Drwing *) absmalloc (sizeof (Drwing), "newdrwing:drwing ");
  drwing->x1 = dr->x1;
  drwing->y1 = dr->y1;
  drwing->x2 = dr->x2;
  drwing->y2 = dr->y2;
  drwing->type = dr->type;
  drwing->style = NULL;
  return drwing;

}

int
drwing_copy (drwing, dr)
     Drwing *drwing;
     Drwing *dr;
{

  drwing->x1 = dr->x1;
  drwing->y1 = dr->y1;
  drwing->x2 = dr->x2;
  drwing->y2 = dr->y2;
  drwing->type = dr->type;
  drwing->style = NULL;

}

int
freedrwing (dr)
     Drwing *dr;
{
  if (dr != NULL);
  absfree (dr, "freedrwing:drwing ");
}

Drwing *
drwing_r (fp)
     FILE *fp;
{
  Drwing *dr = newdrwing ();
  fscanf (fp, "%d %d %d %d %d",
	  &(dr->type), &(dr->x1), &(dr->y1), &(dr->x2), &(dr->y2));

  return dr;

}

int
drwing_w (dr, fp)
     Drwing *dr;
     FILE *fp;
{
  int j;
  char *type;

  if (dr->type == 0)
    {
      fprintf (fp, "ActiveSheet.Shapes.AddLine(%d, %d,%d,%d)\n", dr->x1, dr->y1, dr->x2, dr->y2);
      return;
    }
  switch (dr->type)
    {
    case 25:
      type = "msoShapeArc";
      break;
    case 1:
      type = "msoShapeRectangle";
      break;
    case 9:
      type = "msoShapeOval";
      break;
    default:
      type = "msoShapeRectangle";
      break;
    }
  fprintf (fp, "ActiveSheet.Shapes.AddShape(%s,%d, %d,%d,%d)\n", type, dr->x1, dr->y1, dr->x2, dr->y2);
  return;

}

int
ActivateDrwing (drwing)
     Drwing *drwing;
{
  ActivDrwing = drwing;
}

int
drwing_pnt (dr, x, y, dx, dy)
     Drwing *dr;
     int x, y, dx, dy;
{
  int mx, my;
  int x1, x2, y1, y2;

  x1 = dr->x1;
  x2 = dr->x2;
  y1 = dr->y1;
  y2 = dr->y2;

  mx = (x1 + x2) / 2;
  my = (y1 + y2) / 2;
  switch (dr->type)
    {
    case 0:
      {
	if (abs (x1 - x) <= dx && abs (y1 - y) <= dy ||
	    abs (x2 - x) <= dx && abs (y2 - y) <= dy)
	  return 1;
      }
      break;
    case 1:
      {
	if (
	   ((x1 - dx < x && x < x1 + dx) && (y1 - dy < y && y < y2 + dy)) ||
	   ((x2 - dx < x && x < x2 + dx) && (y1 - dy < y && y < y2 + dy)) ||
	   ((y1 - dy < y && y < y1 + dy) && (x1 - dx < x && x < x2 + dx)) ||
	     ((y2 - dy < y && y < y2 + dy) && (x1 - dx < x && x < x2 + dx))
	  )
	  return 1;
	break;
      }
    case 2:
      {
	if (fabs (x1 - x) <= dx && fabs (y1 - y) <= dy ||
	    fabs (x2 - x) <= dx && fabs (y2 - y) <= dy)
	  return 1;
      }
      break;
    case 3:
      {
	if (fabs (x1 - x) <= dx && fabs (y1 - y) <= dy ||
	    fabs (x2 - x) <= dx && fabs (y2 - y) <= dy)
	  return 1;
      }
      break;
    default:
      {
	if (fabs (x1 - x) <= dx && fabs (y1 - y) <= dy ||
	    fabs (x2 - x) <= dx && fabs (y2 - y) <= dy)
	  return 1;
      }
    }

  return 0;
}
