/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "info_dialog.h"

static InfoDialog AbsInfoDialog;
static int absinfodialogexist = 0;

#define RET_NONE 0
#define RET_OK 1

void
InfoCancelAction (widget, pointer, junk)
     Widget widget;
     XtPointer pointer;
     XtPointer junk;
{

  int *answer = (int *) pointer;
  *answer = RET_OK;

}

void
InfoAcceptAction (widget, pointer, junk)
     Widget widget;
     XtPointer pointer;
     XtPointer junk;
{

  int *answer = (int *) pointer;
  *answer = RET_OK;

}

static int answer;

Inform (message, func)
     char *message;
     void (*func) ();
{
  Arg arglist[1];
  XtAppContext context;
  answer = RET_NONE;

  if (!absinfodialogexist)
    CreateInfoDialog ((Widget) gettop (), func);

  TextPrintf (AbsInfoDialog.Message, message);
  XtUnmanageChild (AbsInfoDialog.cancel);

  InfoDisplay ((Widget) gettop ());

  context = XtWidgetToApplicationContext ((Widget) gettop ());
  while (answer == RET_NONE || XtAppPending (context))
    {
      XtAppProcessEvent (context, XtIMAll);
    }

  XtSetKeyboardFocus (AbsInfoDialog.parent, (Widget) None);
  XtUnmanageChild (AbsInfoDialog.Shell);
  return answer;
};

Ask (message, func)
     char *message;
     void (*func) ();
{
  Arg arglist[1];

  if (!absinfodialogexist)
    CreateInfoDialog ((Widget) gettop (), func);

  XtSetArg (arglist[0], XtNlabel, message);
  XtSetValues (AbsInfoDialog.Message, arglist, 1);

  InfoDisplay ((Widget) gettop ());
};

void
CreateInfoDialog (parent, func)
     Widget parent;
     void (*func) ();
{
  Arg args[20];
  int n, i;
  Widget ok, box;

  AbsInfoDialog.parent = parent;

  n = 0;
  XtSetArg (args[n], XtNinput, True);
  n++;
  XtSetArg (args[n], XtNwidth, 600);
  n++;
  XtSetArg (args[n], XtNheight, 300);
  n++;
  AbsInfoDialog.Shell = XtCreatePopupShell
    ("abs info", transientShellWidgetClass, parent, args, 3);

  n = 0;
  XtSetArg (args[n], XtNwidth, 600);
  n++;
  XtSetArg (args[n], XtNheight, 300);
  n++;
  AbsInfoDialog.Dialog = XtCreateManagedWidget ("absinfodialog", formWidgetClass,
					      AbsInfoDialog.Shell, args, n);

  n = 0;
  XtSetArg (args[n], XtNwidth, 600);
  n++;
  XtSetArg (args[n], XtNheight, 1);
  n++;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  box = XtCreateManagedWidget ("InfoDialog", formWidgetClass,
			       AbsInfoDialog.Dialog, args, n);
  n = 0;
  XtSetArg (args[n], XtNwidth, 1);
  n++;
  XtSetArg (args[n], XtNheight, 300);
  n++;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  box = XtCreateManagedWidget ("InfoDialog", formWidgetClass,
			       AbsInfoDialog.Dialog, args, n);

  AbsInfoDialog.Message = CreateText (AbsInfoDialog.Dialog, "Info", 10, 10, 580, 250);

  w_n = 0;
  w_dim (65, 20);
  w_rel (NULL, 70, 270);
  ok = XtCreateManagedWidget
    ("Ok", commandWidgetClass, AbsInfoDialog.Dialog, w_args, w_n);
  XtAddCallback (ok, XtNcallback, InfoAcceptAction, &answer);

  w_n = 0;
  w_dim (65, 20);
  w_rel (NULL, 190, 270);
  AbsInfoDialog.cancel = XtCreateManagedWidget
    ("Cancel", commandWidgetClass, AbsInfoDialog.Dialog, w_args, w_n);
  XtAddCallback (AbsInfoDialog.cancel, XtNcallback, InfoCancelAction, NULL);

  XtRealizeWidget (AbsInfoDialog.Shell);

  AbsInfoDialog.function = func;
  absinfodialogexist = 1;
}

void
InfoDisplay (centerw)
     Widget centerw;
{
  Arg valueArgs[2];
  Arg centerArgs[2];
  Arg args[20];
  int n, i;
  Position source_x, source_y;
  Position dest_x, dest_y;
  Dimension center_width, center_height;
  Dimension Info_width, Info_height;

  XtSetArg (centerArgs[0], XtNwidth, &center_width);
  XtSetArg (centerArgs[1], XtNheight, &center_height);
  XtGetValues (centerw, centerArgs, 2);
  XtSetArg (centerArgs[0], XtNwidth, &Info_width);
  XtSetArg (centerArgs[1], XtNheight, &Info_height);
  XtGetValues (AbsInfoDialog.Shell, centerArgs, 2);
  source_x = (int) (center_width - Info_width) / 2;
  source_y = (int) (center_height - Info_height) / 3;
  XtTranslateCoords (centerw, source_x, source_y, &dest_x, &dest_y);
  XtSetArg (centerArgs[0], XtNx, dest_x);
  XtSetArg (centerArgs[1], XtNy, dest_y);
  XtSetValues (AbsInfoDialog.Shell, centerArgs, 2);
  XtManageChild (AbsInfoDialog.Shell);

  XtSetKeyboardFocus (AbsInfoDialog.parent, AbsInfoDialog.Shell);

}
