/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "macro.h"

#define Mf Macrofile
#define mf macrofile

#include "absrec.h"

static int macrodebug = 0;
static char *bbuf;
static Mf *Amf;

Macrofile *
newmacrofile ()
{
  Macrofile *nm = (Macrofile *) absmalloc (sizeof (Macrofile), "Macrofile:nm ");
  nm->routines = NULL;
  nm->nroutines = 0;
  nm->in = NULL;
  nm->acl = 0;
  nm->nsub = 0;
  nm->mainid = 0;

  return nm;

}

enterif (value)
     double value;
{
  int i = 0;
  if (macrodebug)
    printf ("enter if val %lf\n", value);

  if (value)
    return;
  routine_gonotif (Amf->in);
}

enterelse ()
{
  int i = 0;
  if (macrodebug)
    printf ("enter else\n");
  routine_goendif (Amf->in);
}

enterloop (id, start, end, step)
     int id;
     double start, end, step;
{
  if (macrodebug)
    printf ("enter loop %s %lf %lf %lf\n", id2str (id), start, end, step);

  routine_enterloop (Amf->in, id, start, end, step, Amf->in->acl);
}

nextloop (id)
     int id;
{
  if (macrodebug)
    printf ("next %s\n", id2str (id));
  routine_nextloop (Amf->in, id);
}

addroutine (id, nparam, paramid)
     int id, nparam;
     int *paramid;
{
  int i;
  if (macrodebug)
    printf ("addroutine %s\n", id2str (id));

  Amf->routines = (Routine **) absrealloc (Amf->routines, sizeof (Routine *) * (Amf->nroutines + 1), "addroutine:Amf->routines ");
  Amf->routines[Amf->nroutines] = (Routine *) newroutine (id, nparam, paramid);
  Amf->in = Amf->routines[Amf->nroutines];
  Amf->nroutines++;

}

addreturn ()
{
  if (Amf->in != NULL)
    {
      if (macrodebug)
	printf ("addreturn %s\n", id2str (Amf->in->subid));
      routine_addreturn (Amf->in);
    }

}

obj
userfct (id, args, nargs)
     int id;
     obj *args;
     int nargs;
{

}

callroutine2 (id, args, nargs)
     int id;
     obj *args;
     int nargs;
{
  int j;
  int i = 0;
  char *first;
  char *end;

  while (Amf->routines[i]->subid != id && i < Amf->nroutines)
    i++;
  if (i >= Amf->nroutines)
    return;
  if (macrodebug)
    printf ("callroutine %d id %d nargs %d args:\n", i, id, nargs);

  for (j = 0; j < Amf->routines[i]->subnparam && j < nargs; j++)
    {
      setidval3 (Amf->routines[i]->subparamid[j], args[j]);
    }

  if (Amf->in != NULL)
    {
      Amf->routines[i]->calling = Amf->in;
      Amf->in->acl++;
    }
  else
    Amf->routines[i]->calling = NULL;

  Amf->in = Amf->routines[i];

  if (macrodebug)
    routine_print (Amf->in);

  Amf->in->acl = 0;
}

callroutine (id, args, nargs)
     int id;
     obj *args;
     int nargs;
{
  int j;
  int i = 0;
  char *first;
  char *end;
  char info[32];
  int found = 0;

  while (i < Amf->nroutines && !found)
    {
      if (Amf->routines[i]->subid == id)
	found = 1;
      else
	i++;
    }
  if (i >= Amf->nroutines)
    return;
  if (macrodebug)
    printf ("callroutine %d id %d nargs %d args:\n", i, id, nargs);

  for (j = 0; j < Amf->routines[i]->subnparam && j < nargs; j++)
    {
      setidval3 (Amf->routines[i]->subparamid[j], args[j]);

    }

  if (Amf->in != NULL)
    Amf->routines[i]->calling = Amf->in;
  else
    Amf->routines[i]->calling = NULL;

  Amf->in = Amf->routines[i];

  if (macrodebug)
    routine_print (Amf->in);

  Amf->in->acl = 1;
  while (Amf->in->acl < Amf->in->nlc &&
	 Amf->in->acl <= Amf->in->endl)
    {
      if (macrodebug)
	printf ("line %d:%s\n", Amf->in->acl, Amf->in->sourcecode[Amf->in->acl]);
      if (Amf->in->subid == Amf->mainid)
	{
	  sprintf (info, "line %d of %d", Amf->in->acl, Amf->in->endl);
	  inform (info);
	}

      bbuf = Amf->in->sourcecode[Amf->in->acl];

      first = bbuf;
      end = bbuf + strlen (bbuf);
      while (first[0] == ' ' && first < end)
	first++;
      if (first[0] != '\'' && first[0] != '\n')
	{
	  macroparsing (Amf->in->sourcecode[Amf->in->acl]);
	}
      Amf->in->acl = 1 + Amf->in->acl;

      if (Amf->in->acl == 1 + Amf->in->endl && Amf->in->calling != NULL)
	{
	  Amf->in = Amf->in->calling;
	}
    }

  Amf->in = NULL;

  return;
}

error_in_macro ()
{

  char *end;
  end = (char *) strchr (bbuf, '\n');
  if (end != NULL)
    end[0] = '\0';
  if (macrodebug)
    printf ("error executing line (%s)\n", bbuf);
}

addlinecode (mf, buf)
     Mf *mf;
     char *buf;
{
  char *first;
  char *end;

  if (mf == NULL)
    return;

  if (macrodebug)
    {
      printf ("addline %s\n", buf);

    }
  Amf = mf;

  if (buf == NULL)
    return;
  if (strlen (buf) < 1)
    return;

  if (strstr (buf, "DEBUG_ON") != 0)
    {
      macrodebug = 1;
      return;
    }
  if (strstr (buf, "DEBUG_OFF") != 0)
    {
      macrodebug = 0;
      return;
    }

  first = buf;
  end = buf + strlen (buf);
  while (first[0] == ' ' && first < end)
    first++;

  if (first[0] == '\'' || first[0] == '\n')
    {
      if (Amf->in != NULL)
	routine_addlinecode (Amf->in, buf);
      return;
    }

  if (strstr (buf, "Sub") != 0 && strstr (buf, "End") == 0)
    {
      if (macrodebug)
	printf ("new routine %s", buf);
      bbuf = buf;
      macroparsing (buf);
    }

  if (strstr (buf, "Sub") != 0 && strstr (buf, "End") != 0)
    {
      addreturn ();
      routine_addlinecode (Amf->in, buf);
      Amf->in = NULL;
      return;
    }

  routine_addlinecode (Amf->in, buf);

  if (strstr (buf, "If") != 0)
    {
      routine_addif (Amf->in);
    }
  if (strstr (buf, "Else") != 0)
    {
      routine_addelse (Amf->in);
    }
  if (strstr (buf, "Endif") != 0)
    {
      routine_addendif (Amf->in);
    }

}

runcode (mf)
     Mf *mf;
{
  Amf = mf;

  if (macrodebug)
    printf ("----run start------\n");

  Amf->mainid = findid ("MAIN");

  if (Amf->mainid < 0)
    return -1;

  Amf->in = NULL;

  callroutine (Amf->mainid, NULL, 0);

}

freecode (mf)
     Mf *mf;
{
  int i;
  for (i = 0; i < mf->nroutines; i++)
    routine_freecode (mf->routines[i]);
  mf->nroutines = 0;
  return;
}

int
getnmacro (mf)
     Mf *mf;
{
  return mf->nroutines;
}

char *
getmacroname (mf, v)
     Mf *mf;
     int v;
{
  return (char *) id2str (mf->routines[v]->subid);
}

void
playmacro (mf, pos)
     Mf *mf;
     int pos;
{
  int i;
  Amf = mf;
  if (pos < 0 || pos > Amf->nroutines)
    return;

  callroutine (Amf->routines[pos]->subid, NULL, 0);

  xrepnt ();
};

void
editmacro (mf, pos)
     Mf *mf;
     int pos;
{
  FILE *tmpmac;
  int i;
  int plugnum;
  int end = 1;
  char buf[256];
  char line[258];
  Routine *cursub;

  Amf = mf;

  tmpmac = fopen ("abstmpmac", "w");

  cursub = mf->routines[pos];

  for (i = 0; i < cursub->nlc; i++)
    fprintf (tmpmac, "%s", cursub->sourcecode[i]);

  fclose (tmpmac);

  system ("xedit abstmpmac");

  routine_delete (cursub);
  mf->nroutines--;
  for (i = pos; i < mf->nroutines; i++)
    {
      mf->routines[i] = mf->routines[i + 1];
    }

  mf->in = NULL;

  tmpmac = fopen ("abstmpmac", "r");
  while (fgets (line, 256, tmpmac))
    {
      addlinecode (mf, line);
    }

  fclose (tmpmac);

};

void
macrosave (mf, fp)
     Mf *mf;
     FILE *fp;
{
  int i, j;

  for (i = 0; i < mf->nroutines; i++)
    {

      if (mf->routines[i]->subid != mf->mainid)
	{
	  for (j = 0; j < mf->routines[i]->nlc; j++)
	    if (mf->routines[i]->sourcecode[j] != NULL)
	      fprintf (fp, "%s", mf->routines[i]->sourcecode[j]);
	  fprintf (fp, "\n");
	}
    }
}
