/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include <stdio.h>
#include <sys/stat.h>
#include <string.h>

#include "style.h"
#include "cl.h"
#include "wks.h"
#include "drw.h"
#include "chrt.h"
#include "y.tab.h"

#include "main.h"

int
incr ()
{
  scanf ("%d", &copy_incr);
}

static Cl *brushref = NULL;

int
setbrsh (Cl * cl)
{
  brushref = cl;
  brushing = 1;
}

int
makebrush ()
{
  int i, j;

  if (ActivWks == NULL)
    return;

  if (si < 1 || sj < 1 || li > ActivWks->nlin || lj > ActivWks->ncol)
    {
      return;
    }

  for (i = si; i <= li; i++)
    for (j = sj; j <= lj; j++)
      cl_stycpy ((Cl *) appliccl (i, j), brushref);
  xrepnt ();
  brushref = NULL;
  brushing = 0;
}

int
makesorta ()
{
  int i, j;

  Cl **tosort;
  int k = 0;
  char buf[256];
  int istext = 0;

  if (ActivWks == NULL)
    return;

  if (si < 1 || sj < 1 || li > ActivWks->nlin || lj > ActivWks->ncol)
    {
      return;
    }

  tosort = (Cl **) absmalloc (sizeof (Cl *) * (lj - sj + 1) * (li - si + 1), "makesorta:tosort");
  k = 0;

  for (i = si; i <= li; i++)
    for (j = sj; j <= lj; j++)
      {
	tosort[k] = (Cl *) appliccl (i, j);
	if (tosort[k]->flag1)
	  istext = 1;
	k++;
      }

  if (k < 2)
    return;

  if (istext)
    for (i = 0; i < k; i++)
      {
	for (j = i + 1; j < k; j++)
	  {
	    if (strcmp (tosort[i]->val.rec.s, tosort[j]->val.rec.s) > 0)
	      {
		strcpy (buf, tosort[i]->form);
		cl_setform (tosort[i], tosort[j]->form);
		cl_setform (tosort[j], buf);
	      }
	  }
      }
  else
    for (i = 0; i < k; i++)
      {
	for (j = i + 1; j < k; j++)
	  {
	    if (tosort[i]->val.rec.d > tosort[j]->val.rec.d)
	      {
		strcpy (buf, tosort[i]->form);
		cl_setform (tosort[i], tosort[j]->form);
		cl_setform (tosort[j], buf);
	      }
	  }
      }

  xrepnt ();

}
int
makesortd ()
{
  int i, j;

  Cl **tosort;
  int k = 0;
  char buf[256];
  int istext = 0;

  if (ActivWks == NULL)
    return;

  if (si < 1 || sj < 1 || li > ActivWks->nlin || lj > ActivWks->ncol)
    {
      return;
    }

  tosort = (Cl **) absmalloc (sizeof (Cl *) * (lj - sj + 1) * (li - si + 1), "makesorta:tosort");
  k = 0;

  for (i = si; i <= li; i++)
    for (j = sj; j <= lj; j++)
      {
	tosort[k] = (Cl *) appliccl (i, j);
	if (tosort[k]->flag1)
	  istext = 1;
	k++;
      }

  if (k < 2)
    return;

  if (istext)
    for (i = 0; i < k; i++)
      {
	for (j = i + 1; j < k; j++)
	  {
	    if (strcmp (tosort[i]->val.rec.s, tosort[j]->val.rec.s) < 0)
	      {
		strcpy (buf, tosort[i]->form);
		cl_setform (tosort[i], tosort[j]->form);
		cl_setform (tosort[j], buf);
	      }
	  }
      }
  else
    for (i = 0; i < k; i++)
      {
	for (j = i + 1; j < k; j++)
	  {
	    if (tosort[i]->val.rec.d < tosort[j]->val.rec.d)
	      {
		strcpy (buf, tosort[i]->form);
		cl_setform (tosort[i], tosort[j]->form);
		cl_setform (tosort[j], buf);
	      }
	  }
      }

  xrepnt ();

}

int
addd2matrix (v)
     double v;
{
  char tmp[256];
  sprintf (tmp, "%lf\0", v);
  set_form (curi, curj, tmp);
  curj++;
  nj++;
  if (curj > ActivWks->ncol)
    {
      curj = ActivWks->ncol;
      nj--;
    }
  return 0;
}

int
addi2matrix (v)
     int v;
{
  char tmp[16];
  sprintf (tmp, "%d\0", v);
  set_form (curi, curj, tmp);
  set_format (curi, curj, 1, 0);
  curj++;
  nj++;
  if (curj > ActivWks->ncol)
    {
      curj = ActivWks->ncol;
      nj--;
    }
  return 0;
}

int
add2matrix (word)
     char *word;
{
  set_form (curi, curj, word);

  curj++;
  nj++;
  if (curj > ActivWks->ncol)
    {
      curj = ActivWks->ncol;
      nj--;
    }
  return 0;
}

int
nextline ()
{
  curi++;
  curj -= nj;
  nj = 0;
  if (curi > ActivWks->nlin)
    curi = ActivWks->nlin;
  return 0;
}

void
insertfl (filename)
     char *filename;
{
  int i, j, maxi, maxj, jj, k;
  struct stat statbuf;
  int file_length;
  unsigned char *file_string;
  char line[1024];
  char word[1024];
  char *token;
  char *tab;
  char *first;
  int plugin;
  int nrow, ncol;
  FILE *fp = NULL;

  double val;
  i = 1;
  j = 1;

  fp = fopen (filename, "r");
  if(fp ==NULL) return;
  sprintf (word, "");

  if (curi < 0)
    curi = 1;
  if (curj < 0)
    curj = 1;
  while (fgets (line, 1020, fp) != NULL)
    {
      k = 0;
      first = line; 
      token = strtok (first, "	");
      while (token != NULL)
	{
	  strcpy (word, token);
          tab = strrchr(word,'	');
          if (tab!=NULL) *tab = '\n';
          else
          strcat(word,"\n");
	  if (strlen (word) > 1)
	    { 
	      set_form (curi + i - 1, curj + j - 1, word);
	    }
	  sprintf (word, "");
	  token = strtok (NULL, "	");
	  j++;
	}

      if (j > ncol)
	ncol = j;
      nrow++;
      j = 1;
      i++;
    }
  fclose (fp);

}

void
exportfile (filename)
     char *filename;
{
  int i, j;
  FILE *fp = NULL;

 if (ActivWks == NULL) return;

 if (si < 1 || sj < 1 || li > ActivWks->nlin || lj > ActivWks->ncol)
    {
      si = 1;
      sj = 1;
      li = ActivWks->mrow;
      lj = 0;
      for (i = 0; i < ActivWks->mrow; i++)
        {
          if (ActivWks->mcol[i] > lj)
            lj = ActivWks->mcol[i];
        }
     }


  fp = fopen (filename, "w");
  {
    for (i = si; i <= li; i++)
      {
	if (i > si)
	  fprintf (fp, "\n");
	for (j = sj; j <= lj; j++)
	  {
	    if (j > sj)
	      fprintf (fp, "	");
            if (get_text (i,j) !=NULL)
	    fprintf (fp, "%s", get_text (i, j));
            else
            fprintf (fp, "0", get_text (i, j));
	  }
      }
  }
  fclose (fp);

}

void
m_exit ()
{
  exit (0);
}

int
copy ()
{
  if (ActivWks == NULL)
    return;

  if (si < 1 || sj < 1 || li > ActivWks->nlin || lj > ActivWks->ncol)
    {
      return;
    }

  copy_applic (si, sj, li, lj);
}

int
paste ()
{
  int i, j;

  if (ActivWks == NULL)
    return;

  if (si < 1 || sj < 1 || li > ActivWks->nlin || lj > ActivWks->ncol)
    {
      return;
    }

  paste_applic (si, sj, li, lj);

  for (i = si; i <= li; i++)
    {
      for (j = sj; j <= lj; j++)
	{
	  xdrwcl (i, j, 4);
	}
    }
  cpypix ();
  return 0;

}

int
cut ()
{
  int i, j;
  int i0, j0;

  if (ActivWks == NULL)
    return;

  if (si < 1 || sj < 1 || li > ActivWks->nlin || lj > ActivWks->ncol)
    {
      return;
    }

  cut_applic (si, sj, li, lj);
  for (i = si; i <= li; i++)
    {
      for (j = sj; j <= lj; j++)
	{
	  xdrwcl (i, j, 4);
	}
    }
  cpypix ();
  return 0;
}

int
makeformat (type, decimal)
     int type, decimal;
{
  int i, j;
  int i0, j0;
  Cl *cl;

  if (type < 1 || type > 5)
    return;

  if (sj > 0 && si > 0 && lj <= ActivWks->ncol && li <= ActivWks->nlin)
    for (i = 0; i <= lj - sj; i++)
      {
	for (j = 0; j <= li - si; j++)
	  {
	    cl = (Cl *) appliccl (si + j, sj + i, 1);
	    cl_setformat (cl, type, decimal);
	    xdrwcl (si + j, sj + i, 4);
	  }
      }
  cpypix ();
  return 0;
}

int
makejust (just)
     int just;
{
  int i, j;
  int i0, j0;
  Cl *cl;

  if (sj > 0 && si > 0 && lj <= ActivWks->ncol && li <= ActivWks->nlin)
    for (i = 0; i <= lj - sj; i++)
      {
	for (j = 0; j <= li - si; j++)
	  {
	    cl = (Cl *) appliccl (si + j, sj + i, 1);
	    cl_setjust (cl, just);
	    xdrwcl (si + j, sj + i, 4);
	  }
      }
  cpypix ();
  return 0;
}

int
makefont (fontnum, fontweight, fontsize)
     int fontnum, fontweight, fontsize;
{
  int i, j;
  int i0, j0;
  Cl *cl;

  if (sj > 0 && si > 0 && lj <= ActivWks->ncol && li <= ActivWks->nlin)
    for (i = 0; i <= lj - sj; i++)
      {
	for (j = 0; j <= li - si; j++)
	  {
	    cl = (Cl *) appliccl (si + j, sj + i, 1);
	    cl_setfont (cl, fontnum, fontweight, fontsize);
	    xdrwcl (si + j, sj + i, 4);
	  }
      }
  cpypix ();
  return 0;
}

int
border (border, type)
     int border;
     int type;
{
  int i, j;
  int i0, j0;
  Cl *cl;

  for (i = 0; i <= lj - sj; i++)
    {
      for (j = 0; j <= li - si; j++)
	{
	  cl = (Cl *) appliccl (si + j, sj + i, 1);
	  cl_setbord (cl, border, type);
	  xdrwcl (si + j, sj + i, 4);
	}
    }

  cpypix ();
  return 0;
}

int
fit ()
{
  int i, j;
  int i0, j0;
  double f;

  if (sj < 1 || si < 1 || lj > ActivWks->ncol || li > ActivWks->nlin) return;

  for (i = 0; i <= lj - sj; i++)
    {
      for (j = 0; j <= li - si; j++)
	{
	  xfit (si + j, sj + i, get_text (si + j, sj + i));
	}
    }
  return 0;
}

void
formatcolumn ()
{
  int i, j;

  if (sj < 1 || si < 1 || lj > ActivWks->ncol || li > ActivWks->nlin) return;
  for (i = 0; i <= lj - sj; i++)
    {
      for (j = 0; j <= li - si; j++)
	{
	  xfitcolumn (si + j, sj + i);
	}
    }
  xrepnt ();
  return;
}

void
formatline ()
{
  int i, j;

  if (sj < 1 || si < 1 || lj > ActivWks->ncol || li > ActivWks->nlin) return;
  for (i = 0; i <= lj - sj; i++)
    {
      for (j = 0; j <= li - si; j++)
	{
	  xfitline (si + j, sj + i);
	}
    }
  xrepnt ();
  return;
}

int
color ()
{
  makecolor (1, 0);
}

int
makecolor (fg, bg)
     int fg, bg;
{
  int i, j;
  int i0, j0;
  double f;
  Cl *cl;

  if (sj < 1 || si < 1 || lj > ActivWks->ncol || li > ActivWks->nlin) return;
  for (i = 0; i <= lj - sj; i++)
    {
      for (j = 0; j <= li - si; j++)
	{
	  cl = (Cl *) appliccl (si + j, sj + i, 1);
	  cl_setfg (cl, fg);
	  cl_setbg (cl, bg);
	  xdrwcl (si + j, sj + i, 4);
	}
    }
  cpypix ();
  return 0;
}

int
cpyd ()
{
  int i, j;
  int i0, j0;

  if (sj < 1 || si < 1 || lj > ActivWks->ncol || li > ActivWks->nlin) return;
  for (i = 0; i <= lj - sj; i++)
    {
      for (j = 1; j <= li - si; j++)
	{
	  copymod (si, sj + i, si + j, sj + i, copy_incr);
	}
    }
  for (i = 0; i <= lj - sj; i++)
    {
      for (j = 0; j <= li - si; j++)
	{
	  xdrwcl (si, sj + i, 4);
	  xdrwcl (si + j, sj + i, 4);
	}
    }

  cpypix ();
  return 0;
}

int
cpyr ()
{
  int i, j;
  int i0, j0;

  if (sj < 1 || si < 1 || lj > ActivWks->ncol || li > ActivWks->nlin) return;
  for (i = 1; i <= lj - sj; i++)
    {
      for (j = 0; j <= li - si; j++)
	{
	  copymod (si + j, sj, si + j, sj + i, copy_incr);
	}
    }

  for (i = 0; i <= lj - sj; i++)
    {
      for (j = 0; j <= li - si; j++)
	{
	  xdrwcl (si + j, sj, 4);
	  xdrwcl (si + j, sj + i, 4);
	}
    }

  cpypix ();
  return 0;
}

int
rstsel (ii, jj, k)
     int ii, jj;
{
  int i, j;
  char label[2];
  sprintf (label, "");
  inform (label);

  if (ii == 0 && jj == 0 && k == 0)
    {
      si = li = 0;
      sj = lj = 0;
      high = 0;
      selectingto0 ();
      cpypix ();
      return 0;
    }
  if (ii >= si && ii <= li && jj >= sj && jj <= lj && !k)
    return 1;

  si = li = 0;
  sj = lj = 0;
  high = 0;
  selectingto0 ();
  cpypix ();
  return 0;
}

int
setselection (i1, j1, i2, j2)
     int i1, j1, i2, j2;
{
  int t;
  char label[16];
  if (i1 > i2)
    {
      t = i1;
      i1 = i2;
      i2 = t;
    }
  if (j1 > j2)
    {
      t = j1;
      j1 = j2;
      j2 = t;
    }

  si = i1;
  sj = j1;
  li = i2;
  lj = j2;
  high = 1;

  sprintf (label, "%d L x %d C\n", li - si + 1, lj - sj + 1);
  inform (label);

  cpypix ();
  return 0;

}

int
selectlin (lin)
     int lin;
{
  int i, j;
  char label[16];
  if (high && si == lin)
    {
      high = 0;
    }
  else
    {
      si = li = lin;
      sj = 1;
      lj = ActivWks->ncol - 1;
      high = 1;
      sprintf (label, "%d L x %d C\n", li - si + 1, lj - sj + 1);
      inform (label);

    }
  cpypix ();
  return 0;
}

int
selectcol (col)
     int col;
{
  int i, j;
  char label[16];
  if (high && sj == col)
    {
      high = 0;
    }
  else
    {
      si = 1;
      li = ActivWks->nlin - 1;
      sj = lj = col;
      high = 1;
      sprintf (label, "%d L x %d C\n", li - si + 1, lj - sj + 1);
      inform (label);
    }

  cpypix ();
  return 0;
}

int
selection (ii, jj)
     int ii, jj;
{
  int i, j;
  char label[16];

  if (lj == jj && li == ii)
    return 0;
  if (high == 0)
    {
      sj = lj = jj;
      si = li = ii;
    }
  lj = jj;
  li = ii;
  high = 1;

  sprintf (label, "%d L x %d C\n", li - si + 1, lj - sj + 1);
  inform (label);

  cpypix ();
  return 0;

};

mvsel (fri, frj, toi, toj, k)
     int fri, frj, toi, toj, k;
{
  int i, j;

  if (high == 0)
    return;

  if (k)
    {

      if (toi - fri >= 0 && toj - frj >= 0)
	{
	  for (i = li; i >= si; i--)
	    {
	      for (j = lj; j >= sj; j--)
		{
		  copymod (i, j, i + toi - fri, j + toj - frj, -1);
		  xdrwcl (i + toi - fri, j + toj - frj, 4);
		}
	    }
	}
      else if (toi - fri <= 0 && toj - frj >= 0)
	{
	  for (i = si; i <= li; i++)
	    {
	      for (j = lj; j >= sj; j--)
		{
		  copymod (i, j, i + toi - fri, j + toj - frj, -1);
		  xdrwcl (i + toi - fri, j + toj - frj, 4);
		}
	    }
	}
      else if (toi - fri >= 0 && toj - frj <= 0)
	{
	  for (i = li; i >= si; i--)
	    {
	      for (j = sj; j <= lj; j++)
		{
		  copymod (i, j, i + toi - fri, j + toj - frj, -1);
		  xdrwcl (i + toi - fri, j + toj - frj, 4);
		}
	    }

	}
      else if (toi - fri <= 0 && toj - frj <= 0)
	{
	  for (i = si; i <= li; i++)
	    {
	      for (j = sj; j <= lj; j++)
		{
		  copymod (i, j, i + toi - fri, j + toj - frj, -1);
		  xdrwcl (i + toi - fri, j + toj - frj, 4);
		}
	    }
	}

      for (i = si; i <= li; i++)
	{
	  for (j = sj; j <= lj; j++)
	    {
	      if (!(i >= si + toi - fri && i <= li + toi - fri && j >= sj + toj - frj && j <= lj + toj - frj))
		{
		  set_form (i, j, "");
		  xdrwcl (i, j, 4);
		}
	    }
	}

      si += toi - fri;
      li += toi - fri;
      sj += toj - frj;
      lj += toj - frj;
      cpypix ();
    }
  else
    {
      si += toi - fri;
      li += toi - fri;
      sj += toj - frj;
      lj += toj - frj;
      cpypix ();
      si -= toi - fri;
      li -= toi - fri;
      sj -= toj - frj;
      lj -= toj - frj;

    }

}

myfprintf (fp, word, l)
     FILE *fp;
     char *word;
     int l;
{
  int i;
  if (strcmp (word, "\n") == 0)
    return;
  for (i = 0; i <= l - strcspn (word, " "); i++)
    fprintf (fp, " ");
  fprintf (fp, "%s", word);
  return;
}

void
makeprint (destination, destname, format)
     int destination, format;
     char *destname;
{
  if (destination == 0 && format == 1)
    printfl (destname);
  if (destination == 0 && format == 0)
    printascii (destname);
  else
    printf ("not yet\n");
  return;
}

int
printascii (filename)
     char *filename;
{
  FILE *fp;
  char *buf;
  int i, j, maxj, maxi, jj, l;

  fp = fopen (filename, "w");
  for (i = si; i <= li; i++)
    {
      for (j = sj; j <= lj; j++)
	{
	  buf = get_text (i, j);
	  if (strlen (buf) > 0)
	    fprintf (fp, "%s	", buf);
	  else
	    fprintf (fp, "	");
	}
      fprintf (fp, "\n");
    }
  fclose (fp);
}

int
printfl (filename)
     char *filename;
{
  FILE *fp;
  int i, j, maxj, maxi, jj, l;
  double xscale, yscale, h, w, H, W;
  int x, y, x0, y0, xn, yn;
  int x1, y1;
  int xmax, ymax;
  int portrait = 0;
  Chrt *gr;
  Drwing *dr;

  if (ActivWks == NULL)
    return;
  if (ActivWks->mrow < 1)
    return;

  if (si < 1 || sj < 1 || li > ActivWks->nlin || lj > ActivWks->ncol)
    {
      si = 1;
      sj = 1;
      li = ActivWks->mrow;
      lj = 0;
      for (i = 0; i < ActivWks->mrow; i++)
	{
	  if (ActivWks->mcol[i] > lj)
	    lj = ActivWks->mcol[i];
	}

      absijtoxy (si, sj, &x0, &y0);
      absijtoxy (li + 1, lj + 1, &xn, &yn);

      for (i = 0; i < getngr (); i++)
	{
	  gr = (Chrt *) getgr (i);
	  if (gr->x + gr->w > xn)
	    xn = gr->x + gr->w;
	  if (gr->y + gr->h > yn)
	    yn = gr->y + gr->h;
	}

      for (i = 0; i < getndr (); i++)
	{
	  dr = (Drwing *) getdr (i);
	  if (dr->x2 > xn)
	    xn = dr->x2;
	  if (dr->y2 > yn)
	    yn = dr->y2;
	  if (dr->x1 > xn)
	    xn = dr->x1;
	  if (dr->y1 > yn)
	    yn = dr->y1;
	}
    }

  else
    {
      absijtoxy (si, sj, &x0, &y0);
      absijtoxy (li + 1, lj + 1, &xn, &yn);
    }

  fp = fopen (filename, "w");
  FIG_init (fp);

  W = abs (xn - x0);
  H = abs (yn - y0);
  if (W > H)
    FIG_options (1200);
  else
    FIG_options (1200);

  setout (2, 15, 15, x0, y0);

  for (i = si; i <= li; i++)
    {
      for (j = sj; j <= lj; j++)
	{
	  xdrwcl (i, j, 5);
	}
    }

  for (i = 0; i < getngr (); i++)
    {
      gr = (Chrt *) getgr (i);
      if (gr->x >= x0 && gr->y >= y0 && gr->x + gr->w <= x0 + W && gr->y + gr->h <= y0 + H)
	newplot (i, gr);
    }

  for (i = 0; i < getndr (); i++)
    {
      dr = (Drwing *) getdr (i);
      if (
      (dr->x1 >= x0 && dr->y1 >= y0 && dr->x1 <= x0 + W && dr->y1 <= y0 + H)
	   ||
      (dr->x2 >= x0 && dr->y2 >= y0 && dr->x2 <= x0 + W && dr->y2 <= y0 + H)
	)
	redrwdr (dr);
    }

  setout (0, 1, 1, x0, y0);

  fclose (fp);

}

int
printfl2 (filename)
     char *filename;
{
  FILE *fp;
  int i, j, maxj, maxi, jj, l;
  double xscale, yscale, h, w, H, W;
  int x, y, x0, y0, xn, yn;
  int x1, y1;
  int xmax, ymax;
  int portrait = 0;
  Chrt *gr;
  Drwing *dr;

  fp = fopen (filename, "w");
  PS_init (fp);
  PS_options ();

  absijtoxy (si, sj, &x0, &y0);
  absijtoxy (li + 1, lj + 1, &xn, &yn);
  if (portrait)
    {
      xmax = 5040;
      ymax = 7200;
    }
  else
    {
      xmax = 7200;
      ymax = -5040;
    }

  PS_set_size (xmax, ymax);

  PS_graphics ();
  PS_linetype (-2);

  ijtoxy (si, sj, &x0, &y0);
  ijtoxy (li + 1, lj + 1, &xn, &yn);

  PS_set_size (xn - x0, yn - y0);
  W = xscale = (xn - x0);
  H = yscale = (yn - y0);

  xscale = xmax / xscale;
  yscale = ymax / yscale;
  if (abs (yscale) < abs (xscale) && portrait)
    xscale = yscale;
  if (abs (yscale) > abs (xscale) && !portrait)
    xscale = -yscale;

  if (portrait)
    {
      x0 = -x0 * xscale;
      y0 = y0 * xscale + ymax - 50;
      yscale = -xscale;
    }
  else
    {
      x0 = -x0 * xscale;
      y0 = -y0 * xscale;
      yscale = -xscale;
    }

  for (i = si; i <= li; i++)
    {
      for (j = sj; j <= lj; j++)
	{
	  xdrwcl (i, j, 4);
	}
    }

  for (i = 0; i < getngr (); i++)
    {
      gr = (Chrt *) getgr (i);
      newplot (i, gr);
    }

  for (i = 0; i < getndr (); i++)
    {
      dr = (Drwing *) getdr (i);
      redrwdr (dr);
    }

  PS_text ();
  PS_reset ();
  fclose (fp);

}

void
plot (type)
     int type;
{
  int i, j, maxj, maxi, jj, l;
  int k;
  Chrt *gr;
  double *X, *Y, *x, *y;
  double xx, yy;
  int dim1, dim2;
  gr = (Chrt *) getlastgra ();
  if (gr->numrange == 0 && (si != li || sj != lj) && (si != 0 && li != 0 && sj != 0 && lj != 0))
    for (j = 0; j <= lj - sj; j++)
      {
	(gr->range[j]).i1 = si;
	(gr->range[j]).i2 = li;
	(gr->range[j]).j1 = j + sj;
	(gr->range[j]).j2 = j + sj;
	gr->numrange++;
      }

  newplot (getactivgra (), gr);
  return;

}

printplot (number)
     int number;
{
}

int
is_selecting ()
{
  return high;
}

int
get_select (i1, j1, i2, j2)
     int *i1, *j1, *i2, *j2;
{
  *i1 = si;
  *i2 = li;
  *j1 = sj;
  *j2 = lj;
  return high;
}

void
updatechrt ()
{
  plot (0);
}

int
alphatonum (buf, ii, jj)
     char *buf;
     int *ii;
     int *jj;
{

  int len;
  char token[2];
  int k, n;
  int i;
  int c = 0;
  int r = 0;
  int pos = 0;

  len = strlen (buf);

  for (i = 0; i < len; i++)
    {
      if ('a' <= buf[i] && buf[i] <= 'z')
	buf[i] = 'A' + buf[i] - 'a';
    }

  if (buf[0] == '$')
    {
      pos++;
    }

  while ('A' <= buf[pos] && buf[pos] <= 'Z')
    {
      c *= 26;
      c += (buf[pos] - 'A' + 1);
      pos++;
    }

  if (buf[pos] == '$')
    {
      pos++;
    }
  sscanf (buf + pos, "%d", &r);

  *ii = r;
  *jj = c;

}

int
n2a (buf, j, i)
     char *buf;
     int i;
     int j;
{

  int len = strlen (buf);
  char token;
  int k, n;
  sprintf (buf, "");
  if (i > 26 * 26)
    return;
  if (i > 26)
    {
      sprintf (buf, "%s%s", buf, ALPHA[(i - 1) / 26 - 1]);
      i = i - 26 * (i / 26);
    }
  if (i < 1)
    i = 1;
  sprintf (buf, "%s%s", buf, ALPHA[i - 1]);
  sprintf (buf, "%s%d", buf, j);

}
int
n2a2 (buf, j, i, dj, di)
     char *buf;
     int i;
     int j;
{

  int len = strlen (buf);
  char token;
  int k, n;
  sprintf (buf, "");
  if (di)
    sprintf (buf, "$");
  if (i > 26 * 26)
    return;
  if (i > 26)
    {
      sprintf (buf, "%s%s", buf, ALPHA[(i - 1) / 26 - 1]);
      i = i - 26 * (i / 26);
    }
  if (i < 1)
    i = 1;
  sprintf (buf, "%s%s", buf, ALPHA[i - 1]);
  if (dj)
    sprintf (buf, "%s$", buf);
  sprintf (buf, "%s%d", buf, j);

}

int
coltoalpha (buf, i)
     char *buf;
     int i;
{

  int len = strlen (buf);
  char token;
  int k, n;
  sprintf (buf, "");
  if (i > 26 * 26)
    return;
  if (i > 26)
    {
      sprintf (buf, "%s%s", buf, ALPHA[(i - 1) / 26 - 1]);
      i = i - 26 * (i / 26);
    }
  if (i < 1)
    i = 1;
  sprintf (buf, "%s%s", buf, ALPHA[i - 1]);

}
