/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include <stdlib.h>
#include <stdio.h>

#define MEMSIZE 1

static void *memmap[MEMSIZE];
static size_t memsize[MEMSIZE];
static size_t allocation = 0;
static int nptr = 0;

void *
absmalloc (Size, message)
     size_t Size;
     char *message;
{
#ifdef MEMDEBUG
  void *ptr;
  ptr = malloc (Size);

  memmap[nptr] = ptr;
  memsize[nptr] = Size;
  allocation += Size;
  nptr++;
  if (nptr > 999)
    nptr = 999;

  fprintf (stderr, "(%d/%d) %35s:	malloc size %d at %d\n", allocation, nptr, message, Size, ptr);
  return ptr;
#endif

#ifndef MEMDEBUG
  return malloc (Size);
#endif

}

void
absfree (Pointer, message)
     void *Pointer;
     char *message;
{
#ifdef MEMDEBUG
  int i, j;
  size_t Size = 0;
  if (Pointer != NULL)
    {
      i = nptr - 1;
      while (i >= 0 && memmap[i] != Pointer)
	i--;
      if (i >= 0)
	{
	  allocation -= memsize[i];
	  Size = memsize[i];
	  for (j = i; j < nptr - 1; j++)
	    {
	      memsize[j] = memsize[j + 1];
	      memmap[j] = memmap[j + 1];
	    }
	}
      nptr--;
    }

  fprintf (stderr, "(%d/%d) %35s:	free size %d at  %d\n", allocation, nptr, message, Size, Pointer);
  if (Size <= 0)
    return;
#endif

  free (Pointer);
}

void *
absrealloc (Pointer, Size, message)
     void *Pointer;
     size_t Size;
     char *message;
{
#ifdef MEMDEBUG
  int i, j;
  size_t oldSize = 0;
  void *ptr;
  ptr = realloc (Pointer, Size);

  if (Pointer != NULL)
    {
      int i = nptr - 1;
      while (i >= 0 && memmap[i] != Pointer)
	i--;
      if (i >= 0)
	{
	  oldSize = memsize[i];
	  allocation -= memsize[i];
	  allocation += Size;
	  memsize[i] = Size;
	  memmap[i] = ptr;
	}
    }
  else
    {
      i = nptr;
      nptr++;
      if (nptr > 999)
	nptr = 999;
      allocation -= memsize[i];
      allocation += Size;
      memsize[i] = Size;
      memmap[i] = ptr;
    }

  fprintf (stderr, "(%d/%d) %35s:	realloc size %d from %d to ", allocation, nptr, message, oldSize, Pointer);
  fprintf (stderr, "size %d at %d\n", Size, ptr);
  return ptr;
#endif

#ifndef MEMDEBUG
  return realloc (Pointer, Size);
#endif

}
