/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include "myscroll.h"
#include "pixmap.h"
#include "math.h"

int nosetval = 0;

val2pos (vmin, vmax, h, val)
     int vmin, vmax, val, h;
{
  int pos = (val - vmin) * (h - 52) / (vmax - vmin) + 16;

  return pos;
}

pos2val (vmin, vmax, h, pos)
     int vmin, vmax, pos, h;
{
  int val = (vmax - vmin) * (pos - 16) / (h - 52) + vmin;

  return val;
}

sethpos (ms, pos)
     Myscroll *ms;
     int pos;
{
  int n, h;
  Arg args[2];
  if (pos < 16)
    pos = 16;
  if (pos > ms->w - 52)
    pos = ms->w - 52;
  ms->curx = pos;

  XtSetArg (args[0], XtNhorizDistance, ms->curx + 1);
  XtSetValues (ms->thumb, args, 1);

}
setvpos (ms, pos)
     Myscroll *ms;
     int pos;
{
  int n;
  Arg args[2];

  if (pos < 16)
    pos = 16;
  if (pos > ms->h - 52)
    pos = ms->h - 52;
  ms->cury = pos;

  XtSetArg (args[0], XtNvertDistance, ms->cury + 1);
  XtSetValues (ms->thumb, args, 1);

}

void
resizescroll (widget, pointer, event, ctd)
     Widget widget;
     XtPointer pointer;
     XEvent *event;
     Boolean *ctd;
{
  int n;
  Dimension h, w;
  Myscroll *ms;
  Arg args[2];
  ms = (Myscroll *) pointer;

  n = 0;
  XtSetArg (args[n], XtNheight, &h);
  n++;
  XtSetArg (args[n], XtNwidth, &w);
  n++;
  XtGetValues (widget, args, n);

  ms->h = h;
  ms->w = w;

  if (h > w)
    {
      setvpos (ms, ms->cury);
      XtSetArg (args[0], XtNvertDistance, h - 16);
      XtSetValues (ms->down, args, 1);

    }
  else
    {
      sethpos (ms, ms->curx);
      XtSetArg (args[0], XtNhorizDistance, w - 16);
      XtSetValues (ms->right, args, 1);

    }
}

void
btncb (widget, pointer, junk)
     Widget widget;
     XtPointer pointer;
     XtPointer junk;
{
  Arg args[20];
  int n, i;
  double rel;
  int pos;

  Myscroll *ms = (Myscroll *) pointer;

  if (widget == ms->up || widget == ms->left)
    {
      ms->val -= 1;
      if (ms->val < ms->vmin)
	ms->val = ms->vmin;
    }
  if (widget == ms->down || widget == ms->right)
    {
      ms->val += 1;
      if (ms->val > ms->vmax)
	ms->val = ms->vmax;
    }
  if (widget == ms->thumb)
    {
    }

  if (ms->h > ms->w)
    {
      pos = val2pos (ms->vmin, ms->vmax, ms->h, ms->val);
      setvpos (ms, pos);

    }
  else
    {
      pos = val2pos (ms->vmin, ms->vmax, ms->w, ms->val);
      sethpos (ms, pos);

    }
  nosetval = 1;
  ms->cbfunction (ms->val);
  nosetval = 0;
};

void
movecb (w, pointer, event, ctd)
     Widget w;
     XtPointer pointer;
     XEvent *event;
     Boolean *ctd;
{
  int x, y;
  Arg args[20];
  int n, i;
  double rel;

  Myscroll *ms = (Myscroll *) pointer;

  x = event->xbutton.x_root;
  y = event->xbutton.y_root;

  ms->cury = ms->ssy + y - ms->thumbpy;
  ms->curx = ms->ssx + x - ms->thumbpx;

  if (ms->h > ms->w)
    {
      if (ms->cury < 16)
	ms->cury = 16;
      if (ms->cury > ms->h - 32)
	ms->cury = ms->h - 32;

      ms->val = pos2val (ms->vmin, ms->vmax, ms->h, ms->cury);
      setvpos (ms, ms->cury);

    }
  else
    {
      if (ms->curx < 16)
	ms->curx = 16;
      if (ms->curx > ms->w - 32)
	ms->curx = ms->w - 32;

      ms->val = pos2val (ms->vmin, ms->vmax, ms->w, ms->curx);
      sethpos (ms, ms->curx);

    }
  nosetval = 1;
  ms->cbfunction (ms->val);
  nosetval = 0;
};

void
stepcb (w, pointer, event, ctd)
     Widget w;
     XtPointer pointer;
     XEvent *event;
     Boolean *ctd;
{
  int x, y;
  Arg args[20];
  int n, i;
  double rel, pos;
  double delta;

  Myscroll *ms = (Myscroll *) pointer;

  x = event->xbutton.x;
  y = event->xbutton.y;
  if (ms->h > ms->w)
    {
      if (y < 16 || y > ms->h - 16)
	return;

      if (fabs (y - ms->cury) < 10)
	ms->cury = y;
      else
	{
	  delta = (y - ms->cury) / 3.0;
	  ms->cury += delta;
	}
      if (ms->cury < 16)
	ms->cury = 16;
      if (ms->cury > ms->h - 52)
	ms->cury = ms->h - 52;

      ms->val = pos2val (ms->vmin, ms->vmax, ms->h, ms->cury);
      setvpos (ms, ms->cury);

    }
  else
    {
      if (x < 16 || x > ms->w - 16)
	return;

      if (fabs (x - ms->curx) < 10)
	ms->curx = x;
      else
	{
	  delta = (x - ms->curx) / 3.0;
	  ms->curx += delta;
	}
      if (ms->curx < 16)
	ms->curx = 16;
      if (ms->curx > ms->w - 52)
	ms->curx = ms->w - 52;

      ms->val = pos2val (ms->vmin, ms->vmax, ms->w, ms->curx);
      sethpos (ms, ms->curx);

    }
  nosetval = 1;
  ms->cbfunction (ms->val);
  nosetval = 0;

};

void
presscb (w, pointer, event, ctd)
     Widget w;
     XtPointer pointer;
     XEvent *event;
     Boolean *ctd;
{
  int x, y;
  Arg args[16];
  int n, i;
  double rel, pos;

  Myscroll *ms = (Myscroll *) pointer;

  x = event->xbutton.x_root;
  y = event->xbutton.y_root;

  ms->thumbpy = y;
  ms->thumbpx = x;
  ms->ssx = ms->curx;
  ms->ssy = ms->cury;
};

int
setscrolllimit (ms, vmin, vmax)
     Myscroll *ms;
     int vmin, vmax;
{
  ms->vmin = vmin;
  ms->vmax = vmax;

};

void
setscrollval (ms, val)
     Myscroll *ms;
     int val;
{
  double rel, pos;
  Arg args[20];
  int n;
  if (nosetval)
    return;

  ms->val = val;
  if (ms->val < ms->vmin)
    ms->val = ms->vmin;
  if (ms->val > ms->vmax)
    ms->val = ms->vmax;

  if (ms->h > ms->w)
    {

      ms->cury = val2pos (ms->vmin, ms->vmax, ms->h, ms->val);
      setvpos (ms, ms->cury);

    }
  else
    {
      ms->curx = val2pos (ms->vmin, ms->vmax, ms->w, ms->val);
      sethpos (ms, ms->curx);

    }

}

Myscroll *
CreateMyscroll (parent, h, w, vmin, vmax, func)
     Widget parent;
     int h, w, vmin, vmax;
     void (*func) ();
{

  XtTranslations scroll_trans_table;
  Arg args[16];
  int n, i;
  Myscroll *ms;
  Myscroll local;
  Widget box;
  ms = (Myscroll *) absmalloc (sizeof (Myscroll), "CreateMyscroll:ms ");

  ms->parent = parent;
  ms->cbfunction = func;
  ms->h = h;
  ms->w = w;
  ms->vmin = vmin;
  ms->vmax = vmax;
  ms->val = 0;

  XtSetArg (w_args[w_n], XtNheight, h);
  w_n++;
  XtSetArg (w_args[w_n], XtNwidth, w);
  w_n++;
  XtSetArg (w_args[w_n], XtNborderWidth, 0);
  w_n++;
  ms->back = (Widget) XtCreateManagedWidget ("window", formWidgetClass, parent,
					     w_args, w_n);
  if (h > w)
    {
      ms->up = (Widget) make_repeater_pixmap (ms->back, "up", up_xpm, btncb, (caddr_t) ms);
      n = 0;
      XtSetArg (args[n], XtNfromHoriz, NULL);
      n++;
      XtSetArg (args[n], XtNhorizDistance, 1);
      n++;
      XtSetArg (args[n], XtNfromVert, NULL);
      n++;
      XtSetArg (args[n], XtNvertDistance, 1);
      n++;
      XtSetValues (ms->up, args, n);

      ms->cury = 16;

      ms->thumb = (Widget) make_button_pixmap (ms->back, "vthumb", vthumb_xpm, btncb, (caddr_t) ms);
      n = 0;
      XtSetArg (args[n], XtNfromVert, NULL);
      n++;
      XtSetArg (args[n], XtNvertDistance, ms->cury);
      n++;
      XtSetArg (args[n], XtNfromHoriz, NULL);
      n++;
      XtSetArg (args[n], XtNhorizDistance, 1);
      n++;
      XtSetArg (w_args[w_n], XtNtop, XtChainTop);
      w_n++;
      XtSetArg (w_args[w_n], XtNbottom, XtChainTop);
      w_n++;
      XtSetValues (ms->thumb, args, n);

      ms->down = (Widget) make_repeater_pixmap (ms->back, "down", down_xpm, btncb, (caddr_t) ms);
      n = 0;
      XtSetArg (args[n], XtNfromVert, (XtPointer) NULL);
      n++;
      XtSetArg (args[n], XtNvertDistance, h - 14);
      n++;
      XtSetArg (args[n], XtNfromHoriz, NULL);
      n++;
      XtSetArg (args[n], XtNhorizDistance, 1);
      n++;
      XtSetValues (ms->down, args, n);
    }
  else
    {
      ms->left = (Widget) make_repeater_pixmap (ms->back, "left", left_xpm, btncb, (caddr_t) ms);
      n = 0;
      XtSetArg (args[n], XtNfromHoriz, NULL);
      n++;
      XtSetArg (args[n], XtNhorizDistance, 1);
      n++;
      XtSetArg (args[n], XtNfromVert, NULL);
      n++;
      XtSetArg (args[n], XtNvertDistance, 1);
      n++;
      XtSetValues (ms->left, args, n);

      ms->curx = 16;

      ms->thumb = (Widget) make_button_pixmap (ms->back, "hthumb", hthumb_xpm, btncb, (caddr_t)
					       ms);
      n = 0;
      XtSetArg (args[n], XtNfromHoriz, NULL);
      n++;
      XtSetArg (args[n], XtNhorizDistance, ms->curx);
      n++;
      XtSetArg (args[n], XtNfromVert, NULL);
      n++;
      XtSetArg (args[n], XtNvertDistance, 1);
      n++;
      XtSetValues (ms->thumb, args, n);

      ms->right = (Widget) make_repeater_pixmap (ms->back, "right", right_xpm, btncb, (caddr_t)
						 ms);
      n = 0;
      XtSetArg (args[n], XtNfromHoriz, NULL);
      n++;
      XtSetArg (args[n], XtNhorizDistance, w - 14);
      n++;
      XtSetArg (args[n], XtNfromVert, NULL);
      n++;
      XtSetArg (args[n], XtNvertDistance, 1);
      n++;
      XtSetValues (ms->right, args, n);

    }

  XtAddEventHandler (ms->back, ButtonPressMask, False, stepcb, (XtPointer) ms);
  XtAddEventHandler (ms->back, ExposureMask, FALSE, shadow2, (caddr_t) ms->back);
  XtAddEventHandler (ms->thumb, ButtonPressMask, False, presscb, (XtPointer) ms);
  XtAddEventHandler (ms->thumb, Button1MotionMask, False, movecb, (XtPointer) ms);

  XtAddEventHandler (ms->back, StructureNotifyMask, False, resizescroll, (caddr_t) ms);
  return ms;
}
