/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include "stack.h"
#include "y.tab.h"
#include <math.h>
#include <stdio.h>

makelt (s)
     stack *s;
{
  obj o1, o2, o3;
  o2 = stack_pop (s);
  o1 = stack_pop (s);
  o3.rec.d = (o1.rec.d < o2.rec.d);
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

makele (s)
     stack *s;
{
  obj o1, o2, o3;
  o2 = stack_pop (s);
  o1 = stack_pop (s);
  o3.rec.d = (o1.rec.d <= o2.rec.d);
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

makegt (s)
     stack *s;
{
  obj o1, o2, o3;
  o2 = stack_pop (s);
  o1 = stack_pop (s);
  o3.rec.d = (o1.rec.d > o2.rec.d);
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

makege (s)
     stack *s;
{
  obj o1, o2, o3;
  o2 = stack_pop (s);
  o1 = stack_pop (s);
  o3.rec.d = (o1.rec.d >= o2.rec.d);
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

makeeq (s)
     stack *s;
{
  obj o1, o2, o3;
  o2 = stack_pop (s);
  o1 = stack_pop (s);
  o3.rec.d = (o1.rec.d == o2.rec.d);
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

makene (s)
     stack *s;
{
  obj o1, o2, o3;
  o2 = stack_pop (s);
  o1 = stack_pop (s);
  o3.rec.d = (o1.rec.d != o2.rec.d);
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

makeand (s)
     stack *s;
{
  obj o1, o2, o3;
  o2 = stack_pop (s);
  o1 = stack_pop (s);
  o3.rec.d = (o1.rec.d && o2.rec.d);
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

makeor (s)
     stack *s;
{
  obj o1, o2, o3;
  o2 = stack_pop (s);
  o1 = stack_pop (s);
  o3.rec.d = (o1.rec.d || o2.rec.d);
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

makemult (s)
     stack *s;
{
  obj o1, o2, o3;

  o2 = stack_pop (s);
  o1 = stack_pop (s);
  o3.rec.d = o1.rec.d * o2.rec.d;
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

makediv (s)
     stack *s;
{
  obj o1, o2, o3;

  o2 = stack_pop (s);
  o1 = stack_pop (s);
  o3.rec.d = o1.rec.d / o2.rec.d;
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

makediff (s)
     stack *s;
{
  obj o1, o2, o3;

  o2 = stack_pop (s);
  o1 = stack_pop (s);
  o3.rec.d = o1.rec.d - o2.rec.d;
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

makeinv (s)
     stack *s;
{
  obj o1, o3;

  o1 = stack_pop (s);
  o3.rec.d = -o1.rec.d;
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

int
makesum (s)
     stack *s;
{
  obj o1, o2, o3;

  o2 = stack_pop (s);
  o1 = stack_pop (s);

  o3.rec.d = o1.rec.d + o2.rec.d;
  o3.type = FLOATING_CONSTANT;
  stack_push (s, o3);
}

double
makefct (s, name, narg)
     stack *s;
     int name, narg;
{
  obj *o = (obj *) absmalloc (sizeof (obj) * narg, "makefct:o ");
  obj o3;
  int i;

  o3.rec.d = 0.0;

  for (i = narg - 1; i >= 0; i--)
    {
      o[i] = stack_pop (s);

    }

  switch (name)
    {
    case ACOS:
      {
	o3.rec.d = acos (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case ASIN:
      {
	o3.rec.d = asin (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case ATAN:
      {
	o3.rec.d = atan (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case CONCATENATE:
      {
	int len = 1;
	for (i = 0; i < narg; i++)
	  if (o[i].type == STRING_CONSTANT)
	    if (o[i].rec.s != NULL)
	      len += strlen (o[i].rec.s);

	o3.rec.s = (char *) absmalloc (sizeof (char) * len, "makefct:o3.rec.s ");
	o3.rec.s[0] = '\0';
	for (i = 0; i < narg; i++)
	  if (o[i].type == STRING_CONSTANT)
	    if (o[i].rec.s != NULL)
	      {
		strcat (o3.rec.s, o[i].rec.s);

	      }

	o3.type = STRING_CONSTANT;
	break;
      }
    case CEIL:
      {
	o3.rec.d = ceil (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case COS:
      {
	o3.rec.d = cos (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case COSH:
      {
	o3.rec.d = cosh (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case EXP:
      {
	o3.rec.d = exp (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case FABS:
      {
	o3.rec.d = fabs (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case FLOOR:
      {
	o3.rec.d = floor (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case LOG:
      {
	o3.rec.d = log (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case LOG10:
      {
	o3.rec.d = log10 (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case SEND:
      {
	o3.rec.d = send (obj2int (o[0]), obj2string (o[1]));
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case SIN:
      {
	o3.rec.d = sin ((double) o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case SINH:
      {
	o3.rec.d = sinh (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case SPAWN:
      {
	switch (narg)
	  {
	  case 1:
	    o3.rec.d = spawn (o[0].rec.s, NULL);
	    break;
	  case 2:
	    o3.rec.d = spawn (o[0].rec.s, o[1].rec.s, NULL);
	    break;
	  case 3:
	    o3.rec.d = spawn (o[0].rec.s, o[1].rec.s, o[2].rec.s, NULL);
	    break;
	  case 4:
	    o3.rec.d = spawn (o[0].rec.s, o[1].rec.s, o[2].rec.s, o[3].rec.s, NULL);
	    break;
	  }
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case SQRT:
      {
	o3.rec.d = sqrt (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case RECEIVE:
      {
	absfree (o3.rec.s, "makefct:o3.rec.s:RECEIVE ");
	o3.rec.s = (char *) receive (obj2int (o[0]), obj2int (o[1]));
	o3.type = STRING_CONSTANT;
	break;
      }
    case TAN:
      {
	o3.rec.d = sin ((double) o[0].rec.d) / cos ((double) o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case TANH:
      {
	o3.rec.d = tanh (o[0].rec.d);
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case absMIN:
      {
	o3.rec.d = o[0].rec.d;
	for (i = 1; i < narg; i++)
	  if (o[i].rec.d < o3.rec.d)
	    o3.rec.d = o[i].rec.d;
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case absMAX:
      {
	o3.rec.d = o[0].rec.d;
	for (i = 1; i < narg; i++)
	  if (o[i].rec.d > o3.rec.d)
	    o3.rec.d = o[i].rec.d;
	o3.type = FLOATING_CONSTANT;
	break;
      }
    case MAXMEAN:
      {
	double *max;
	int k, numele, j;
	double val = 0.0;
	double min;

	if (narg < 2)
	  {
	    o3.rec.d = o[1].rec.d;
	    break;
	  }

	max = (double *) absmalloc (sizeof (double) * narg, "makefct:max ");
	numele = o[0].rec.d;
	if (numele >= narg || numele < 0)
	  {
	    o3.rec.d = o[1].rec.d;
	    break;
	  }
	for (k = 0; k < numele; k++)
	  max[k] = -1.e199;

	for (k = 1; k < narg; k++)
	  {
	    min = max[0];
	    j = 0;
	    for (i = 1; i < numele; i++)
	      if (max[i] < min)
		{
		  j = i;
		  min = max[i];
		}
	    if (o[k].rec.d > max[j])
	      max[j] = o[k].rec.d;
	  }

	for (k = 0; k < numele; k++)
	  val += max[k];
	val /= numele;

	absfree (max, "makefct:max ");

	o3.type = FLOATING_CONSTANT;
	break;
      }
    case SUM:
      {
	o3.rec.d = 0;
	for (i = 0; i < narg; i++)
	  o3.rec.d += o[i].rec.d;
	o3.type = FLOATING_CONSTANT;
	break;
      }

    }

  stack_push (s, o3);
  absfree (o, "makefct:o ");

}
