/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include "post.h"

#define POINT_TYPES             6

FILE *outfile;
float xsize = 50;
float ysize = 80;
int v_char, h_char;

char ps_font[MAX_ID_LEN + 1] = "Helvetica";
int ps_fontsize = 14;
TBOOLEAN ps_portrait = FALSE;
TBOOLEAN ps_color = FALSE;
TBOOLEAN ps_solid = FALSE;
TBOOLEAN ps_eps = FALSE;

int ps_page = 0;
int ps_path_count = 0;
int ps_ang = 0;
enum JUSTIFY ps_justify = LEFT;

char GPFAR *GPFAR PS_header[] =
{
  "/M {moveto} bind def\n",
  "/L {lineto} bind def\n",
  "/R {rmoveto} bind def\n",
  "/V {rlineto} bind def\n",
  "/vpt2 vpt 2 mul def\n",
  "/hpt2 hpt 2 mul def\n",

  "/Lshow { currentpoint stroke M\n",
  "  0 vshift R show } def\n",

  "/Rshow { currentpoint stroke M\n",
  "  dup stringwidth pop neg vshift R show } def\n",

  "/Cshow { currentpoint stroke M\n",
  "  dup stringwidth pop -2 div vshift R show } def\n",

  "/DL { Color {setrgbcolor Solid {pop []} if 0 setdash }\n",
  " {pop pop pop Solid {pop []} if 0 setdash} ifelse } def\n",

  "/BL { stroke gnulinewidth 2 mul setlinewidth } def\n",

  "/AL { stroke gnulinewidth 2 div setlinewidth } def\n",

  "/PL { stroke gnulinewidth setlinewidth } def\n",

  "/LTb { BL [] 0 0 0 DL } def\n",
  "/LTa { AL [1 dl 2 dl] 0 setdash 0 0 0 setrgbcolor } def\n",
  "/LT0 { PL [] 0 1 0 DL } def\n",
  "/LT1 { PL [4 dl 2 dl] 0 0 1 DL } def\n",
  "/LT2 { PL [2 dl 3 dl] 1 0 0 DL } def\n",
  "/LT3 { PL [1 dl 1.5 dl] 1 0 1 DL } def\n",
  "/LT4 { PL [5 dl 2 dl 1 dl 2 dl] 0 1 1 DL } def\n",
  "/LT5 { PL [4 dl 3 dl 1 dl 3 dl] 1 1 0 DL } def\n",
  "/LT6 { PL [2 dl 2 dl 2 dl 4 dl] 0 0 0 DL } def\n",
  "/LT7 { PL [2 dl 2 dl 2 dl 2 dl 2 dl 4 dl] 1 0.3 0 DL } def\n",
"/LT8 { PL [2 dl 2 dl 2 dl 2 dl 2 dl 2 dl 2 dl 4 dl] 0.5 0.5 0.5 DL } def\n",
  "/P { stroke [] 0 setdash\n",
  "  currentlinewidth 2 div sub M\n",
  "  0 currentlinewidth V stroke } def\n",
  "/D { stroke [] 0 setdash 2 copy vpt add M\n",
  "  hpt neg vpt neg V hpt vpt neg V\n",
  "  hpt vpt V hpt neg vpt V closepath stroke\n",
  "  P } def\n",
  "/A { stroke [] 0 setdash vpt sub M 0 vpt2 V\n",
  "  currentpoint stroke M\n",
  "  hpt neg vpt neg R hpt2 0 V stroke\n",
  "  } def\n",
  "/B { stroke [] 0 setdash 2 copy exch hpt sub exch vpt add M\n",
  "  0 vpt2 neg V hpt2 0 V 0 vpt2 V\n",
  "  hpt2 neg 0 V closepath stroke\n",
  "  P } def\n",
  "/C { stroke [] 0 setdash exch hpt sub exch vpt add M\n",
  "  hpt2 vpt2 neg V currentpoint stroke M\n",
  "  hpt2 neg 0 R hpt2 vpt2 V stroke } def\n",
  "/T { stroke [] 0 setdash 2 copy vpt 1.12 mul add M\n",
  "  hpt neg vpt -1.62 mul V\n",
  "  hpt 2 mul 0 V\n",
  "  hpt neg vpt 1.62 mul V closepath stroke\n",
  "  P  } def\n",
  "/S { 2 copy A C} def\n",
  NULL
};

#define PS_XOFF	50
#define PS_YOFF	50

#define PS_XMAX 7200
#define PS_YMAX 5040

#define PS_XLAST (PS_XMAX - 1)
#define PS_YLAST (PS_YMAX - 1)

#define PS_VTIC (PS_YMAX/80)
#define PS_HTIC (PS_YMAX/80)

#define PS_SC (10)
#define	PS_LW (0.3*PS_SC)

#define PS_VCHAR (14*PS_SC)
#define PS_HCHAR (14*PS_SC*6/10)

int PS_pen_x, PS_pen_y;
int PS_taken;
int PS_linetype_last;
TBOOLEAN PS_relative_ok;

PS_set_size (x, y)
     int x, y;
{
  xsize = x;
  ysize = y;
}

PS_options ()
{
  extern struct value *const_express ();
  extern double real ();
  ps_portrait = FALSE;
  ps_eps = FALSE;
  ps_color = FALSE;
  strcpy (ps_font, "Helvetica");
  ps_fontsize = 8;
  v_char = (unsigned int) (ps_fontsize * PS_SC);
  h_char = (unsigned int) (ps_fontsize * PS_SC * 6 / 10);

}

PS_init (name)
     FILE *name;
{

  static char GPFAR psi1[] = "%%%%Creator: gnuplot\n\
%%%%DocumentFonts: %s\n\
%%%%BoundingBox: %d %d ";
  static char GPFAR psi2[] = "%%%%EndComments\n\
/gnudict 40 dict def\ngnudict begin\n\
/Color %s def\n\
/Solid %s def\n\
/gnulinewidth %.3f def\n\
/vshift %d def\n\
/dl {%d mul} def\n\
/hpt %.1f def\n\
/vpt %.1f def\n";

  int i;
  outfile = name;
  ps_page = 0;
  if (!ps_eps)
    fprintf (outfile, "%%!PS-Adobe-2.0\n");
  else
    fprintf (outfile, "%%!PS-Adobe-2.0 EPSF-2.0\n");
  fprintf (outfile, psi1, ps_font, PS_XOFF, PS_YOFF);
  if (ps_portrait)
    fprintf (outfile, "%d %d\n",
    (int) (xsize * (ps_eps ? 0.5 : 1.0) * (PS_XMAX) / PS_SC + 0.5 + PS_XOFF),
	     (int) (ysize * (ps_eps ? 0.5 : 1.0) * (PS_YMAX) / PS_SC + 0.5 + PS_YOFF));
  else
    fprintf (outfile, "%d %d\n",
    (int) (ysize * (ps_eps ? 0.5 : 1.0) * (PS_YMAX) / PS_SC + 0.5 + PS_XOFF),
	     (int) (xsize * (ps_eps ? 0.5 : 1.0) * (PS_XMAX) / PS_SC + 0.5 + PS_YOFF));
  if (!ps_eps)
    fprintf (outfile, "%%%%Pages: (atend)\n");
  fprintf (outfile, psi2,
	   ps_color ? "true" : "false",
	   ps_solid ? "true" : "false",
	   PS_LW,
	   (int) v_char / (-3),
	   PS_SC,
	   PS_HTIC / 2.0,
	   PS_VTIC / 2.0);

  for (i = 0; PS_header[i] != NULL; i++)
    fprintf (outfile, "%s", PS_header[i]);
  fprintf (outfile, "end\n%%%%EndProlog\n");
}

PS_graphics ()
{
  static char GPFAR psg1[] = "0 setgray\n/%s findfont %d scalefont setfont\nnewpath\n";

  ps_page++;
  if (!ps_eps)
    fprintf (outfile, "%%%%Page: %d %d\n", ps_page, ps_page);
  fprintf (outfile, "gnudict begin\ngsave\n");
  fprintf (outfile, "%d %d translate\n", PS_XOFF, PS_YOFF);
  fprintf (outfile, "%.3f %.3f scale\n", (ps_eps ? 0.5 : 1.0) / PS_SC,
	   (ps_eps ? 0.5 : 1.0) / PS_SC);
  if (!ps_portrait)
    {

      fprintf (outfile, "90 rotate\n");
    }
  fprintf (outfile, psg1, ps_font, v_char);
  ps_path_count = 0;
  PS_relative_ok = FALSE;
  PS_pen_x = PS_pen_y = -4000;
  PS_taken = 0;
  PS_linetype_last = -1;
}

PS_text ()
{
  ps_path_count = 0;
  fprintf (outfile, "stroke\ngrestore\nend\nshowpage\n");

}

PS_reset ()
{
  fprintf (outfile, "%%%%Trailer\n");
  if (!ps_eps)
    fprintf (outfile, "%%%%Pages: %d\n", ps_page);
}

PS_linetype (linetype)
     int linetype;
{
  char *line = "ba012345678";
  linetype = (linetype % 9) + 2;
  PS_relative_ok = FALSE;
  if (PS_linetype_last == linetype)
    return (0);
  PS_linetype_last = linetype;
  fprintf (outfile, "LT%c\n", line[linetype]);
  ps_path_count = 0;
}

PS_move (x, y)
     unsigned int x, y;
{
  int dx, dy;
  char abso[20], rel[20];
  dx = x - PS_pen_x;
  dy = y - PS_pen_y;

  if (dx == 0 && dy == 0 && PS_relative_ok)
    return (0);
  sprintf (abso, "%d %d M\n", x, y);
  sprintf (rel, "%d %d R\n", dx, dy);
  if (strlen (rel) < strlen (abso) && PS_relative_ok)
    {
      fputs (rel, outfile);
      PS_taken++;
    }
  else
    fputs (abso, outfile);
  PS_relative_ok = TRUE;
  ps_path_count += 1;
  PS_pen_x = x;
  PS_pen_y = y;
}

PS_vector (x, y)
     unsigned int x, y;
{
  int dx, dy;
  char abso[20], rel[20];
  dx = x - PS_pen_x;
  dy = y - PS_pen_y;
  if (dx == 0 && dy == 0)
    return (0);
  sprintf (abso, "%d %d L\n", x, y);
  sprintf (rel, "%d %d V\n", dx, dy);
  if (strlen (rel) < strlen (abso) && PS_relative_ok)
    {
      fputs (rel, outfile);
      PS_taken++;
    }
  else
    fputs (abso, outfile);
  PS_relative_ok = TRUE;
  ps_path_count += 1;
  PS_pen_x = x;
  PS_pen_y = y;
  if (ps_path_count >= 400)
    {
      fprintf (outfile, "currentpoint stroke M\n");
      ps_path_count = 0;
    }
}

PS_put_text (x, y, str)
     unsigned int x, y;
     char *str;
{
  char ch;
  if (!strlen (str))
    return (0);
  PS_move (x, y);
  if (ps_ang != 0)
    fprintf (outfile, "currentpoint gsave translate %d rotate 0 0 M\n"
	     ,ps_ang * 90);
  putc ('(', outfile);
  ch = *str++;
  while (ch != '\0')
    {
      if ((ch == '(') || (ch == ')') || (ch == '\\'))
	putc ('\\', outfile);
      putc (ch, outfile);
      ch = *str++;
    }
  switch (ps_justify)
    {
    case LEFT:
      fprintf (outfile, ") Lshow\n");
      break;
    case CENTRE:
      fprintf (outfile, ") Cshow\n");
      break;
    case RIGHT:
      fprintf (outfile, ") Rshow\n");
      break;
    }
  if (ps_ang != 0)
    fprintf (outfile, "grestore\n");
  ps_path_count = 0;
  PS_relative_ok = FALSE;
}

int
PS_text_angle (ang)
     int ang;
{
  ps_ang = ang;
  return TRUE;
}

int
PS_justify_text (mode)
     enum JUSTIFY mode;
{
  ps_justify = mode;
  return TRUE;
}

PS_point (x, y, number)
     int x, y;
     int number;
{
  char *point = "PDABCTS";
  number %= POINT_TYPES;
  if (number < -1)
    number = -1;
  fprintf (outfile, "%d %d %c\n", x, y, point[number + 1]);
  PS_relative_ok = 0;
  ps_path_count = 0;
  PS_linetype_last = -1;
}
