/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#define PROGRAM "G N U P L O T"
#define PROMPT "gnuplot> "
#if defined(AMIGA_SC_6_1) || defined(AMIGA_AC_5)
#define SHELL "NewShell"
#else
#ifdef ATARI
#define SHELL "gulam.prg"
#else
#ifdef OS2
#define SHELL "c:\\cmd.exe"
#else
#define SHELL "/bin/sh"
#endif
#endif
#endif

#if defined(__unix__) && !defined(unix)
#define unix
#endif

#define SAMPLES 100
#define ISO_SAMPLES 10
#define ZERO	1e-8

#ifndef TERM

#define TERM "unknown"
#endif

#define TRUE 1
#define FALSE 0

#define Pi 3.141592653589793
#define DEG2RAD (Pi / 180.0)

#define MIN_CRV_POINTS 100
#define MIN_SRF_POINTS 1000

#define MAX_LINE_LEN 1024
#define MAX_TOKENS 400
#define MAX_ID_LEN 50

#define MAX_AT_LEN 150
#define STACK_DEPTH 100
#define NO_CARET (-1)

#ifdef MSDOS
#define MAX_NUM_VAR	3

#else
#define MAX_NUM_VAR	5

#endif

#define MAP3D_CARTESIAN		0
#define MAP3D_SPHERICAL		1
#define MAP3D_CYLINDRICAL	2

#define CONTOUR_NONE	0
#define CONTOUR_BASE	1
#define CONTOUR_SRF	2
#define CONTOUR_BOTH	3

#define CONTOUR_KIND_LINEAR	0

#define CONTOUR_KIND_CUBIC_SPL	1
#define CONTOUR_KIND_BSPLINE	2

#define LEVELS_AUTO			0
#define LEVELS_INCREMENTAL	1
#define LEVELS_DISCRETE		2
#define MAX_DISCRETE_LEVELS   30

#define ANGLES_RADIANS	0
#define ANGLES_DEGREES	1

#if defined(AMIGA_SC_6_1) || defined(AMIGA_AC_5)
#define OS "Amiga "
#endif

#ifdef OS2
#ifdef unix
#undef unix
#define OS "OS/2"
#endif
#endif

#ifdef vms
#define OS "VMS "
#endif

#ifdef linux
#define OS "Linux "
#else
#ifdef unix
#define OS "unix "
#endif
#endif

#ifdef _WINDOWS
#define _Windows
#endif

#ifdef DOS386
#define OS "DOS 386 "
#endif
#ifdef _Windows
#define OS "MS-Windows "
#else
#ifdef MSDOS
#ifdef unix
#undef OS
#undef unix
#endif
#define OS "MS-DOS "
#endif
#endif

#ifdef ATARI
#define OS "TOS "
#endif

#ifndef OS
#define OS ""
#endif

#if (defined(__TURBOC__) && defined(MSDOS)) || (defined(_Windows) && !defined(WIN32))
#define GPHUGE huge
#define GPFAR far
#else
#define GPHUGE
#define GPFAR
#endif

#ifdef ATARI
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#define VERYLARGE	HUGE_VAL
#else
#if defined(MSDOS) || defined(_Windows)
#include <float.h>
#define VERYLARGE (FLT_MAX/2 -1)
#else
#if defined( vms ) || defined( _CRAY ) || defined( NEXT ) || defined(__osf__) || defined( OS2 ) || defined(__EMX__) || defined( DOS386) || defined(KSR)
#include <float.h>
#if defined ( NEXT )
#if defined ( DBL_MAX)
#undef DBL_MAX
#endif
#define DBL_MAX 1.7976931348623157e+308
#undef HUGE
#define HUGE	DBL_MAX
#undef HUGE_VAL
#define HUGE_VAL DBL_MAX
#endif
#define VERYLARGE (DBL_MAX/2 -1)
#else
#ifdef AMIGA_AC_5
#include <math.h>
#define VERYLARGE (HUGE/2 -1)
#else
#ifdef AMIGA_SC_6_1
#include <float.h>
#ifndef HUGE
#define HUGE DBL_MAX
#endif
#define VERYLARGE (HUGE/2 -1)
#else

#ifdef ISC22
#include <float.h>
#ifndef HUGE
#define HUGE DBL_MAX
#endif
#endif

#ifndef HUGE
#define HUGE DBL_MAX
#endif
#define VERYLARGE (HUGE/2 -1)

#endif
#endif
#endif
#endif
#endif

#ifdef AMIGA_SC_6_1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#endif

#define END_OF_COMMAND (c_token >= num_tokens || equals(c_token,";"))

#ifdef vms

#define is_comment(c) ((c) == '#' || (c) == '!')
#define is_system(c) ((c) == '$')

#else

#define is_comment(c) ((c) == '#')
#define is_system(c) ((c) == '!')

#endif

#if defined(NOVFORK) || defined(MSDOS) || defined( OS2 ) || defined(_Windows) || defined(DOS386)
#undef VFORK
#else
#ifdef unix
#define VFORK
#endif
#endif

#ifdef vms
#define memcpy(dest,src,len) lib$movc3(&len,src,dest)
#else
#if defined(MEMCPY) || defined(MSDOS) || defined (ATARI) || defined( OS2 ) || defined(_Windows) || defined(DOS386)

#else
#ifdef NOCOPY

#else

#define memcpy(dest,src,len) bcopy(src,dest,len)
#endif
#endif
#endif

#if defined(MEMSET) || defined(MSDOS) || defined( OS2 ) || defined(_Windows) || defined(DOS386)
#define bzero(dest,len)  (void)(memset(dest, 0, len))
#endif

#define top_of_stack stack[s_p]

typedef int TBOOLEAN;

#ifdef __ZTC__
typedef int (*FUNC_PTR) (...);
#else
typedef int (*FUNC_PTR) ();
#endif

enum operators
  {
    PUSH, PUSHC, PUSHD1, PUSHD2, PUSHD, CALL, CALLN, LNOT, BNOT, UMINUS,
    LOR, LAND, BOR, XOR, BAND, EQ, NE, GT, LT, GE, LE, PLUS, MINUS, MULT,
    DIV, MOD, POWER, FACTORIAL, BOOLE, JUMP, JUMPZ, JUMPNZ, JTERN, SF_START
  };

#define is_jump(operator) ((operator) >=(int)JUMP && (operator) <(int)SF_START)

enum DATA_TYPES
  {
    INTGR, CMPLX
  };

enum PLOT_TYPE
  {
    FUNC, DATA, FUNC3D, DATA3D
  };

enum PLOT_STYLE
  {
    LINES, POINTSTYLE, IMPULSES, LINESPOINTS, DOTS, ERRORBARS, BOXES, BOXERROR, STEPS
  };

enum JUSTIFY
  {
    LEFT, CENTRE, RIGHT
  };

#if !(defined(ATARI)&&defined(__GNUC__)&&defined(_MATH_H))

struct cmplx
  {
    double real, imag;
  };
#endif

struct value
  {
    enum DATA_TYPES type;
    union
      {
	int int_val;
	struct cmplx cmplx_val;
      }
    v;
  };

struct lexical_unit
  {
    TBOOLEAN is_token;
    struct value l_val;
    int start_index;
    int length;
  };

struct ft_entry
  {
    char *f_name;
    FUNC_PTR func;
  };

struct udft_entry
  {
    struct udft_entry *next_udf;
    char udf_name[MAX_ID_LEN + 1];
    struct at_type *at;
    char *definition;
    struct value dummy_values[MAX_NUM_VAR];

  };

struct udvt_entry
  {
    struct udvt_entry *next_udv;
    char udv_name[MAX_ID_LEN + 1];
    TBOOLEAN udv_undef;
    struct value udv_value;
  };

union argument
  {
    int j_arg;
    struct value v_arg;
    struct udvt_entry *udv_arg;
    struct udft_entry *udf_arg;
  };

struct at_entry
  {
    enum operators index;
    union argument arg;
  };

struct at_type
  {
    int a_count;
    struct at_entry actions[MAX_AT_LEN];

  };

enum coord_type
  {
    INRANGE,
    OUTRANGE,
    UNDEFINED
  };

#if defined(MSDOS) || defined(_Windows)
typedef float coordval;
#else
typedef double coordval;
#endif

struct coordinate
  {
    enum coord_type type;
    coordval x, y, z;
    coordval ylow, yhigh;
#if (defined(_Windows) && !defined(WIN32)) || (defined(MSDOS) && defined(__TURBOC__))
    char pad[10];
#endif
  };

struct curve_points
  {
    struct curve_points *next_cp;
    enum PLOT_TYPE plot_type;
    enum PLOT_STYLE plot_style;
    char *title;
    int line_type;
    int point_type;
    int p_max;
    int p_count;
    struct coordinate GPHUGE *points;
  };

struct gnuplot_contours
  {
    struct gnuplot_contours *next;
    struct coordinate GPHUGE *coords;
    char isNewLevel;
    char label[12];
    int num_pts;
  };

struct iso_curve
  {
    struct iso_curve *next;
    int p_max;
    int p_count;
    struct coordinate GPHUGE *points;
  };

struct surface_points
  {
    struct surface_points *next_sp;
    enum PLOT_TYPE plot_type;
    enum PLOT_STYLE plot_style;
    char *title;
    int line_type;
    int point_type;
    int has_grid_topology;
    int num_iso_r;

    struct gnuplot_contours *contours;
    struct iso_curve *iso_crvs;
  };

struct TERMENTRY
  {
    char *name;
#if defined(_Windows) && !defined(WIN32)
    char GPFAR description[80];
#else
    char *description;
#endif
    unsigned int xmax, ymax, v_char, h_char, v_tic, h_tic;
    FUNC_PTR options, init, reset, text, scale, graphics, move, vector,
      linetype, put_text, text_angle, justify_text, point, flech;
  };

#ifdef _Windows
#define termentry TERMENTRY far
#else
#define termentry TERMENTRY
#endif

struct text_label
  {
    struct text_label *next;
    int tag;
    double x, y, z;
    enum JUSTIFY pos;
    char text[MAX_LINE_LEN + 1];
  };

struct flech_def
  {
    struct flech_def *next;
    int tag;
    double sx, sy, sz;
    double ex, ey, ez;
    TBOOLEAN head;
  };

struct ticdef
  {
    int type;
#define TIC_COMPUTED 1
#define TIC_SERIES 2
#define TIC_USER 3
#define TIC_MONTH 4
#define TIC_DAY 5
    union
      {
	struct
	  {
	    double start, incr;
	    double end;
	  }
	series;
	struct ticmark *user;
      }
    def;
  };

struct ticmark
  {
    double position;
    char *label;
    struct ticmark *next;
  };

#ifdef	vms
#include		<ssdef.h>
#include		<stsdef.h>
#define	IO_SUCCESS	(SS$_NORMAL | STS$M_INHIB_MSG)
#define	IO_ERROR	SS$_ABORT
#endif

#ifndef	IO_SUCCESS

#define	IO_SUCCESS	0
#endif
#ifndef	IO_ERROR
#define	IO_ERROR	1
#endif

extern TBOOLEAN screen_ok;
extern TBOOLEAN term_init;
extern TBOOLEAN undefined;
extern struct termentry term_tbl[];

extern char *alloc ();
extern char GPFAR *gpfaralloc ();
extern char GPFAR *gpfarrealloc ();
extern void gpfarfree ();

extern struct curve_points *cp_alloc ();
extern int cp_extend ();
extern int cp_free ();

extern struct surface_points *sp_alloc ();
extern int sp_replace ();
extern int sp_free ();

extern struct iso_curve *iso_alloc ();
extern int iso_extend ();
extern int iso_free ();

#ifdef _Windows
#include "win/wtext.h"
#endif
