/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "routine.h"
#include <string.h>

extern double getidval2 (int a);

Routine *
newroutine (id, nparam, paramid)
     int id, nparam;
     int *paramid;
{
  int i;

  Routine *nr = (Routine *) absmalloc (sizeof (Routine), "newroutine:nr ");
  if (nr == NULL)
    fprintf (stderr, "routine not allocated!");

  nr->nlc = 0;
  nr->endl = 0;
  nr->nl = 0;
  nr->acl = 0;

  nr->nloop = 0;
  nr->nif = 0;
  nr->subid = id;
  nr->subnparam = nparam;
  nr->sourcecode = NULL;
  nr->dimsource = 0;
  nr->ifdeep = 0;
  nr->maxdeep = 0;
  for (i = 0; i < nparam; i++)
    nr->subparamid[i] = paramid[i];
  return nr;
}

routine_delete (nr)
     Routine *nr;
{
  routine_freecode (nr);
  absfree (nr, "routine_delete:nr ");

}

routine_print (nr)
     Routine *nr;
{
  int i;

  printf ("Sub routine %s\n", id2str (nr->subid));
  printf ("number of lines:    %d\n", nr->nlc);
  printf ("number of ifblocks: %d\n", nr->nif);

  for (i = 0; i < nr->nif; i++)
    printf ("ifblock %d: IF @ %d ELSE @ %d ENDIF @ %d\n",
	    i,
	    nr->ifblks[i].ifline,
	    nr->ifblks[i].elseline,
	    nr->ifblks[i].endifline);

}

routine_addlinecode (nr, buf)
     Routine *nr;
     char *buf;
{
  char **tmp;
  int i;
  if (nr == NULL || buf == NULL)
    return;

  if (nr->nlc >= nr->dimsource)
    {
      nr->dimsource += 10;
      nr->sourcecode = (char **) absrealloc
	(nr->sourcecode, sizeof (char *) * (nr->dimsource), "routine_addlinecode:nr->sourcecode ");
      if (!nr->sourcecode)
	fprintf (stderr, "Not enougth memory in routine_addlinecode (dimsource = %d)\n", nr->dimsource);

    }
  nr->sourcecode[nr->nlc] = (char *) absmalloc (sizeof (char) * (strlen (buf) + 3), "routine_addlinecode:nr->sourcecode[nr->nlc] ");
  if (!nr->sourcecode[nr->nlc])
    fprintf (stderr, "Not enougth memory in routine_addlinecode (dimsource = %d)\n", nr->dimsource);

  strcpy (nr->sourcecode[nr->nlc], buf);

  if (strchr (nr->sourcecode[nr->nlc], '\n') == NULL)
    strcat (nr->sourcecode[nr->nlc], "\n\0");
  nr->acl = nr->nlc;
  nr->nlc++;

}

routine_freecode (nr)
     Routine *nr;
{
  int i;
  if (nr == NULL)
    return;
  for (i = 0; i < nr->nlc; i++)
    {
      if (nr->sourcecode[i] != NULL)
	absfree (nr->sourcecode[i], "routine_freecode:nr->sourcecode[i] ");
    }
  if (nr->sourcecode != NULL)
    absfree (nr->sourcecode, "routine_freecode:nr->sourcecode ");
}

routine_enterloop (nr, id, start, end, step)
     Routine *nr;
     int id;
     double start, end, step;
{

  if (nr->nloop <= nr->nl)
    {
      nr->nl++;

    }

  nr->loops[nr->nloop].loopid = id;
  nr->loops[nr->nloop].loopend = end;
  nr->loops[nr->nloop].loopstep = step;
  nr->loops[nr->nloop].loopline = nr->acl;

  setidval2 (id, start);

  nr->nloop++;
}

int
routine_nextloop (nr, id)
     Routine *nr;
     int id;
{
  double idval = getidval2 (nr->loops[nr->nloop - 1].loopid);
  double step = nr->loops[nr->nloop - 1].loopstep;
  double end = nr->loops[nr->nloop - 1].loopend;

  if (step > 0 && idval + step > end)
    {
      nr->nloop--;
      return 0;
    }
  if (step < 0 && idval + step < end)
    {
      nr->nloop--;
      return 0;
    }

  idval = idval + step;

  setidval2 (nr->loops[nr->nloop - 1].loopid, idval);

  nr->acl = nr->loops[nr->nloop - 1].loopline;

}

int
routine_addif (nr)
     Routine *nr;
{
  int line;
  int actif;

  line = nr->acl;

  nr->ifdeep++;

  nr->maxdeep++;

  actif = nr->nif + nr->ifdeep - 1;
  nr->ifblks[actif].ifline = line;
  nr->ifblks[actif].elseline = 0;
  nr->ifblks[actif].endifline = 0;
}

int
routine_addelse (nr)
     Routine *nr;
{
  int line;
  int actif;
  line = nr->acl;

  actif = nr->nif + nr->ifdeep - 1;
  nr->ifblks[actif].elseline = line;
}

int
routine_addendif (nr)
     Routine *nr;
{
  int actif;
  int line;
  line = nr->acl;
  actif = nr->nif + nr->ifdeep - 1;

  nr->ifblks[actif].endifline = line;
  nr->ifdeep--;
  if (nr->ifdeep == 0)
    {
      nr->nif += nr->maxdeep;
      nr->maxdeep = 0;
    }
}

int
routine_addreturn (nr)
     Routine *nr;
{
  nr->nif += nr->maxdeep;
  nr->maxdeep = 0;
  nr->endl = nr->acl;
}

int
routine_gonotif (nr)
     Routine *nr;

{
  int i = 0;
  int line = nr->acl;

  while (nr->ifblks[i].ifline != line && i < nr->nif + 1)
    i++;

  if (nr->ifblks[i].elseline)
    line = nr->ifblks[i].elseline;
  else
    line = nr->ifblks[i].endifline;

  nr->acl = line;

}

int
routine_goendif (nr)
     Routine *nr;
{
  int i = 0;
  int line = nr->acl;

  while (nr->ifblks[i].elseline != line && i < nr->nif + 1)
    i++;
  nr->acl = nr->ifblks[i].endifline;

}
