/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "stack.h"
#include "stdlib.h"
#include "y.tab.h"
#include "stdio.h"

stack *
stack_n (dim)
     int dim;
{
  stack *s;
  s = (stack *) absmalloc (sizeof (stack), "stack_n:s ");
  if (dim > 0)
    {
      s->matrix = (obj *) absmalloc (sizeof (obj) * dim, "stack_n:s->matrix ");
      s->dim = dim;
    }
  else
    {
      s->matrix = NULL;
      s->dim = 0;
    }

  s->pos = 0;
  return s;
}

stack_delete (s)
     stack *s;
{
  if (s == NULL)
    return;
  if (s->matrix == NULL)
    return;
  absfree (s->matrix, "stack_delete:s->matrix ");
  absfree (s, "stack_delete:s ");
  return;
}

stack_flush (s)
     stack *s;
{
  int i;

  if (s == NULL)
    return;
  if (s->matrix == NULL)
    return;

  s->pos = 0;
  return;
}

stack *
stack_push (s, o)
     stack *s;
     obj o;
{
  if (s == NULL)
    s = stack_n (100);
  if (s->dim == s->pos)
    {
      s->matrix = (obj *) realloc (s->matrix, sizeof (obj) * (s->dim + 1));
      (s->dim)++;
    }
  s->matrix[s->pos] = o;
  (s->pos)++;

  return s;
}

obj
stack_pop (s)
     stack *s;
{
  if (s == NULL)
    {
      return;
    }
  (s->pos)--;

  if (s->pos < 0)
    {
      fprintf (stderr, "empty stack for pop!\n");
      return s->matrix[0];
    }

  return s->matrix[s->pos];
}

int
stack_print (s)
     stack *s;
{
  int i;
  obj o;
  for (i = 0; i < s->pos; i++)
    {
      o = s->matrix[i];
      switch (o.type)
	{
	case STRING_CONSTANT:
	  printf ("%d type %d val %s\n", i, o.type, o.rec.s);
	  break;
	case INTEGER_CONSTANT:
	  printf ("%d type %d val %d\n", i, o.type, o.rec.i);
	  break;
	case FLOATING_CONSTANT:
	  printf ("%d type %d val %lf\n", i, o.type, o.rec.d);
	  break;
	}

    }
}
