/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "y.tab.h"
#include "toto.h"
#include "stack.h"

#include "cl.h"
#include "wkb.h"
#include "wks.h"
#include "range.h"

static char tokenbuf[512];
static int retint;
static double retdouble;

static idrec idt[1024];

static int numid = 0;
static int activid = 0;
static int r = 0;
static int c = 0;
static int inmacro = 0;
int
isinmacro ()
{
  return inmacro;
};
static int debug = 0;
static char *lasttoken;
static Cl **depend = NULL;
static int ndep = 0;
static int maxdep = 0;
extern int *getargnum ();

static char *os[256];
static int is[256];
static double ds[256];
static int ot[256];
static int op = 0;

static char newstring[1024];
static int di;
static int dj;
static int nodollars;
static int curi;
static int curj;
static int REFUSE_NEXT = 0;
static char *NULL_STRING = " ";

int
adddep (cl)
     Cl *cl;
{
  if (maxdep == ndep)
    {
      depend = (Cl **) realloc (depend, sizeof (Cl *) * (5 + maxdep));
      maxdep += 5;
    }

  depend[ndep] = cl;
  ndep++;
}

int
cleardep ()
{
  if (maxdep > 0)
    {
      absfree (depend, "cleardep:depend ");
      depend = NULL;
      maxdep = 0;
      ndep = 0;
    }
}

int
setmacro ()
{
  inmacro = 1;
}
int
unsetmacro ()
{
  inmacro = 0;
}

error_in_parser ()
{
  if (inmacro)
    error_in_macro ();
}

int
setdebug ()
{
  debug = 1;
};

flushos ()
{
  int i;
  for (i = 0; i < op; i++)
    {

      os[i] = NULL;
      is[i] = 0;
      ds[i] = 0;
      ot[i] = 0;
    }
  op = 0;
  cleardep ();
}

initobject (a, b)
     int a, b;
{
  printf ("init obj %d := %d\n", a, b);
  switch (ot[a])
    {
    case SOURCE:
      switch (ot[b])
	{
	case RANGE:
	  printf ("source := range\n");
	  break;
	}
    }
}

dereference (a, b)
     int a, b;
{

  return b;
}

int
putdouble (v)
     double v;
{
  ot[op] = FLOATING_CONSTANT;
  ds[op] = v;
  op++;
  return op - 1;
}

double
getdouble ()
{
  op--;
  return ds[op];
}

int
putwkb (v)
     double v;
{
  int i = v;
  ot[op] = WORKBOOK_OBJ;
  os[op] = (char *) applic_gwkb (i);
  op++;
  printobs ();
  return op - 1;
}

int
putwkb2 (buf)
     char *buf;
{

  ot[op] = WORKBOOK_OBJ;
  os[op] = (char *) applic_gwkb (buf);
  op++;
  printobs ();
  return op - 1;
}

int
putwks (v)
     double v;
{
  int i = v;
  ot[op] = WORKSHEET_OBJ;

  if (v < 0)
    {
      os[op] = (char *) ActivWks;
      op++;
      printobs ();
      return op - 1;
    }

  if (op > 0)
    {
      if (ot[op - 1] == WORKBOOK_OBJ)
	{
	  os[op] = (char *) wkb_gsht2 ((Wkb *) os[op - 1], i);
	  op++;
	  printobs ();
	  return op - 1;
	}
    }
  os[op] = (char *) wkb_gsht2 (ActivWkb, i);
  op++;
  printobs ();
  return op - 1;
}

int
putwks2 (buf)
     char *buf;
{

  ot[op] = WORKSHEET_OBJ;
  if (op > 1)
    {
      if (ot[op - 2] == WORKBOOK_OBJ)
	{
	  os[op] = (char *) wkb_gsht ((Wkb *) os[op - 2], buf);
	  op++;
	  printobs ();
	  return op - 1;
	}
    }
  os[op] = (char *) wkb_gsht (ActivWkb, buf);
  op++;
  printobs ();
  return op - 1;
}

int
putwks3 ()
{

  ot[op] = WORKSHEET_OBJ;
  os[op] = NULL;
  op++;
  printobs ();
  return op - 1;
}

int
putcolumn (buf)
     char *buf;
{
  char col[10];
  int i, j;

  ot[op] = COLUMNS;
  sprintf (col, "%s1", buf);
  alphatonum (col, &i, &j);
  is[op] = j;
  op++;
  printobs ();
  return op - 1;

}

int
putrow (buf)
     char *buf;
{
  char col[10];
  int i, j;

  ot[op] = ROWS;
  sscanf (buf, "%d", &i);
  is[op] = i;
  op++;
  printobs ();
  return op - 1;

}

int
putcht (v)
     int v;
{

  ot[op] = CHART;

  if (v < 0)
    {
      os[op] = (char *) ActivChrt;
      op++;
      printobs ();
      return op - 1;
    }
  if (op > 1)
    {
      if (ot[op - 1] == WORKSHEET_OBJ)
	{
	  os[op] = (char *) wks_gcht ((Wks *) os[op - 1], v);
	  op++;
	  printobs ();
	  return op - 1;
	}
    }
  os[op] = (char *) wks_gcht (ActivWks, v);
  op++;
  printobs ();
  return op - 1;
}

putserie (val)
     double val;
{
  int v = val;

  ot[op] = SERIE;
  is[op] = v;
  op++;

  printobs ();
  return op - 1;
}

putaxes (val)
     double val;
{
  int v = val;

  ot[op] = AXES;
  is[op] = v;
  op++;

  printobs ();
  return op - 1;
}

int
putcl (rr, cc)
     double rr, cc;
{
  int i = rr;
  int j = cc;
  Cl *cl;

  ot[op] = CELL_OBJ;
  if (op > 0)
    {
      if (ot[op - 1] == WORKSHEET_OBJ)
	{
	  os[op] = (char *) wks_gcl ((Wks *) os[op - 1], i, j, 1);
	  op++;
	  printobs ();
	  return op - 1;
	}
    }
  cl = wks_gcl (ActivWks, i, j, 1);
  os[op] = (char *) cl;
  op++;
  printobs ();

  return op - 1;
}

int
putcl2 (ws)
     int ws;
{
  int i = r;
  int j = c;
  Cl *cl;

  if (!ws)
    {
      cl = wks_gcl (ActivWks, r, c, 1);
    }
  else
    {
      cl = wks_gcl (
		     wkb_gsht (ActivWkb, idt[activid].name), r, c, 1);
    }

  ot[op] = CELL_OBJ;
  os[op] = (char *) cl;
  op++;
  printobs ();

  return op - 1;
}

int
putrange (buf)
     char *buf;
{
  Range *ran;
  ot[op] = RANGE;
  if (buf)
    os[op] = (char *) newrange (buf);
  else
    os[op] = NULL;

  op++;
  printobs ();

  return op - 1;
}

int
getrangeval (index, val)
     int index;
     double *val;
{
  int r1, c1, r2, c2;
  int rr, cc;
  int i;
  Cl *cl1;
  Cl *cl2;
  Cl *cl;

  if (ot[op - 1] != RANGE || ot[op - 2] != CELL_OBJ || ot[op - 3] != CELL_OBJ)
    return 0;

  cl1 = (Cl *) os[op - 3];
  cl2 = (Cl *) os[op - 2];

  r1 = cl1->r;
  c1 = cl1->c;
  r2 = cl2->r;
  c2 = cl2->c;

  if (r2 < r1)
    {
      rr = r2;
      r2 = r1;
      r1 = rr;
    }
  if (c2 < c1)
    {
      cc = c2;
      c2 = c1;
      c1 = cc;
    }

  if (index >= (r2 - r1 + 1) * (c2 - c1 + 1) || index < 0)
    return 0;
  rr = r1;
  cc = c1;

  rr = index / (c2 - c1 + 1) + r1;
  cc = index - (rr - r1) * (c2 - c1 + 1) + c1;

  cl = wks_gcl (ActivWks, rr, cc, 1);
  *val = cl_gvalue (cl);

  adddep (cl);

  return 1;
}

int
putform ()
{
  Cl *cl;

  ot[op] = FORMULA;

  if (op > 0)
    {
      if (ot[op - 1] == CELL_OBJ)
	{
	  os[op] = (char *) cl_gform ((Cl *) os[op - 1]);
	  op++;
	  printobs ();
	  return op - 1;
	}
    }
  os[op] = NULL;
  op++;
  printobs ();
  return op - 1;
}

int
putnumberformat ()
{
  Cl *cl;

  ot[op] = NUMBERFORMAT;

  if (op > 0)
    {
      if (ot[op - 1] == CELL_OBJ)
	{
	  os[op] = (char *) cl_gnumberformat ((Cl *) os[op - 1]);
	  op++;
	  printobs ();
	  return op - 1;
	}
    }
  os[op] = NULL;
  op++;
  printobs ();
  return op - 1;
}

int
putobj (v)
     int v;
{
  ot[op] = v;
  os[op] = NULL;
  op++;
  printobs ();
  return op - 1;
}

int
putbord (val)
     double val;
{
  int v = val;
  ot[op] = BORDERS;
  is[op] = v;
  op++;
  printobs ();
  return op - 1;
}

int
putstring ()
{
  char *quote1 = strchr (lasttoken, '"');
  char *quote2 = strrchr (lasttoken, '"');

  if (quote1 == NULL || quote2 == NULL || quote1 == quote2)

    {
      ot[op] = STRING_CONSTANT;
      os[op] = NULL_STRING;
      return;
    }

  quote1++;
  if (quote1 >= quote2)
    {
      ot[op] = STRING_CONSTANT;
      os[op] = NULL_STRING;
      return;
    }

  ot[op] = STRING_CONSTANT;
  os[op] = (char *) absmalloc (sizeof (char) * (quote2 - quote1 + 2), "putstring:os[op] ");
  if (os[op] == NULL)
    fprintf (stderr, "os[op] not allocated!\n");
  quote2[0] = '\0';
  sprintf (os[op], "%s", quote1);
  op++;
  printobs ();
  return op - 1;
}

char *
getstackstr (v)
     int v;
{
  if (v < 0 || v >= op)
    return NULL;
  return os[v];
}

int
putidstring ()
{
  char *str = idt[activid].str;
  ot[op] = STRING_CONSTANT;

  if (str == NULL)
    os[op] = NULL_STRING;
  else
    {
      os[op] = (char *) absmalloc (sizeof (char) * (strlen (idt[activid].str) + 1), "putidstring:os[op] ");
      if (os[op] == NULL)
	fprintf (stderr, "os[op] not allocated!\n");
      strcpy (os[op], str);
    }
  op++;
  printobs ();
  return op - 1;
}

int
printobs ()
{
  int i;
  if (!debug)
    return 0;
  for (i = 0; i <= op; i++)
    {
      switch (ot[i])
	{
	case WORKBOOK_OBJ:
	  printf ("%d: WORKBOOK_OBJ  	ref %d\n", i, os[i]);
	  break;
	case WORKSHEET_OBJ:
	  printf ("%d: WORKSHEET_OBJ 	ref %d\n", i, os[i]);
	  break;
	case CELL_OBJ:
	  printf ("%d: CELL_OBJ      	ref %d\n", i, os[i]);
	  break;
	case FORMULA:
	  printf ("%d: FORMULA       	ref %d\n", i, os[i]);
	  break;
	case NUMBERFORMAT:
	  printf ("%d: NUMBERFORMAT          ref %d\n", i, os[i]);
	  break;
	case STRING_CONSTANT:
	  printf ("%d: STRING_CONSTANT  	val %s\n", i, os[i]);
	  break;
	case BORDERS:
	  printf ("%d: BORDERS       	val %d\n", i, is[i]);
	  break;
	case LINESTYLE:
	  printf ("%d: LINESTYLE           \n", i);
	  break;
	case RANGE:
	  printf ("%d: RANGE         	      \n", i);
	  break;
	case CHARTOBJECTS:
	  printf ("%d: CHARTOBJECTS  	      \n", i);
	  break;
	case SERIE:
	  printf ("%d: SERIE                    val %d\n", i, is[i]);
	  break;
	case CHART:
	  printf ("%d: CHART         	      \n", i, os[i]);
	  break;
	case SOURCE:
	  printf ("%d: SOURCE        	      \n", i);
	  break;
	case NAME:
	  printf ("%d: NAME          	      \n", i);
	  break;
	case VALUES:
	  printf ("%d: VALUES        	      \n", i);
	  break;
	case CHARTTYPE:
	  printf ("%d: VALUES                 \n", i);
	  break;
	}
    }
  printf ("-------\n");
}

int
setobjectval (object, val)
     int object;
     double val;
{
  char str[512];
  int v = val;
  sprintf (str, "%lf\n", val);

  switch (ot[object])
    {
    case CELL_OBJ:
      {
	unsetmacro ();
	cl_setform ((Cl *) os[object], str);
	setmacro ();
	break;
      }
    case WORKSHEET_OBJ:
      break;
    case WORKBOOK_OBJ:
      break;
    case CHARTTYPE:
      {
	if (ot[object - 1] == CHART)
	  chrt_settype ((Chrt *) os[object - 1], v);
	break;
      }
    case HASMAJORGRIDLINES:
      {
	if (ot[object - 1] == AXES && ot[object - 2] == CHART)
	  chrt_setgrid ((Chrt *) os[object - 2], is[object - 1], v);
	break;
      }
    case HASTITLE:
      {
	if (ot[object - 1] == AXES && ot[object - 2] == CHART)
	  {
	  }
	if (ot[object - 1] == CHART)
	  {
	  }
	break;
      }

    case BOLD:
      {
	if (ot[object - 1] == FONT && ot[object - 2] == CELL_OBJ)
	  cl_fblt ((Cl *) os[object - 2], v);
	break;
      }
    case ITALIC:
      {
	if (ot[object - 1] == FONT && ot[object - 2] == CELL_OBJ)
	  cl_fital ((Cl *) os[object - 2], v);
	break;
      }
    case SIZE:
      {
	if (ot[object - 1] == FONT && ot[object - 2] == CELL_OBJ)
	  cl_fontsize ((Cl *) os[object - 2], v);
	break;
      }
    case HORIZONTALALIGNMENT:
      {
	if (ot[object - 1] == CELL_OBJ)
	  cl_setjust ((Cl *) os[object - 1], v);
	break;
      }
    case COLORINDEX:
      {
	if (ot[object - 1] == FONT && ot[object - 2] == CELL_OBJ)
	  cl_setfg ((Cl *) os[object - 2], v);
	if (ot[object - 1] == INTERIOR && ot[object - 2] == CELL_OBJ)
	  cl_setbg ((Cl *) os[object - 2], v);
	break;
      }
    case COLUMNWIDTH:
      {
	if (ot[object - 1] == COLUMNS)
	  wks_setcw (ActivWks, is[object - 1], v);
	break;
      }
    case ROWHEIGHT:
      {
	if (ot[object - 1] == ROWS)
	  wks_setrh (ActivWks, is[object - 1], v);
	break;
      }
    case LINESTYLE:
      {
	switch (ot[object - 1])
	  {
	  case BORDERS:
	    {
	      switch (ot[object - 2])
		{
		case CELL_OBJ:
		  cl_setbord ((Cl *) os[object - 2], is[object - 1], v);
		  break;
		}
	    }
	  }
      }
    }

}

int
execsystem (buf)
     char *buf;
{
  system (buf);
}

int
setobjectstring (object, str)
     int object;
     char *str;
{
  char buf[256];
  if (str == NULL)
    buf[0] = '\0';
  else if (strlen (str) < 1)
    buf[0] = '\0';
  else
    strncpy (buf, str, 255);

  switch (ot[object])
    {
    case FORMULA:
      {
	unsetmacro ();
	cl_setform ((Cl *) os[object - 1], buf);
	setmacro ();
	op--;
	op--;
	break;
      }
    case NUMBERFORMAT:
      {
	unsetmacro ();
	cl_setnumberformat ((Cl *) os[object - 1], buf);
	setmacro ();
	op--;
	op--;
	break;
      }
    case CELL_OBJ:
      break;
    case WORKSHEET_OBJ:
      break;
    case WORKBOOK_OBJ:
      break;
    case NAME:
      {
	if (ot[object - 1] == SERIE && ot[object - 2] == CHART)
	  chrt_setseriename ((Chrt *) os[object - 2], is[object - 1], buf);
	if (ot[object - 1] == FONT && ot[object - 2] == CELL_OBJ)
	  cl_fname ((Cl *) os[object - 2], buf);
	if (ot[object - 1] == WORKSHEET_OBJ)
	  wks_setname ((Wks *) os[object - 1], buf);
	break;
      }
    case VALUES:
      {
	if (ot[object - 1] == SERIE && ot[object - 2] == CHART)
	  chrt_setserievalues ((Chrt *) os[object - 2], is[object - 1], buf);
	break;
      }
    case TEXT:
      {
	if (ot[object - 1] == AXISTITLE && ot[object - 2] == AXES && ot[object - 3] == CHART)
	  {
	    chrt_setaxistitle ((Chrt *) os[object - 3], is[object - 2], buf);
	  }
	if (ot[object - 1] == CHARTTITLE && ot[object - 2] == CHART)
	  {
	    title_settext (&((Cht *) os[object - 2])->ChtTitle, buf);
	  }
	break;
      }
    }

}

double
getobjectval (object)
     int object;
{

  switch (ot[object])
    {
    case CELL_OBJ:
      return cl_gvalue ((Cl *) os[object]);
      break;
    case WORKSHEET_OBJ:
      break;
    case WORKBOOK_OBJ:
      break;
    }

}

double
memberfct (s, object, member, n)
     stack *s;
     int object, member;
     int n;
{
  double darg[10];
  int iarg[10];
  int i;
  obj o;

  for (i = 0; i < n; i++)
    {
      o = stack_pop (s);

      darg[n - 1 - i] = o.rec.d;
      iarg[n - 1 - i] = o.rec.d;
    }

  if (debug)
    printf ("member fct object val ref %d member %d \n", os[object], member);
  printobs ();

  switch (ot[object])
    {
    case CELL_OBJ:
      switch (member)
	{
	case CREATE:
	  break;
	case ACTIVATE:
	  break;
	}
      break;

    case RANGE:
      switch (member)
	{
	case SELECT:
	  {
	    char *i1j1;
	    char *i2j2;
	    char *separ;
	    Range *ran = (Range *) os[object];
	    int i1, j1, i2, j2;
	    i1 = ran->plag[0].r1;
	    j1 = ran->plag[0].c1;
	    i2 = ran->plag[0].r2;
	    j2 = ran->plag[0].c2;
	    setselection (i1, j1, i2, j2);

	  };
	  break;
	}
      break;

    case SELECTION:
      switch (member)
	{
	case PRINTOUT:
	  {
	    printfl ("output.fig");
	  };
	  break;
	}
      break;

    case WORKSHEET_OBJ:
      switch (member)
	{
	case ADD:
	  wkb_as (ActivWkb, NULL);
	  break;
	case CREATE:
	  printf (".create\n");
	  break;
	case ACTIVATE:
	  ActivateWks ((Wks *) os[object]);
	  break;
	}
      break;

    case WORKBOOK_OBJ:
      switch (member)
	{
	case CREATE:
	  break;
	case ACTIVATE:
	  break;
	}
      break;

    case SHAPES:
      switch (member)
	{
	case ADDLINE:
	  {
	    if (op > 2)
	      if (ot[op - 1] == WORKSHEET_OBJ)
		{
		  wks_addshape ((Wks *) os[op - 1], iarg, n);
		  break;
		}
	    wks_addshape (ActivWks, iarg, n);
	    break;

	  }
	  break;
	case ADDSHAPE:
	  {
	    if (op > 2)
	      if (ot[op - 1] == WORKSHEET_OBJ)
		{
		  wks_addshape ((Wks *) os[op - 1], iarg, n);
		  break;
		}
	    wks_addshape (ActivWks, iarg, n);
	    break;
	  }
	  break;
	}
      break;

    case CHARTOBJECTS:
      switch (member)
	{
	case ADD:
	  {
	    if (op > 2)
	      if (ot[op - 1] == WORKSHEET_OBJ)
		{
		  wks_addcht ((Wks *) os[op - 1], iarg, n);
		  break;
		}
	    wks_addcht (ActivWks, iarg, n);
	    break;
	  }
	case SETSOURCEDATA:
	  break;
	case CREATE:
	  break;
	case ACTIVATE:
	  break;
	}
      break;

    }
  op--;
  return 0.0;
}

int
setdi (v)
     int v;
{
  di = v;
}

int
setdj (v)
     int v;
{
  dj = v;
}

int
freenewstring ()
{
  sprintf (newstring, "");
}

int
lennewstring ()
{
  return strlen (newstring);
}

char *
getnewstring ()
{
  return newstring;
}

int
setnodollars (v)
     int v;
{
  nodollars = v;
}

int
catnewstring (buf)
     char *buf;
{
  lasttoken = buf;
  if (!REFUSE_NEXT)
    sprintf (newstring, "%s%s", newstring, buf);
  else
    REFUSE_NEXT = 0;
}

lexecho ()
{
  if (newstring[0] != '\n' && strlen (newstring) > 1)
    printf ("error:%s<-\n", newstring);
}

void
FillTokenBuf (char *c)
{

  sprintf (tokenbuf, "%s", c);

}

int
getint ()
{
  sscanf (tokenbuf, "%d", &retint);
  return retint;
}

double
getfloat ()
{
  sscanf (tokenbuf, "%lf", &retdouble);
  return retdouble;
}

double
getidval ()
{
  return idt[activid].val;
}

double
getidval2 (v)
     int v;
{
  return idt[v].val;
}

int
obj2int (o)
     obj o;
{
  switch (o.type)
    {
    case FLOATING_CONSTANT:
      return (int) o.rec.d;
    case INTEGER_CONSTANT:
      return o.rec.i;
    case STRING_CONSTANT:
      return o.rec.s[0];
    }
  return 0;
}

double
obj2double (o)
     obj o;
{
  switch (o.type)
    {
    case FLOATING_CONSTANT:
      return o.rec.d;
    case INTEGER_CONSTANT:
      return (double) o.rec.i;
    case STRING_CONSTANT:
      return (double) o.rec.s[0];
    }
  return 0.0;
}

static char tmp[64];

char *
obj2string (o)
     obj o;
{

  switch (o.type)
    {
    case FLOATING_CONSTANT:
      {
	sprintf (tmp, "%lf", o.rec.d);
	return tmp;
      }
    case INTEGER_CONSTANT:
      {
	sprintf (tmp, "%d", o.rec.i);
	return tmp;
      }
    case STRING_CONSTANT:
      return o.rec.s;
    }
  return NULL;
}

obj
getidval3 (v)
     int v;
{
  obj o;
  o.type = idt[v].type;
  if (o.type == STRING_CONSTANT)
    {
      o.rec.s = idt[v].str;
    }
  if (o.type == INTEGER_CONSTANT)
    {
      o.rec.i = idt[v].i;
    }
  if (o.type == FLOATING_CONSTANT)
    {
      o.rec.d = idt[v].val;
    }
  return o;
}

int
getidpos ()
{
  return activid;
}

int
clistext ()
{
  Cl *cl2;

  cl2 = wks_gcl (ActivWks, r, c, 1);
  adddep (cl2);

  return cl_istext (cl2);
}

int
wsclistext ()
{
  Cl *cl2;

  cl2 = wks_gcl (
		  wkb_gsht (ActivWkb, idt[activid].name), r, c, 1);
  adddep (cl2);

  return cl_istext (cl2);
}

double
getclval ()
{
  Cl *cl2;

  cl2 = wks_gcl (ActivWks, r, c, 1);

  return cl_gvalue (cl2);

}

double
wsgetclval ()
{
  Cl *cl2;

  cl2 = wks_gcl (
		  wkb_gsht (ActivWkb, idt[activid].name), r, c, 1);

  return cl_gvalue (cl2);

}

int
clputtext ()
{
  Cl *cl2;

  cl2 = wks_gcl (ActivWks, r, c, 1);

  ot[op] = STRING_CONSTANT;
  os[op] = cl_gtext (cl2);
  op++;
  return op - 1;
}

int
wsclputtext ()
{
  Cl *cl2;

  cl2 = wks_gcl (
		  wkb_gsht (ActivWkb, idt[activid].name), r, c, 1);

  ot[op] = STRING_CONSTANT;
  os[op] = cl_gtext (cl2);
  op++;
  return op - 1;
}

int
setval (val)
     double val;
{
  int i;

  if (!inmacro)
    {
      cl_setvalue (ActivCl, val);
      for (i = 0; i < ndep; i++)
	cl1isfcl2 (ActivCl, depend[i]);
      cleardep ();
    }
}

int
settext (buf)
     char *buf;
{
  int i;
  if (!inmacro)
    {
      cl_settext (ActivCl, buf);
      for (i = 0; i < ndep; i++)
	cl1isfcl2 (ActivCl, depend[i]);
      cleardep ();
    }
}

int
setidval (d)
     double d;
{
  idt[activid].val = d;
}

int
setidval2 (v, d)
     int v;
     double d;
{
  idt[v].val = d;
  idt[v].type = FLOATING_CONSTANT;
}

int
setidval3 (v, o)
     int v;
     obj o;
{
  idt[v].type = o.type;
  if (o.type == STRING_CONSTANT)
    {
      if (idt[v].str != NULL)
	absfree (idt[v].str, "setidval3:idt[v].str ");
      if (o.rec.s != NULL)
	{
	  idt[v].str = (char *) absmalloc (sizeof (char) * strlen (o.rec.s), "setidval3:idt[v].str ");
	  strcpy (idt[v].str, o.rec.s);
	}
      else
	idt[v].str = NULL;

    }
  if (o.type == INTEGER_CONSTANT)
    {

      idt[v].type = FLOATING_CONSTANT;
      idt[v].val = o.rec.i;
    }
  if (o.type == FLOATING_CONSTANT)
    {
      idt[v].val = o.rec.d;
    }
}

int
o1eqo2 (o1, o2)
     obj o1, o2;
{
  o1.type = o2.type;
  if (o1.type == FLOATING_CONSTANT)
    o1.rec.d = o2.rec.d;
  if (o1.type == INTEGER_CONSTANT)
    o1.rec.i = o2.rec.i;
  if (o1.type == STRING_CONSTANT)
    o1.rec.s = o2.rec.s;
}

int
printobj (o)
     obj o;
{
  if (o.type == FLOATING_CONSTANT)
    fprintf (stderr, "double %lf\n", o.rec.d);
  else if (o.type == INTEGER_CONSTANT)
    fprintf (stderr, "int %d\n", o.rec.i);
  else if (o.type == STRING_CONSTANT)
    fprintf (stderr, "str %s \n", o.rec.s);
  else
    fprintf (stderr, "not a valid type (%d)!\n", o.type);

}

int
setidstring (str)
     char *str;
{
  if (idt[activid].str != NULL)
    absfree (idt[activid].str, "setidstring:idt[activid].str ");
  idt[activid].str = (char *) absmalloc (sizeof (char) * strlen (str), "setidstring:idt[activid].str ");
  strcpy (idt[activid].str, str);
  idt[activid].type = STRING_CONSTANT;
  op--;
}

int
setidstring2 (v, str)
     int v;
     char *str;
{
  if (idt[v].str != NULL)
    absfree (idt[v].str, "setidstring2:idt[v].str ");
  idt[v].str = (char *) absmalloc (sizeof (char) * strlen (str), "setidstring2:idt[v].str ");
  strcpy (idt[v].str, str);
  idt[v].type = STRING_CONSTANT;
  op--;
}

int
check_type ()
{
  int i;
  int pos = 0;
  char buf[64];
  char *left;
  char *right;
  char *loc;
  int dr = di;
  int dc = dj;
  int dolr = 0;
  int dolc = 0;
  c = r = 0;

  for (i = 0; i < strlen (tokenbuf); i++)
    {
      if ('a' <= tokenbuf[i] && tokenbuf[i] <= 'z')
	tokenbuf[i] = 'A' + tokenbuf[i] - 'a';
    }

  if (('A' <= tokenbuf[0] &&
       tokenbuf[pos] <= 'Z' &&
       '0' <= tokenbuf[strlen (tokenbuf) - 1] &&
       tokenbuf[strlen (tokenbuf) - 1] <= '9') ||
      tokenbuf[0] == '$')

    {

      if (tokenbuf[0] == '$')
	{
	  dolc = 1;
	  pos++;
	  dc = 0;
	}
      while ('A' <= tokenbuf[pos] && tokenbuf[pos] <= 'Z')
	{
	  c *= 26;
	  c += (tokenbuf[pos] - 'A' + 1);
	  pos++;
	}

      if (tokenbuf[pos] == '$')
	{
	  dolr = 1;
	  pos++;
	  dr = 0;
	}
      sscanf (tokenbuf + pos, "%d", &r);

      if (di || dj)
	{
	  c += dc;
	  r += dr;
	  n2a2 (buf, r, c, dolr, dolc);
	  catnewstring (buf);
	  REFUSE_NEXT = 1;
	}
      return YY_CELL;
    }

  if (numid == 0)
    initid ();

  for (i = 0; i < numid; i++)
    {
      if (strcmp (idt[i].name, tokenbuf) == 0)
	{
	  activid = i;

	  return IDENTIFIER;

	}
    }

  idt[i].name = (char *) absmalloc (sizeof (char) * strlen (tokenbuf), "check_type:idt[i].name ");
  sprintf (idt[i].name, "%s", tokenbuf);
  idt[i].val = 0.0;
  idt[i].i = 0;
  idt[i].str = NULL;
  idt[i].type = INTEGER_CONSTANT;

  numid++;
  activid = i;
  return IDENTIFIER;

};

findid (buf)
     char *buf;
{
  int i;
  if (numid == 0)
    initid ();

  for (i = 0; i < numid; i++)
    {
      if (strcmp (idt[i].name, buf) == 0)
	return i;
    }
}

initid ()
{
  while (defaultid[numid].name != NULL)
    {
      idt[numid].name = defaultid[numid].name;
      idt[numid].type = defaultid[numid].type;
      idt[numid].val = defaultid[numid].val;
      idt[numid].str = defaultid[numid].str;
      numid++;
    }
}

char *
id2str (v)
     int v;
{
  return idt[v].name;
}
