/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include "widget_makerX11.h"

static String filename = NULL;

static Widget top;
static Widget selectsht[20];

static struct menuEntry popupMenuEntries[] =
{
  "Paste", mpst,
  "Cut", mcut,
  "Copy", mcpy,
  "Copy Down", mcpyd,
  "Copy Right", mcpyr,
};

Pixmap
make_bitmap_from_data (parent, bits, w, h)
     Widget parent;
     char *bits;
     int w, h;
{
  return
    XCreateBitmapFromData (XtDisplay (parent),
			DefaultRootWindow (XtDisplay (parent)), bits, w, h);
}

make_pixmap_from_data (parent, bits, bm)
     Widget parent;
     char **bits;
     Pixmap *bm;
{
  XpmAttributes xpm_attr;
  XpmColorSymbol none_color =
  {NULL, "None", (Pixel) 0};

  xpm_attr.valuemask = XpmSize | XpmCloseness;
  xpm_attr.closeness = 40000;
  xpm_attr.numsymbols = 1;
  xpm_attr.colorsymbols = &none_color;
  XtVaGetValues (parent, XtNbackground, &none_color.pixel, NULL);
  XpmCreatePixmapFromData (XtDisplay (parent),
	 DefaultRootWindow (XtDisplay (parent)), bits, bm, NULL, &xpm_attr);

  return;
}

Widget
make_button_pixmap (parent, name, pix_bits, ftccallback, param)
     Widget parent;
     char *name;
     char **pix_bits;
     void (*ftccallback) ();
     XtPointer param;
{
  Widget button;
  Pixmap bm;
  Arg args[20];
  register int n;

  n = 0;
  make_pixmap_from_data (parent, pix_bits, &bm);
  XtSetArg (args[n], XtNbitmap, bm);
  n++;
  XtSetArg (args[n], XtNlabel, "");
  n++;

  XtSetArg (args[n], XtNhighlightThickness, 1);
  n++;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  XtSetArg (args[n], XtNinternalHeight, 1);
  n++;
  XtSetArg (args[n], XtNinternalWidth, 1);
  n++;
  button = XtCreateManagedWidget (name, commandWidgetClass, parent, args, n);
  if (ftccallback != NULL)
    XtAddCallback (button, XtNcallback, ftccallback, param);
  return button;
}

Widget
make_bar_button_pixmap (menu_bar, name, pix_bits)
     Widget menu_bar;
     char *name;
     char **pix_bits;
{
  Pixmap bm;
  Arg args[20];
  register int n;

  n = 0;
  make_pixmap_from_data (menu_bar, pix_bits, &bm);
  XtSetArg (args[n], XtNbitmap, bm);
  n++;
  XtSetArg (args[n], XtNlabel, "");
  n++;

  XtSetArg (args[n], XtNhighlightThickness, 1);
  n++;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  XtSetArg (args[n], XtNinternalHeight, 1);
  n++;
  XtSetArg (args[n], XtNinternalWidth, 1);
  n++;
  return XtCreateManagedWidget
    (name, menuButtonWidgetClass, menu_bar, args, n);
}

Widget
make_repeater_pixmap (parent, name, pix_bits, ftccallback, param)
     Widget parent;
     char *name;
     char **pix_bits;
     void (*ftccallback) ();
     XtPointer param;
{
  Widget button;
  Pixmap bm;
  Arg args[20];
  register int n;

  n = 0;
  make_pixmap_from_data (parent, pix_bits, &bm);
  XtSetArg (args[n], XtNbitmap, bm);
  n++;
  XtSetArg (args[n], XtNlabel, "");
  n++;

  XtSetArg (args[n], XtNhighlightThickness, 1);
  n++;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  XtSetArg (args[n], XtNinternalHeight, 1);
  n++;
  XtSetArg (args[n], XtNinternalWidth, 1);
  n++;
  button = XtCreateManagedWidget (name, repeaterWidgetClass, parent, args, n);
  XtAddCallback (button, XtNcallback, ftccallback, param);
  return button;
}

Widget
make_toggle_pixmap (parent, name, pix_bits, ftccallback, param)
     Widget parent;
     char *name;
     char **pix_bits;
     void (*ftccallback) ();
     XtPointer param;
{
  Widget button;
  Pixmap bm;
  Arg args[20];
  register int n;

  n = 0;
  make_pixmap_from_data (parent, pix_bits, &bm);
  XtSetArg (args[n], XtNbitmap, bm);
  n++;
  XtSetArg (args[n], XtNlabel, "");
  n++;

  XtSetArg (args[n], XtNhighlightThickness, 1);
  n++;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  XtSetArg (args[n], XtNinternalHeight, 1);
  n++;
  XtSetArg (args[n], XtNinternalWidth, 1);
  n++;
  button = XtCreateManagedWidget (name, toggleWidgetClass, parent, args, n);
  if (ftccallback != NULL)
    XtAddCallback (button, XtNcallback, ftccallback, param);
  return button;
}

static first_check = 1;
static Pixmap check0;
static Pixmap check1;

void
changecheck (w, closure, call_data)
     Widget w;
     XtPointer closure, call_data;
{
  Arg args[1];
  Boolean on;

  XtSetArg (args[0], XtNstate, &on);
  XtGetValues (w, args, 1);

  if (on)
    XtSetArg (args[0], XtNbitmap, check1);
  else
    XtSetArg (args[0], XtNbitmap, check0);

  XtSetValues (w, args, 1);
}

setcheck (w, on)
     Widget w;
     int on;
{
  Arg args[2];

  XtSetArg (args[0], XtNstate, on);

  if (on)
    XtSetArg (args[1], XtNbitmap, check1);
  else
    XtSetArg (args[1], XtNbitmap, check0);

  XtSetValues (w, args, 2);

}

getcheck (w)
     Widget w;
{
  Boolean on;
  Arg args[1];
  XtSetArg (args[0], XtNstate, &on);
  XtGetValues (w, args, 1);
  return on;
}

Widget
make_check (parent, name, ftccallback, param)
     Widget parent;
     char *name;
     void (*ftccallback) ();
     XtPointer param;
{
  Widget button;
  Pixmap bm;
  Arg args[20];
  register int n;

  if (first_check)
    {
      make_pixmap_from_data (parent, check0_xpm, &check0);
      make_pixmap_from_data (parent, check1_xpm, &check1);
      first_check = 0;
    }

  n = 0;
  XtSetArg (args[n], XtNbitmap, check0);
  n++;
  XtSetArg (args[n], XtNlabel, "");
  n++;

  XtSetArg (args[n], XtNhighlightThickness, 0);
  n++;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  XtSetArg (args[n], XtNinternalHeight, 0);
  n++;
  XtSetArg (args[n], XtNinternalWidth, 0);
  n++;
  button = XtCreateManagedWidget (name, toggleWidgetClass, parent, args, n);

  XtAddCallback (button, XtNcallback, changecheck, NULL);

  if (ftccallback != NULL)
    XtAddCallback (button, XtNcallback, ftccallback, param);
  return button;
}

Widget
make_alone_popup (_toplevel, name)
     Widget _toplevel;
     char *name;
{
  Arg args[20];
  register int n;
  Widget popupshell, popupmenu, entry, menu;
  XtTranslations trans_table;
  int i;
  n = 0;

  XtSetArg (args[n], XtNinput, True);
  n++;

  popupmenu = XtCreatePopupShell
    ("popupMenu", simpleMenuWidgetClass, _toplevel, args, n);

  for (i = 0; i < XtNumber (popupMenuEntries); i++)
    {
      entry = XtCreateManagedWidget (popupMenuEntries[i].name,
				     smeBSBObjectClass, popupmenu,
				     NULL, (Cardinal) 0);
      XtAddCallback (entry, XtNcallback, popupMenuEntries[i].function, NULL);
    }

  return popupmenu;
}

void
changesht (w, closure, call_data)
     Widget w;
     XtPointer closure, call_data;
{
  int s;
  for (s = 0; s < 20; s++)
    if (w == selectsht[s])
      cb_changesht (s);

}

#include "wkb.h"

static Widget thumbpar;

void
build_thumb (parent)
     Widget parent;
{
  int i;
  int width;
  int pos = 0;
  char *name;
  char buf[256];

  thumbpar = parent;
  for (i = 0; i < 20; i++)
    selectsht[i] = NULL;

  return;

  if (ActivWkb == NULL)
    return;

  for (i = 0; i < ActivWkb->nbwks; i++)
    {

      name = ActivWkb->Wkss[i]->Name;
      width = gettextw (name, strlen (name), 1, 0, 2);
      w_n = 0;
      w_rel (NULL, pos, 00);
      w_dim (width + 15, 20);
      w_bord (0);
      pos += width + 15;
      XtSetArg (w_args[w_n], XtNbackground, 0);
      w_n++;
      XtSetArg (w_args[w_n], XtNforeground, 1);
      w_n++;

      if (i == 0)
	XtSetArg (w_args[w_n], XtNstate, True);
      else
	XtSetArg (w_args[w_n], XtNstate, False);
      w_n++;
      if (i > 0)
	{
	  XtSetArg (w_args[w_n], XtNradioGroup, selectsht[0]);
	  w_n++;
	}
      selectsht[i] = XtCreateManagedWidget (name, toggleWidgetClass, parent, w_args, w_n);
      XtAddCallback (selectsht[i], XtNcallback, changesht, NULL);

    }

}

rebuild_thumb ()
{
  int i;
  int width;
  int nb;
  int pos = 0;
  char *name;
  char buf[256];

  if (ActivWkb == NULL)
    return;

  for (i = 0; i < 20; i++)
    if (selectsht[i] != NULL)
      {
	XtDestroyWidget (selectsht[i]);
	selectsht[i] = NULL;
      }
  nb = ActivWkb->nbwks;
  if (nb > 19)
    nb = 19;
  for (i = 0; i < nb; i++)
    {
      name = ActivWkb->Wkss[i]->Name;
      if (name == NULL)
	name = "?";
      width = gettextw (name, strlen (name), 1, 0, 2) + 15;
      w_n = 0;
      w_rel (NULL, pos, 00);
      w_dim (width, 20);
      w_bord (0);
      XtSetArg (w_args[w_n], XtNbackground, 0);
      w_n++;
      XtSetArg (w_args[w_n], XtNforeground, 1);
      w_n++;

      pos += width;
      if (ActivWkb->Wkss[i] == ActivWkb->activwks)
	XtSetArg (w_args[w_n], XtNstate, True);
      else
	XtSetArg (w_args[w_n], XtNstate, False);
      w_n++;
      if (i > 0)
	{
	  XtSetArg (w_args[w_n], XtNradioGroup, selectsht[0]);
	  w_n++;
	}
      selectsht[i] = XtCreateManagedWidget (name, toggleWidgetClass, thumbpar, w_args
					    ,w_n);
      XtAddCallback (selectsht[i], XtNcallback, changesht, NULL);

    }

}
