/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "wkb.h"
#include "style.h"

#include "stdio.h"

static int nwkb = 1;

Wkb *ActivWkb;

Wkb *
newwkb (filename)
     char *filename;
{
  int i;
  Wkb *wkb;
  wkb = (Wkb *) absmalloc (sizeof (Wkb), "newwkb:wkb ");
  if (wkb == NULL)
    printf ("not allocated!");

  wkb->nbwks = 3;

  wkb->Wkss =
    (Wks **) absmalloc (sizeof (Wks *) * 3, "newwkb:wkb->Wkss ");

  for (i = 0; i < 3; i++)
    {
      wkb->Wkss[i] = newwks ();
      wkb->Wkss[i]->Name[0] = 'A' + i;
      wkb->Wkss[i]->Name[1] = '\0';
    }

  wkb->activwks = wkb->Wkss[0];

  wkb->style = newstyle (STY_DEF_CELL);
  wkb_setnormalstyle (wkb);

  wkb->macrofile = (Macrofile *) newmacrofile ();

  wkb->name = (char *) absmalloc (sizeof (char) * (strlen (filename) + 1), "newwkb:wkb->name ");
  strcpy (wkb->name, filename);

  wkb->objn = nwkb++;

  return wkb;
}

Wkb *
newwkb1 (nb)
     int nb;
{
  int i;
  Wkb *wkb;
  wkb = (Wkb *) absmalloc (sizeof (Wkb), "newwkb1:wkb ");

  wkb->nbwks = nb;

  if (nb > 0)
    {
      wkb->Wkss =
	(Wks **) absmalloc (sizeof (Wks *) * nb, "newwkb1:wkb->Wkss ");

    }
  if (wkb == NULL)
    printf ("not allocated!");

  wkb->style = newstyle (STY_CELL);
  wkb_setnormalstyle (wkb);
  wkb->macrofile = (Macrofile *) newmacrofile ();

  wkb->objn = nwkb++;

  return wkb;
}

freewkb (wkb)
     Wkb *wkb;
{
  int i;
  if (wkb == NULL)
    return;

  for (i = 0; i < wkb->nbwks; i++)
    {
      freewks (wkb->Wkss[i]);
    }
  absfree (wkb->Wkss, "freewkb:wkb->Wkss ");
  freecode (wkb->macrofile);
  absfree (wkb->macrofile, "freewkb:wkb->macrofile ");
  freestyle (wkb->style);
  absfree (wkb->name, "freewkb:wkb->name ");
  absfree (wkb, "freewkb:wkb ");
}

int
wkb_setnormalstyle (wkb)
     Wkb *wkb;
{
  setstyle (wkb->style, STY_FG, 6);
  setstyle (wkb->style, STY_BG, 45);
  setstyle (wkb->style, STY_BORDER_TOP, 0);
  setstyle (wkb->style, STY_BORDER_BOT, 0);
  setstyle (wkb->style, STY_BORDER_RIG, 0);
  setstyle (wkb->style, STY_BORDER_LEF, 0);
  setstyle (wkb->style, STY_FORMAT, 1);
  setstyle (wkb->style, STY_DECIMAL, 0);
  setstyle (wkb->style, STY_JUSTIF, 2);
  setstyle (wkb->style, STY_FONT, 2);
  setstyle (wkb->style, STY_FONTW, 0);
  setstyle (wkb->style, STY_FONTS, 2);
}

Wkb *
wkb_o (filename)
     char *filename;
{
}

int
wkb_s (wkb, filename)
     Wkb *wkb;
     char *filename;
{
  int i;
  int activ;
  FILE *fp = fopen (filename, "w");

  if (strcmp (filename, wkb->name))
    {
      absfree (wkb->name, "wkb_s:wkb->name ");
      wkb->name = (char *) absmalloc (sizeof (char) * (strlen (filename) + 1), "wkb_s:wkb->name ");
      strcpy (wkb->name, filename);
    }

  fprintf (fp, "Sub main ()\n");
  for (i = 0; i < wkb->nbwks; i++)
    {
      if (i > 2)
	fprintf (fp, "Worksheets.Add\n");
      fprintf (fp, "Worksheets(%d).Name = \"%s\"\n", i + 1, wkb->Wkss[i]->Name);

    }

  for (i = 0; i < wkb->nbwks; i++)
    {
      fprintf (fp, "Worksheets(%d).Activate\n", i + 1);
      if (wkb->Wkss[i] == wkb->activwks)
	activ = i;

      wks_w (wkb->Wkss[i], fp);

    }

  fprintf (fp, "Worksheets(%d).Activate\n", activ + 1);
  fprintf (fp, "End Sub\n");

  macrosave (wkb->macrofile, fp);
  fclose (fp);

}

ActivateWkb (wkb)
     Wkb *wkb;
{
  ActivWkb = wkb;
  rebuild_thumb ();

  ActivateWks (ActivWkb->activwks);
  rstsel (0, 0, 0);

}

Wks *
wkb_gsht (wkb, name)
     Wkb *wkb;
     char *name;
{
  int i;

  if (name == NULL)
    return;
  for (i = 0; i < wkb->nbwks; i++)
    {
      if (strcasecmp (wkb->Wkss[i]->Name, name) == 0)
	{
	  return wkb->Wkss[i];
	}
    }

  return NULL;

}

Wks *
wkb_gsht2 (wkb, num)
     Wkb *wkb;
     int num;
{
  int i;

  num--;
  if (num >= 0 && num < wkb->nbwks)
    return wkb->Wkss[num];
  return wkb->Wkss[0];

}

Wks *
wkb_as (wkb, name)
     Wkb *wkb;
     char *name;
{
  int i;
  i = wkb->nbwks;
  wkb->nbwks++;

  wkb->Wkss =
    (Wks **) absrealloc (wkb->Wkss,
			 sizeof (Wks *) * (i + 1), "wkb_as:wkb->Wkss ");

  wkb->Wkss[i] = newwks ();
  if (name == NULL)
    {
      wkb->Wkss[i]->Name[0] = 'A' + i;
      wkb->Wkss[i]->Name[1] = '\0';
    }
  else
    strcpy (wkb->Wkss[i]->Name, name);
  wkb->activwks = wkb->Wkss[i];
  if (wkb == ActivWkb)
    ActivateWks (ActivWkb->activwks);

  rstsel (0, 0, 0);
  xrepnt ();
  rebuild_thumb ();

}

Wks *
wkb_ds (wkb, num)
     Wkb *wkb;
     int num;
{
  int i, j;
  num--;
  i = wkb->nbwks;

  freewks (num);

  for (j = num; j < wkb->nbwks; j++)
    wkb->Wkss[j] = wkb->Wkss[j + 1];

  wkb->nbwks--;

  (Wks **) absrealloc (wkb->Wkss,
		       sizeof (Wks *) * wkb->nbwks, "wkb_ds:wkb->Wkss ");

  rebuild_thumb ();
}
