/********************************************************************
This file is part of abs, a spreadsheet with graphical user interface

Copyright (C) 1998  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "wks.h"
#include "wkb.h"

Wks *ActivWks;

static int nwks = 1;

Wks *
newwks ()
{
  int i, j;
  Wks *wks;
  wks = (Wks *) absmalloc (sizeof (Wks), "newwks:wks ");

  wks->nlin = NBLIN;
  wks->ncol = NBCOL;
  wks->cw[0] = -WCOL;
  wks->lh[0] = -HLIN;
  for (i = 1; i <= NBCOL; i++)
    wks->cw[i] = wks->cw[i - 1] + WCOL;
  for (i = 1; i <= NBLIN; i++)
    wks->lh[i] = wks->lh[i - 1] + HLIN;

  wks->ll = 1;
  wks->cc = 1;
  wks->nbcl = 0;
  wks->nbchrt = 0;
  wks->nbdrwing = 0;
  wks->zm = 100;

  wks->Chrts = NULL;
  wks->Drwings = NULL;

  wks->selection = newselection ();

  wks->defcl = newcl (-1, -1);

  ActivCl = wks->defcl;
  ActivChrt = NULL;
  ActivDrwing = NULL;

  if (wks == NULL)
    printf ("not allocated!");

  wks->objn = nwks++;

  wks->matrix = NULL;
  wks->mrow = -1;
  wks->mcol = NULL;

  return wks;
}

Wks *
newwks1 (nbcl, nbchrt, nbdrwing)
     int nbcl, nbchrt, nbdrwing;
{
  int i, j;
  Wks *wks;
  wks = (Wks *) absmalloc (sizeof (Wks), "newwks1:wks ");

  wks->nlin = NBLIN;
  wks->ncol = NBCOL;
  wks->cw[0] = -WCOL;
  wks->lh[0] = -HLIN;
  for (i = 1; i <= NBCOL; i++)
    wks->cw[i] = wks->cw[i - 1] + WCOL;
  for (i = 1; i <= NBLIN; i++)
    wks->lh[i] = wks->lh[i - 1] + HLIN;

  wks->ll = 1;
  wks->cc = 1;
  wks->nbcl = nbcl;
  wks->nbchrt = nbchrt;
  wks->nbdrwing = nbdrwing;
  wks->zm = 100;

  wks->Chrts = NULL;
  wks->Drwings = NULL;

  wks->defcl = newcl (-1, -1);
  ActivCl = wks->defcl;

  ActivChrt = wks->Chrts[0];
  ActivDrwing = wks->Drwings[0];
  wks->selection = newselection ();

  if (wks == NULL)
    printf ("not allocated!");

  wks->objn = nwks++;

  wks->matrix = NULL;
  wks->mrow = -1;
  wks->mcol = NULL;
  return wks;
}

freewks (wks)
     Wks *wks;
{
  int i, j;

  if (wks == NULL)
    return;

  if (ActivWks == wks)
    {
      for (i = 0; i < ActivWks->nbchrt; i++)
	deletechrtpixmap (ActivWks->Chrts[i]);
      ActivWks == NULL;
    }

  for (i = 0; i < wks->nbchrt; i++)
    freechrt (wks->Chrts[i]);
  for (i = 0; i < wks->nbdrwing; i++)
    freedrwing (wks->Drwings[i]);

  freeselection (wks->selection);

  for (i = 1; i <= wks->mrow; i++)
    {
      for (j = 1; j <= wks->mcol[i]; j++)
	freecl (wks->matrix[i][j]);
      absfree (wks->matrix[i], "freewks:wks->matrix[i] ");
    }
  absfree (wks->matrix, "freewks:wks->matrix ");
  absfree (wks->mcol, "freewks:wks->mcol ");

  freecl (wks->defcl);

  absfree (wks, "freewks:wks ");
}

int
wks_w (wks, fp)
     Wks *wks;
     FILE *fp;
{
  int i, j;
  char buf[10];
  Cl *cl;
  Cl *nextr;
  double v;

  for (i = 0; i < wks->ncol - 1; i++)
    if ((wks->cw[i + 1] - wks->cw[i]) != WCOL)
      {
	coltoalpha (buf, i);
	v = (wks->cw[i + 1] - wks->cw[i]) / 7.;
	fprintf (fp, "Columns(\"%s\").ColumnWidth = %lf\n", buf, v);
      }

  for (i = 0; i < wks->nlin - 1; i++)
    if ((wks->lh[i + 1] - wks->lh[i]) != HLIN)
      {
	fprintf (fp, "Rows(\"%d\").RowHeight = %d\n", i, wks->lh[i + 1] - wks->lh[i]);
      }

  for (i = 1; i <= wks->mrow; i++)
    for (j = 1; j <= wks->mcol[i]; j++)
      {
	if (wks->matrix[i][j] != NULL)
	  cl_w (wks->matrix[i][j], fp);
      }

  for (i = 0; i < wks->nbchrt; i++)
    chrt_w (wks->Chrts[i], fp);
  for (i = 0; i < wks->nbdrwing; i++)
    drwing_w (wks->Drwings[i], fp);

}

int
wks_update (wks)
     Wks *wks;
{
  int i;

  for (i = 0; i < wks->nbchrt; i++)
    {
      ActivChrt = wks->Chrts[i];
      plot (0);
    }

}

int
wks_f (wks, x, y)
     Wks *wks;
     int x, y;
{

}

Cl *
wks_gcl (wks, r, c, new)
     Wks *wks;
     int r, c, new;
{
  int i, j, found;
  Cl *cl;
  Cl *cl1;
  Cl *cl2;
  Cl *ret;
  cl = wks->defcl;

  if (wks == NULL)
    return NULL;

  if (r < 0 || r > NBLIN)
    return wks->defcl;
  if (c < 0 || c > NBCOL)
    return wks->defcl;

  if (new < 0)
    {
      ret = NULL, new = 0;
    }
  else
    ret = wks->defcl;

  if (r < 0 || c < 0 || r > wks->nlin || c > wks->ncol)
    return ret;

  if (r > wks->mrow)
    {
      if (!new)
	return ret;

      wks->matrix = (Cl ***) absrealloc (wks->matrix, sizeof (Cl **) * (r + 1), "wks_gcl:wks->matrix ");
      wks->mcol = (int *) absrealloc (wks->mcol, sizeof (int) * (r + 1), "wks_gcl:wks->mcol ");

      for (i = wks->mrow + 1; i <= r; i++)
	{
	  wks->mcol[i] = 0;
	  wks->matrix[i] = NULL;
	}

      wks->matrix[r] =
	(Cl **) absrealloc (wks->matrix[r], sizeof (Cl *) * (c + 1), "wks_gcl:wks->matrix[r] ");
      for (i = 0; i <= c; i++)
	wks->matrix[r][i] = NULL;

      wks->mrow = r;
      wks->mcol[r] = c;

      wks->matrix[r][c] = newcl (r, c);
      return wks->matrix[r][c];
    }
  else if (c > wks->mcol[r])
    {
      if (!new)
	return ret;

      wks->matrix[r] =
	(Cl **) absrealloc (wks->matrix[r], sizeof (Cl *) * (c + 1), "wks_gcl:wks->matrix[r] ");
      for (i = wks->mcol[r] + 1; i <= c; i++)
	wks->matrix[r][i] = NULL;
      wks->mcol[r] = c;
      wks->matrix[r][c] = newcl (r, c);
      return wks->matrix[r][c];
    }

  else if (wks->matrix[r][c] == NULL)
    {
      if (!new)
	return ret;
      wks->matrix[r][c] = newcl (r, c);
      return wks->matrix[r][c];
    }

  return wks->matrix[r][c];

}

wks_delcl (wks, r, c)
     Wks *wks;
     int r, c;
{
  Cl *cl = wks_gcl (wks, r, c, -1);
  if (cl == NULL)
    return;

  if (cl->familly->npar > 0)
    cl_isnew (cl);

  if (cl->familly->nchi > 0)
    cl_setform (cl, "");
  else
    {
      wks->matrix[r][c] = NULL;
      freecl (cl);
    }

}

Chrt *
wks_nchrt (wks)
     Wks *wks;
{
  wks->Chrts = (Chrt **) absrealloc (wks->Chrts, sizeof (Chrt *) * (wks->nbchrt + 1), "wks_nchrt:wks->Chrts ");
  wks->Chrts[wks->nbchrt] = newchrt ();
  wks->activchrt = wks->Chrts[wks->nbchrt];
  wks->nbchrt++;
  return wks->Chrts[wks->nbchrt - 1];
}

Drwing *
wks_ndrwing (wks)
     Wks *wks;
{
  wks->Drwings = (Drwing **) absrealloc (wks->Drwings, sizeof (Drwing *) * (wks->nbdrwing + 1), "wks_ndrwing:wks->Drwings ");
  wks->Drwings[wks->nbdrwing] = newdrwing ();
  wks->activdrwing = wks->Drwings[wks->nbdrwing];
  wks->nbdrwing++;
  return wks->Drwings[wks->nbdrwing - 1];
}

ActivateWks (wks)
     Wks *wks;
{
  int i;

  if (wks == NULL)
    return;

  if (ActivWks != wks && ActivWks != NULL)
    {
      for (i = 0; i < ActivWks->nbchrt; i++)
	deletechrtpixmap (ActivWks->Chrts[i]);
    }

  ActivWks = wks;
  ActivWkb->activwks = wks;

  ActivCl = wks->activcl;
  ActivChrt = wks->activchrt;
  ActivDrwing = wks->activdrwing;
  ActivSelection = wks->selection;

  for (i = 0; i < wks->nbchrt; i++)
    {
      createchrtpixmap (wks->Chrts[i]);
      newplot (i, wks->Chrts[i]);
    }

  cb_setzm (wks->zm);
  setzmlabel (wks->zm);
  setscrollvlimit (1, NBLIN);
  setscrollhlimit (1, NBCOL);
  setscrollv ((double) wks->ll);
  setscrollh ((double) wks->cc);

}

Cht *
wks_gcht (wks, n)
     Wks *wks;
     int n;
{
  if (n > 0 && n < wks->nbchrt)
    return wks->Chrts[n];
  return NULL;
}

int
wks_addcht (wks, args, n)
     Wks *wks;
     int *args;
     int n;
{
  wks->Chrts = (Chrt **) absrealloc (wks->Chrts, sizeof (Chrt *) * (wks->nbchrt + 1), "wks_nchrt:wks->Chrts ");
  wks->Chrts[wks->nbchrt] = newchrt ();
  wks->activchrt = wks->Chrts[wks->nbchrt];
  wks->nbchrt++;

  if (n == 4)
    {
      wks->activchrt->x = args[0];
      wks->activchrt->y = args[1];
      wks->activchrt->w = args[2];
      wks->activchrt->h = args[3];
    }
  else
    {
      wks->activchrt->x = 30;
      wks->activchrt->y = 30;
      wks->activchrt->w = 80;
      wks->activchrt->h = 50;
    }

  createchrtpixmap (wks->activchrt);
  ActivChrt = wks->activchrt;

}

int
wks_addshape (wks, args, n)
     Wks *wks;
     int *args;
     int n;
{
  wks->Drwings = (Drwing **) absrealloc (wks->Drwings, sizeof (Drwing *) * (wks->nbdrwing + 1)
					 ,"wks_addshape:wks->Drwings ");
  wks->Drwings[wks->nbdrwing] = (Drwing *) newdrwing ();
  wks->activdrwing = wks->Drwings[wks->nbdrwing];
  wks->nbdrwing++;

  if (n == 5)
    {
      wks->activdrwing->type = args[0];
      wks->activdrwing->x1 = args[1];
      wks->activdrwing->y1 = args[2];
      wks->activdrwing->x2 = args[3];
      wks->activdrwing->y2 = args[4];
    }
  if (n == 4)
    {
      wks->activdrwing->type = 0;
      wks->activdrwing->x1 = args[0];
      wks->activdrwing->y1 = args[1];
      wks->activdrwing->x2 = args[2];
      wks->activdrwing->y2 = args[3];
    }

  ActivDrwing = wks->activdrwing;

}

int
wks_setcw (wks, col, w)
     Wks *wks;
     int col;
     double w;
{
  int i;
  int dw;
  w = w * 7.2;
  dw = w - (wks->cw[col + 1] - wks->cw[col]);
  for (i = col + 1; i < wks->ncol; i++)
    wks->cw[i] += dw;
}

int
wks_setrh (wks, lin, h)
     Wks *wks;
     int lin;
     double h;
{
  int i;
  int dh = h - (wks->lh[lin + 1] - wks->lh[lin]);
  for (i = lin + 1; i < wks->nlin; i++)
    wks->lh[i] += dh;
}

int
wks_setname (wks, name)
     Wks *wks;
     char *name;
{
  if (name == NULL)
    return;
  if (strlen (name) > 63)
    return;
  sprintf (wks->Name, "%s", name);
}
